@@ -41,12 +41,14 @@ public class ClusterEventMetadataGenerator implements EventMetadataGenerator{
 
   public Map<String, String> getMetadata(JobContext jobContext, EventName eventName) {
     List<TaskState> taskStates = jobContext.getJobState().getTaskStates();
+    String taskException = EventMetadataUtils.getTaskFailureExceptions(taskStates);
+    String jobException = EventMetadataUtils.getJobFailureExceptions(jobContext.getJobState());
 
     switch (eventName) {
       case JOB_COMPLETE:
         return ImmutableMap.of(PROCESSED_COUNT_KEY, Long.toString(EventMetadataUtils.getProcessedCount(taskStates)));
       case JOB_FAILED:
-        return ImmutableMap.of(MESSAGE_KEY, EventMetadataUtils.getTaskFailureExceptions(taskStates));
+        return ImmutableMap.of(MESSAGE_KEY, taskException.length() != 0 ? taskException : jobException);
       default:
         break;
     }
