@@ -18,8 +18,6 @@
 package org.apache.gobblin.cluster;
 
 import java.io.IOException;
-import java.lang.reflect.InvocationTargetException;
-import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
@@ -41,12 +39,14 @@ import org.apache.helix.Criteria;
 import org.apache.helix.HelixManager;
 import org.apache.helix.InstanceType;
 import org.apache.helix.messaging.handling.MessageHandlerFactory;
+import org.apache.helix.model.ClusterConfig;
 import org.apache.helix.model.Message;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
+import com.google.common.base.Splitter;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.eventbus.EventBus;
@@ -56,6 +56,7 @@ import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 import com.typesafe.config.ConfigValueFactory;
 
+import lombok.AccessLevel;
 import lombok.Getter;
 import lombok.Setter;
 import lombok.extern.slf4j.Slf4j;
@@ -108,10 +109,12 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
   protected ServiceBasedAppLauncher applicationLauncher;
 
   // An EventBus used for communications between services running in the ApplicationMaster
+  @Getter(AccessLevel.PUBLIC)
   protected final EventBus eventBus = new EventBus(GobblinClusterManager.class.getSimpleName());
 
   protected final Path appWorkDir;
 
+  @Getter
   protected final FileSystem fs;
 
   protected final String applicationId;
@@ -133,23 +136,32 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
   @Getter
   private JobConfigurationManager jobConfigurationManager;
 
-  private final String clusterName;
-  private final Config config;
+  protected final String clusterName;
+  @Getter
+  protected final Config config;
 
-  public GobblinClusterManager(String clusterName, String applicationId, Config config,
+  public GobblinClusterManager(String clusterName, String applicationId, Config sysConfig,
       Optional<Path> appWorkDirOptional) throws Exception {
+    // Set system properties passed in via application config. As an example, Helix uses System#getProperty() for ZK configuration
+    // overrides such as sessionTimeout. In this case, the overrides specified
+    // in the application configuration have to be extracted and set before initializing HelixManager.
+    GobblinClusterUtils.setSystemProperties(sysConfig);
+
+    //Add dynamic config
+    this.config = GobblinClusterUtils.addDynamicConfig(sysConfig);
+
     this.clusterName = clusterName;
-    this.config = config;
-    this.isStandaloneMode = ConfigUtils.getBoolean(config, GobblinClusterConfigurationKeys.STANDALONE_CLUSTER_MODE_KEY,
+    this.isStandaloneMode = ConfigUtils.getBoolean(this.config, GobblinClusterConfigurationKeys.STANDALONE_CLUSTER_MODE_KEY,
         GobblinClusterConfigurationKeys.DEFAULT_STANDALONE_CLUSTER_MODE);
 
     this.applicationId = applicationId;
 
     initializeHelixManager();
 
-    this.fs = buildFileSystem(config);
+    this.fs = GobblinClusterUtils.buildFileSystem(this.config, new Configuration());
     this.appWorkDir = appWorkDirOptional.isPresent() ? appWorkDirOptional.get()
-        : GobblinClusterUtils.getAppWorkDirPathFromConfig(config, this.fs, clusterName, applicationId);
+        : GobblinClusterUtils.getAppWorkDirPathFromConfig(this.config, this.fs, clusterName, applicationId);
+    LOGGER.info("Configured GobblinClusterManager work dir to: {}", this.appWorkDir);
 
     initializeAppLauncherAndServices();
   }
@@ -188,6 +200,11 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
     this.applicationLauncher.addService(this.jobScheduler);
     this.jobConfigurationManager = buildJobConfigurationManager(config);
     this.applicationLauncher.addService(this.jobConfigurationManager);
+
+    if (ConfigUtils.getBoolean(this.config, GobblinClusterConfigurationKeys.CONTAINER_HEALTH_METRICS_SERVICE_ENABLED,
+        GobblinClusterConfigurationKeys.DEFAULT_CONTAINER_HEALTH_METRICS_SERVICE_ENABLED)) {
+      this.applicationLauncher.addService(new ContainerHealthMetricsService(config));
+    }
   }
 
   /**
@@ -218,6 +235,41 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
     }
   }
 
+  /**
+   * Configure Helix quota-based task scheduling.
+   * This config controls the number of tasks that are concurrently assigned to a single Helix instance.
+   * Reference: https://helix.apache.org/0.9.1-docs/quota_scheduling.html
+   */
+  @VisibleForTesting
+  void configureHelixQuotaBasedTaskScheduling() {
+    // set up the cluster quota config
+    List<String> quotaConfigList = ConfigUtils.getStringList(this.config,
+        GobblinClusterConfigurationKeys.HELIX_TASK_QUOTA_CONFIG_KEY);
+
+    if (quotaConfigList.isEmpty()) {
+      return;
+    }
+
+    // retrieve the cluster config for updating
+    ClusterConfig clusterConfig = this.multiManager.getJobClusterHelixManager().getConfigAccessor()
+        .getClusterConfig(this.clusterName);
+    clusterConfig.resetTaskQuotaRatioMap();
+
+    for (String entry : quotaConfigList) {
+      List<String> quotaConfig = Splitter.on(":").limit(2).trimResults().omitEmptyStrings().splitToList(entry);
+
+      if (quotaConfig.size() < 2) {
+        throw new IllegalArgumentException(
+            "Quota configurations must be of the form <key1>:<value1>,<key2>:<value2>,...");
+      }
+
+      clusterConfig.setTaskQuotaRatio(quotaConfig.get(0), Integer.parseInt(quotaConfig.get(1)));
+    }
+
+    this.multiManager.getJobClusterHelixManager().getConfigAccessor()
+        .setClusterConfig(this.clusterName, clusterConfig); // Set the new ClusterConfig
+  }
+
   /**
    * Start the Gobblin Cluster Manager.
    */
@@ -228,6 +280,14 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
     this.eventBus.register(this);
     this.multiManager.connect();
 
+    // Standalone mode registers a handler to clean up on manager leadership change, so only clean up for non-standalone
+    // mode, such as YARN mode
+    if (!this.isStandaloneMode) {
+      this.multiManager.cleanUpJobs();
+    }
+
+    configureHelixQuotaBasedTaskScheduling();
+
     if (this.isStandaloneMode) {
       // standalone mode starts non-daemon threads later, so need to have this thread to keep process up
       this.idleProcessThread = new Thread(new Runnable() {
@@ -300,23 +360,19 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
             .put(GobblinClusterMetricTagNames.APPLICATION_ID, applicationId).build());
   }
 
-  /**
-   * Build the {@link FileSystem} for the Application Master.
-   */
-  private FileSystem buildFileSystem(Config config) throws IOException {
-    return config.hasPath(ConfigurationKeys.FS_URI_KEY) ? FileSystem
-        .get(URI.create(config.getString(ConfigurationKeys.FS_URI_KEY)), new Configuration())
-        : FileSystem.get(new Configuration());
-  }
-
   /**
    * Build the {@link GobblinHelixJobScheduler} for the Application Master.
    */
-  private GobblinHelixJobScheduler buildGobblinHelixJobScheduler(Config config, Path appWorkDir,
+  private GobblinHelixJobScheduler buildGobblinHelixJobScheduler(Config sysConfig, Path appWorkDir,
       List<? extends Tag<?>> metadataTags, SchedulerService schedulerService) throws Exception {
-    Properties properties = ConfigUtils.configToProperties(config);
-    return new GobblinHelixJobScheduler(properties, this.multiManager.getJobClusterHelixManager(), this.eventBus, appWorkDir, metadataTags,
-        schedulerService, this.jobCatalog);
+    return new GobblinHelixJobScheduler(sysConfig,
+        this.multiManager.getJobClusterHelixManager(),
+        this.multiManager.getTaskDriverHelixManager(),
+        this.eventBus,
+        appWorkDir,
+        metadataTags,
+        schedulerService,
+        this.jobCatalog);
   }
 
   /**
@@ -328,16 +384,15 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
 
   private JobConfigurationManager create(Config config) {
     try {
+      List<Object> argumentList = (this.jobCatalog != null)? ImmutableList.of(this.eventBus, config, this.jobCatalog, this.fs) :
+          ImmutableList.of(this.eventBus, config, this.fs);
       if (config.hasPath(GobblinClusterConfigurationKeys.JOB_CONFIGURATION_MANAGER_KEY)) {
-        return (JobConfigurationManager) GobblinConstructorUtils.invokeFirstConstructor(Class.forName(
-            config.getString(GobblinClusterConfigurationKeys.JOB_CONFIGURATION_MANAGER_KEY)),
-            ImmutableList.<Object>of(this.eventBus, config, this.jobCatalog),
-            ImmutableList.<Object>of(this.eventBus, config));
+        return (JobConfigurationManager) GobblinConstructorUtils.invokeLongestConstructor(Class.forName(
+            config.getString(GobblinClusterConfigurationKeys.JOB_CONFIGURATION_MANAGER_KEY)), argumentList.toArray(new Object[argumentList.size()]));
       } else {
         return new JobConfigurationManager(this.eventBus, config);
       }
-    } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException |
-        ClassNotFoundException e) {
+    } catch (ReflectiveOperationException e) {
       throw new RuntimeException(e);
     }
   }
@@ -348,12 +403,6 @@ public class GobblinClusterManager implements ApplicationLauncher, StandardMetri
     stop();
   }
 
-  @VisibleForTesting
-  EventBus getEventBus() {
-    return this.eventBus;
-  }
-
-
   /**
    * Creates and returns a {@link MessageHandlerFactory} for handling of Helix
    * {@link org.apache.helix.model.Message.MessageType#USER_DEFINE_MSG}s.
