@@ -18,10 +18,12 @@
 package org.apache.gobblin.cluster;
 
 import org.apache.gobblin.annotation.Alpha;
+import org.apache.gobblin.metrics.event.TimingEvent;
 
 
 /**
  * A central place for constants of {@link org.apache.gobblin.metrics.MetricContext} tag names for a Gobblin cluster.
+ * Some shared constants have been moved to {@link TimingEvent.FlowEventConstants}.
  *
  * @author Yinan Li
  */
@@ -32,11 +34,4 @@ public class GobblinClusterMetricTagNames {
   public static final String APPLICATION_ID = "application.id";
   public static final String HELIX_INSTANCE_NAME = "helix.instance.name";
   public static final String TASK_RUNNER_ID = "task.runner.id";
-
-  public static final String FLOW_GROUP = "flowGroup";
-  public static final String FLOW_NAME = "flowName";
-  public static final String FLOW_EXECUTION_ID = "flowExecutionId";
-  public static final String JOB_GROUP = "jobGroup";
-  public static final String JOB_NAME = "jobName";
-  public static final String JOB_EXECUTION_ID = "jobExecutionId";
 }
