@@ -50,6 +50,8 @@ import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.metastore.StateStore;
 import org.apache.gobblin.metrics.Tag;
+import org.apache.gobblin.metrics.event.CountEventBuilder;
+import org.apache.gobblin.metrics.event.JobEvent;
 import org.apache.gobblin.metrics.event.TimingEvent;
 import org.apache.gobblin.rest.LauncherTypeEnum;
 import org.apache.gobblin.runtime.AbstractJobLauncher;
@@ -68,6 +70,7 @@ import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.util.Id;
 import org.apache.gobblin.util.JobLauncherUtils;
 import org.apache.gobblin.util.ParallelRunner;
+import org.apache.gobblin.util.PropertiesUtils;
 import org.apache.gobblin.util.SerializationUtils;
 
 
@@ -89,8 +92,8 @@ import org.apache.gobblin.util.SerializationUtils;
  * </p>
  *
  * <p>
- *   This class runs in the {@link GobblinClusterManager}. The actual task execution happens in the in the
- *   {@link GobblinTaskRunner}.
+ *   This class is instantiated by the {@link GobblinHelixJobScheduler} on every job submission to launch the Gobblin job.
+ *   The actual task execution happens in the {@link GobblinTaskRunner}, usually in a different process.
  * </p>
  *
  * @author Yinan Li
@@ -117,22 +120,23 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
   private final int stateSerDeRunnerThreads;
 
   private final TaskStateCollectorService taskStateCollectorService;
-
+  private final Optional<GobblinHelixMetrics> helixMetrics;
   private volatile boolean jobSubmitted = false;
   private final ConcurrentHashMap<String, Boolean> runningMap;
   private final StateStores stateStores;
   private final Config jobConfig;
   private final long workFlowExpiryTimeSeconds;
+  private final long helixJobStopTimeoutSeconds;
 
   public GobblinHelixJobLauncher (Properties jobProps,
                                   final HelixManager helixManager,
                                   Path appWorkDir,
                                   List<? extends Tag<?>> metadataTags,
-                                  ConcurrentHashMap<String, Boolean> runningMap) throws Exception {
+                                  ConcurrentHashMap<String, Boolean> runningMap,
+                                  Optional<GobblinHelixMetrics> helixMetrics) throws Exception {
 
     super(jobProps, addAdditionalMetadataTags(jobProps, metadataTags));
     LOGGER.debug("GobblinHelixJobLauncher: jobProps {}, appWorkDir {}", jobProps, appWorkDir);
-
     this.helixManager = helixManager;
     this.helixTaskDriver = new TaskDriver(this.helixManager);
     this.runningMap = runningMap;
@@ -146,17 +150,20 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
 
     this.stateSerDeRunnerThreads = Integer.parseInt(jobProps.getProperty(ParallelRunner.PARALLEL_RUNNER_THREADS_KEY,
         Integer.toString(ParallelRunner.DEFAULT_PARALLEL_RUNNER_THREADS)));
-
     jobConfig = ConfigUtils.propertiesToConfig(jobProps);
 
     this.workFlowExpiryTimeSeconds = ConfigUtils.getLong(jobConfig,
         GobblinClusterConfigurationKeys.HELIX_WORKFLOW_EXPIRY_TIME_SECONDS,
         GobblinClusterConfigurationKeys.DEFAULT_HELIX_WORKFLOW_EXPIRY_TIME_SECONDS);
 
+    this.helixJobStopTimeoutSeconds = ConfigUtils.getLong(jobConfig,
+        GobblinClusterConfigurationKeys.HELIX_JOB_STOP_TIMEOUT_SECONDS,
+        GobblinClusterConfigurationKeys.DEFAULT_HELIX_JOB_STOP_TIMEOUT_SECONDS);
+
     Config stateStoreJobConfig = ConfigUtils.propertiesToConfig(jobProps)
         .withValue(ConfigurationKeys.STATE_STORE_FS_URI_KEY, ConfigValueFactory.fromAnyRef(
             new URI(appWorkDir.toUri().getScheme(), null, appWorkDir.toUri().getHost(),
-                appWorkDir.toUri().getPort(), null, null, null).toString()));
+                appWorkDir.toUri().getPort(), "/", null, null).toString()));
 
     this.stateStores = new StateStores(stateStoreJobConfig, appWorkDir,
         GobblinClusterConfigurationKeys.OUTPUT_TASK_STATE_DIR_NAME, appWorkDir,
@@ -172,6 +179,7 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
         this.stateStores.getTaskStateStore(),
         this.outputTaskStateDir);
 
+    this.helixMetrics = helixMetrics;
     startCancellationExecutor();
   }
 
@@ -184,9 +192,17 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
     }
   }
 
+  public String getJobId() {
+    return this.jobContext.getJobId();
+  }
+
   @Override
   protected void runWorkUnits(List<WorkUnit> workUnits) throws Exception {
     try {
+      CountEventBuilder countEventBuilder = new CountEventBuilder(JobEvent.WORK_UNITS_CREATED, workUnits.size());
+      this.eventSubmitter.submit(countEventBuilder);
+      LOGGER.info("Emitting WorkUnitsCreated Count: " + countEventBuilder.getCount());
+
       long workUnitStartTime = System.currentTimeMillis();
       workUnits.forEach((k) -> k.setProp(ConfigurationKeys.WORK_UNIT_CREATION_TIME_IN_MILLIS, workUnitStartTime));
 
@@ -198,7 +214,14 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
 
       synchronized (this.cancellationRequest) {
         if (!this.cancellationRequested) {
-          submitJobToHelix(createJob(workUnits));
+          long submitStart = System.currentTimeMillis();
+          if (helixMetrics.isPresent()) {
+            helixMetrics.get().submitMeter.mark();
+          }
+          submitJobToHelix(createHelixJob(workUnits));
+          if (helixMetrics.isPresent()) {
+            this.helixMetrics.get().updateTimeForHelixSubmit(submitStart);
+          }
           jobSubmissionTimer.stop();
           LOGGER.info(String.format("Submitted job %s to Helix", this.jobContext.getJobId()));
           this.jobSubmitted = true;
@@ -208,7 +231,11 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
       }
 
       TimingEvent jobRunTimer = this.eventSubmitter.getTimingEvent(TimingEvent.RunJobTimings.HELIX_JOB_RUN);
+      long waitStart = System.currentTimeMillis();
       waitForJobCompletion();
+      if (helixMetrics.isPresent()) {
+        this.helixMetrics.get().updateTimeForHelixWait(waitStart);
+      }
       jobRunTimer.stop();
       LOGGER.info(String.format("Job %s completed", this.jobContext.getJobId()));
     } finally {
@@ -226,11 +253,16 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
           // TODO : fix this when HELIX-1180 is completed
           // work flow should never be deleted explicitly because it has a expiry time
           // If cancellation is requested, we should set the job state to CANCELLED/ABORT
-          this.helixTaskDriver.waitToStop(this.helixWorkFlowName, 10000L);
-          log.info("stopped the workflow ", this.helixWorkFlowName);
+          this.helixTaskDriver.waitToStop(this.helixWorkFlowName, this.helixJobStopTimeoutSeconds * 1000);
+          log.info("stopped the workflow {}", this.helixWorkFlowName);
         }
+      } catch (RuntimeException e) {
+        // Cancellation may throw an exception, but Helix set the job state to STOP and it should eventually stop
+        // We will keep this.cancellationExecuted and this.cancellationRequested to true and not propagate the exception
+        log.error("Failed to stop workflow {} in Helix", helixWorkFlowName, e);
       } catch (InterruptedException e) {
-        throw new RuntimeException("Failed to stop workflow " + helixWorkFlowName + " in Helix", e);
+        log.error("Thread interrupted while trying to stop the workflow {} in Helix", helixWorkFlowName);
+        Thread.currentThread().interrupt();
       }
     }
   }
@@ -238,7 +270,7 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
   /**
    * Create a job from a given batch of {@link WorkUnit}s.
    */
-  private JobConfig.Builder createJob(List<WorkUnit> workUnits) throws IOException {
+  JobConfig.Builder createHelixJob(List<WorkUnit> workUnits) throws IOException {
     Map<String, TaskConfig> taskConfigMap = Maps.newHashMap();
 
     try (ParallelRunner stateSerDeRunner = new ParallelRunner(this.stateSerDeRunnerThreads, this.fs)) {
@@ -262,23 +294,34 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
         SerializationUtils.serializeState(this.fs, jobStateFilePath, this.jobContext.getJobState());
       }
 
-      LOGGER.debug("GobblinHelixJobLauncher.createJob: jobStateFilePath {}, jobState {} jobProperties {}",
-          jobStateFilePath, this.jobContext.getJobState().toString(), this.jobContext.getJobState().getProperties());
-    }
+      // Block on persistence of all workunits to be finished.
+      // It is necessary when underlying storage being slow and Helix activate task-execution before the workunit being persisted.
+      stateSerDeRunner.waitForTasks(Long.MAX_VALUE);
 
+      LOGGER.debug("GobblinHelixJobLauncher.createHelixJob: jobStateFilePath {}, jobState {} jobProperties {}",
+          jobStateFilePath, this.jobContext.getJobState().toString(), this.jobContext.getJobState().getProperties());
+
+      return translateGobblinJobConfigToHelixJobConfig(this.jobContext.getJobState(), workUnits, taskConfigMap);
+    }
+  }
+
+  /**
+   * Populate {@link JobConfig.Builder} with relevant gobblin job-configurations.
+   */
+  JobConfig.Builder translateGobblinJobConfigToHelixJobConfig(JobState gobblinJobState, List<WorkUnit> workUnits,
+      Map<String, TaskConfig> taskConfigMap) {
     JobConfig.Builder jobConfigBuilder = new JobConfig.Builder();
 
     // Helix task attempts = retries + 1 (fallback to general task retry for backward compatibility)
-    jobConfigBuilder.setMaxAttemptsPerTask(this.jobContext.getJobState().getPropAsInt(
-        GobblinClusterConfigurationKeys.HELIX_MAX_TASK_RETRIES_KEY,
-        this.jobContext.getJobState().getPropAsInt(
+    jobConfigBuilder.setMaxAttemptsPerTask(gobblinJobState.getPropAsInt(
+        GobblinClusterConfigurationKeys.HELIX_TASK_MAX_ATTEMPTS_KEY, gobblinJobState.getPropAsInt(
             ConfigurationKeys.MAX_TASK_RETRIES_KEY,
             ConfigurationKeys.DEFAULT_MAX_TASK_RETRIES)) + 1);
 
     // Helix task timeout (fallback to general task timeout for backward compatibility)
-    jobConfigBuilder.setTimeoutPerTask(this.jobContext.getJobState().getPropAsLong(
+    jobConfigBuilder.setTimeoutPerTask(gobblinJobState.getPropAsLong(
         GobblinClusterConfigurationKeys.HELIX_TASK_TIMEOUT_SECONDS,
-        this.jobContext.getJobState().getPropAsLong(
+        gobblinJobState.getPropAsLong(
             ConfigurationKeys.TASK_TIMEOUT_SECONDS,
             ConfigurationKeys.DEFAULT_TASK_TIMEOUT_SECONDS)) * 1000);
 
@@ -294,11 +337,17 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
       jobConfigBuilder.setInstanceGroupTag(jobTag);
     }
 
+    if (this.jobConfig.hasPath(GobblinClusterConfigurationKeys.HELIX_JOB_TYPE_KEY)) {
+      String jobType = this.jobConfig.getString(GobblinClusterConfigurationKeys.HELIX_JOB_TYPE_KEY);
+      log.info("Job {} has types associated : {}", this.jobContext.getJobId(), jobType);
+      jobConfigBuilder.setJobType(jobType);
+    }
+
     if (Task.getExecutionModel(ConfigUtils.configToState(jobConfig)).equals(ExecutionModel.STREAMING)) {
       jobConfigBuilder.setRebalanceRunningTask(true);
     }
 
-    jobConfigBuilder.setExpiry(this.jobContext.getJobState().getPropAsLong(
+    jobConfigBuilder.setExpiry(gobblinJobState.getPropAsLong(
         GobblinClusterConfigurationKeys.HELIX_WORKFLOW_EXPIRY_TIME_SECONDS, GobblinClusterConfigurationKeys.DEFAULT_HELIX_WORKFLOW_EXPIRY_TIME_SECONDS));
 
     return jobConfigBuilder;
@@ -308,8 +357,12 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
    * Submit a job to run.
    */
   private void submitJobToHelix(JobConfig.Builder jobConfigBuilder) throws Exception {
-    HelixUtils.submitJobToWorkFlow(jobConfigBuilder, this.helixWorkFlowName, this.jobContext.getJobId(),
-        this.helixTaskDriver, this.helixManager, this.workFlowExpiryTimeSeconds);
+    HelixUtils.submitJobToWorkFlow(jobConfigBuilder,
+        this.helixWorkFlowName,
+        this.jobContext.getJobId(),
+        this.helixTaskDriver,
+        this.helixManager,
+        this.workFlowExpiryTimeSeconds);
   }
 
   public void launchJob(@Nullable JobListener jobListener)
@@ -317,6 +370,8 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
     this.jobListener = jobListener;
     boolean isLaunched = false;
     this.runningMap.putIfAbsent(this.jobContext.getJobName(), false);
+
+    Throwable errorInJobLaunching = null;
     try {
       if (this.runningMap.replace(this.jobContext.getJobName(), false, true)) {
         LOGGER.info ("Job {} will be executed, add into running map.", this.jobContext.getJobId());
@@ -325,19 +380,24 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
       } else {
         LOGGER.warn ("Job {} will not be executed because other jobs are still running.", this.jobContext.getJobId());
       }
+      // TODO: Better error handling
+    } catch (Throwable t) {
+      errorInJobLaunching = t;
     } finally {
       if (isLaunched) {
         if (this.runningMap.replace(this.jobContext.getJobName(), true, false)) {
           LOGGER.info ("Job {} is done, remove from running map.", this.jobContext.getJobId());
         } else {
-          throw new IllegalStateException("A launched job should have running state equal to true in the running map.");
+          throw errorInJobLaunching == null ? new IllegalStateException("A launched job should have running state equal to true in the running map.")
+              : new RuntimeException("Failure in launching job:", errorInJobLaunching);
         }
       }
     }
   }
 
   /**
-   * Add a single {@link WorkUnit} (flattened).
+   * Add a single {@link WorkUnit} (flattened) to persistent storage so that worker can fetch that based on information
+   * fetched in Helix task.
    */
   private void addWorkUnit(WorkUnit workUnit, ParallelRunner stateSerDeRunner,
       Map<String, TaskConfig> taskConfigMap) throws IOException {
@@ -392,12 +452,17 @@ public class GobblinHelixJobLauncher extends AbstractJobLauncher {
         GobblinClusterConfigurationKeys.HELIX_JOB_TIMEOUT_SECONDS,
         GobblinClusterConfigurationKeys.DEFAULT_HELIX_JOB_TIMEOUT_SECONDS));
 
+    long stoppingStateTimeoutInSeconds = PropertiesUtils
+        .getPropAsLong(this.jobProps, GobblinClusterConfigurationKeys.HELIX_JOB_STOPPING_STATE_TIMEOUT_SECONDS,
+            GobblinClusterConfigurationKeys.DEFAULT_HELIX_JOB_STOPPING_STATE_TIMEOUT_SECONDS);
+
     try {
       HelixUtils.waitJobCompletion(
           this.helixManager,
           this.helixWorkFlowName,
           this.jobContext.getJobId(),
-          timeoutEnabled? Optional.of(timeoutInSeconds) : Optional.empty());
+          timeoutEnabled? Optional.of(timeoutInSeconds) : Optional.empty(),
+          stoppingStateTimeoutInSeconds);
     } catch (TimeoutException te) {
       HelixUtils.handleJobTimeout(helixWorkFlowName, jobContext.getJobId(),
           helixManager, this, this.jobListener);
