@@ -17,23 +17,22 @@
 
 package org.apache.gobblin.cluster;
 
-import java.io.IOException;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 
 import org.apache.commons.lang.exception.ExceptionUtils;
 import org.apache.hadoop.fs.Path;
+import org.apache.helix.HelixException;
 import org.apache.helix.HelixManager;
 import org.apache.helix.task.Task;
 import org.apache.helix.task.TaskCallbackContext;
 import org.apache.helix.task.TaskConfig;
 import org.apache.helix.task.TaskResult;
 
-import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Optional;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.io.Closer;
 import com.typesafe.config.Config;
@@ -41,17 +40,15 @@ import com.typesafe.config.Config;
 import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.configuration.ConfigurationKeys;
-import org.apache.gobblin.configuration.WorkUnitState;
 import org.apache.gobblin.instrumented.Instrumented;
 import org.apache.gobblin.instrumented.StandardMetricsBridge;
 import org.apache.gobblin.metrics.ContextAwareTimer;
 import org.apache.gobblin.metrics.MetricContext;
 import org.apache.gobblin.metrics.Tag;
 import org.apache.gobblin.runtime.JobException;
-import org.apache.gobblin.runtime.TaskState;
-import org.apache.gobblin.runtime.util.StateStores;
-import org.apache.gobblin.source.extractor.partition.Partitioner;
 import org.apache.gobblin.util.ConfigUtils;
+import org.apache.gobblin.util.PropertiesUtils;
+
 
 /**
  * An implementation of Helix's {@link org.apache.helix.task.Task} that runs original {@link GobblinHelixJobLauncher}.
@@ -62,24 +59,32 @@ class GobblinHelixJobTask implements Task {
   private final TaskConfig taskConfig;
   private final Config sysConfig;
   private final Properties jobPlusSysConfig;
-  private final StateStores stateStores;
+  private final HelixJobsMapping jobsMapping;
+  private final String applicationName;
+  private final String instanceName;
   private final String planningJobId;
-  private final HelixManager helixManager;
+  private final HelixManager jobHelixManager;
   private final Path appWorkDir;
+  private final String jobName;
   private final List<? extends Tag<?>> metadataTags;
   private GobblinHelixJobLauncher launcher;
   private GobblinHelixJobTaskMetrics jobTaskMetrics;
+  private GobblinHelixMetrics helixMetrics;
   private GobblinHelixJobLauncherListener jobLauncherListener;
 
   public GobblinHelixJobTask (TaskCallbackContext context,
-                              StateStores stateStores,
+                              HelixJobsMapping jobsMapping,
                               TaskRunnerSuiteBase.Builder builder,
                               GobblinHelixJobLauncherMetrics launcherMetrics,
-                              GobblinHelixJobTaskMetrics jobTaskMetrics) {
+                              GobblinHelixJobTaskMetrics jobTaskMetrics,
+                              GobblinHelixMetrics helixMetrics) {
+    this.applicationName = builder.getApplicationName();
+    this.instanceName = builder.getInstanceName();
     this.jobTaskMetrics = jobTaskMetrics;
+    this.helixMetrics = helixMetrics;
     this.taskConfig = context.getTaskConfig();
     this.sysConfig = builder.getConfig();
-    this.helixManager = builder.getHelixManager();
+    this.jobHelixManager = builder.getJobHelixManager();
     this.jobPlusSysConfig = ConfigUtils.configToProperties(sysConfig);
     this.jobLauncherListener = new GobblinHelixJobLauncherListener(launcherMetrics);
 
@@ -95,8 +100,9 @@ class GobblinHelixJobTask implements Task {
       throw new RuntimeException("Job doesn't have planning ID");
     }
 
+    this.jobName = jobPlusSysConfig.getProperty(ConfigurationKeys.JOB_NAME_KEY);
     this.planningJobId = jobPlusSysConfig.getProperty(GobblinClusterConfigurationKeys.PLANNING_ID_KEY);
-    this.stateStores = stateStores;
+    this.jobsMapping = jobsMapping;
     this.appWorkDir = builder.getAppWorkPath();
     this.metadataTags = Tag.fromMap(new ImmutableMap.Builder<String, Object>()
         .put(GobblinClusterMetricTagNames.APPLICATION_NAME, builder.getApplicationName())
@@ -117,7 +123,7 @@ class GobblinHelixJobTask implements Task {
     public void updateTimeBetweenJobSubmissionAndExecution(Properties jobProps) {
       long jobSubmitTime = Long.parseLong(jobProps.getProperty(GobblinClusterConfigurationKeys.PLANNING_JOB_CREATE_TIME, "0"));
       if (jobSubmitTime != 0) {
-        Instrumented.updateTimer(Optional.of(this.timeBetweenJobSubmissionAndExecution),
+        Instrumented.updateTimer(com.google.common.base.Optional.of(this.timeBetweenJobSubmissionAndExecution),
             System.currentTimeMillis() - jobSubmitTime,
             TimeUnit.MILLISECONDS);
       }
@@ -127,10 +133,11 @@ class GobblinHelixJobTask implements Task {
   private GobblinHelixJobLauncher createJobLauncher()
       throws Exception {
     return new GobblinHelixJobLauncher(jobPlusSysConfig,
-        this.helixManager,
+        this.jobHelixManager,
         this.appWorkDir,
         this.metadataTags,
-        new ConcurrentHashMap<>());
+        new ConcurrentHashMap<>(),
+        Optional.of(this.helixMetrics));
   }
 
   /**
@@ -138,30 +145,71 @@ class GobblinHelixJobTask implements Task {
    */
   @Override
   public TaskResult run() {
-    log.info("Running planning job {}", this.planningJobId);
+    log.info("Running planning job {} [{} {}]", this.planningJobId, this.applicationName, this.instanceName);
     this.jobTaskMetrics.updateTimeBetweenJobSubmissionAndExecution(this.jobPlusSysConfig);
+
     try (Closer closer = Closer.create()) {
-      this.launcher = createJobLauncher();
-      closer.register(launcher).launchJob(this.jobLauncherListener);
-      setResultToUserContent(ImmutableMap.of(Partitioner.IS_EARLY_STOPPED, "false"));
+      Optional<String> planningIdFromStateStore = this.jobsMapping.getPlanningJobId(jobName);
+
+      long timeOut = PropertiesUtils.getPropAsLong(jobPlusSysConfig,
+                                         GobblinClusterConfigurationKeys.HELIX_WORKFLOW_DELETE_TIMEOUT_SECONDS,
+                                         GobblinClusterConfigurationKeys.DEFAULT_HELIX_WORKFLOW_DELETE_TIMEOUT_SECONDS) * 1000;
+
+      if (planningIdFromStateStore.isPresent() && !planningIdFromStateStore.get().equals(this.planningJobId)) {
+        return new TaskResult(TaskResult.Status.FAILED, "Exception occurred for job " + planningJobId
+            + ": because planning job in state store has different id (" + planningIdFromStateStore.get() + ")");
+      }
+
+      while (true) {
+        Optional<String> actualJobIdFromStateStore = this.jobsMapping.getActualJobId(jobName);
+        if (actualJobIdFromStateStore.isPresent()) {
+          String previousActualJobId = actualJobIdFromStateStore.get();
+          if (HelixUtils.isJobFinished(previousActualJobId, previousActualJobId, this.jobHelixManager)) {
+            log.info("Previous actual job {} [plan: {}] finished, will launch a new job.", previousActualJobId, this.planningJobId);
+          } else {
+            log.info("Previous actual job {} [plan: {}] not finished, kill it now.", previousActualJobId, this.planningJobId);
+            try {
+              HelixUtils.deleteWorkflow(previousActualJobId, this.jobHelixManager, timeOut);
+            } catch (HelixException e) {
+              log.error("Helix cannot delete previous actual job id {} within {} seconds.", previousActualJobId, timeOut / 1000);
+              return new TaskResult(TaskResult.Status.FAILED, ExceptionUtils.getFullStackTrace(e));
+            }
+          }
+        } else {
+          log.info("No previous actual job [plan: {}]. First time run.", this.planningJobId);
+        }
+
+        String actualJobId = HelixJobsMapping.createActualJobId(jobPlusSysConfig);
+        log.info("Planning job {} creates actual job {}", this.planningJobId, actualJobId);
+
+        this.jobPlusSysConfig.setProperty(ConfigurationKeys.JOB_ID_KEY, actualJobId);
+
+        this.launcher = createJobLauncher();
+
+        this.jobsMapping.setActualJobId(jobName, this.planningJobId, this.launcher.getJobId());
+
+        closer.register(launcher).launchJob(this.jobLauncherListener);
+
+        if (!this.launcher.isEarlyStopped()) {
+          break;
+        } else {
+          log.info("Planning job {} has more runs due to early stop.", this.planningJobId);
+        }
+      }
+      log.info("Completing planning job {}", this.planningJobId);
+      return new TaskResult(TaskResult.Status.COMPLETED, "");
     } catch (Exception e) {
+      log.info("Failing planning job {}", this.planningJobId);
       return new TaskResult(TaskResult.Status.FAILED, "Exception occurred for job " + planningJobId + ":" + ExceptionUtils
           .getFullStackTrace(e));
+    } finally {
+      // always cleanup the job mapping for current job name.
+      try {
+        this.jobsMapping.deleteMapping(jobName);
+      } catch (Exception e) {
+        return new TaskResult(TaskResult.Status.FAILED,"Cannot delete jobs mapping for job : " + jobName);
+      }
     }
-    return new TaskResult(TaskResult.Status.COMPLETED, "");
-  }
-
-  //TODO: change below to Helix UserConentStore
-  @VisibleForTesting
-  protected void setResultToUserContent(Map<String, String> keyValues) throws IOException {
-    WorkUnitState wus = new WorkUnitState();
-    wus.setProp(ConfigurationKeys.JOB_ID_KEY, this.planningJobId);
-    wus.setProp(ConfigurationKeys.TASK_ID_KEY, this.planningJobId);
-    wus.setProp(ConfigurationKeys.TASK_KEY_KEY, this.planningJobId);
-    keyValues.forEach((key, value)->wus.setProp(key, value));
-    TaskState taskState = new TaskState(wus);
-
-    this.stateStores.getTaskStateStore().put(this.planningJobId, this.planningJobId, taskState);
   }
 
   @Override
@@ -172,6 +220,13 @@ class GobblinHelixJobTask implements Task {
         launcher.cancelJob(this.jobLauncherListener);
       } catch (JobException e) {
         throw new RuntimeException("Unable to cancel planning job " + this.planningJobId + ": ", e);
+      } finally {
+        // always cleanup the job mapping for current job name.
+        try {
+          this.jobsMapping.deleteMapping(jobName);
+        } catch (Exception e) {
+          throw new RuntimeException("Cannot delete jobs mapping for job : " + jobName);
+        }
       }
     }
   }
