@@ -87,6 +87,13 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
   @Getter
   private HelixManager jobClusterHelixManager = null;
 
+  /**
+   * Helix manager to handle planning job distribution.
+   * Corresponds to cluster with key name {@link GobblinClusterConfigurationKeys#HELIX_CLUSTER_NAME_KEY}.
+   */
+  @Getter
+  private Optional<HelixManager> taskDriverHelixManager = Optional.empty();
+
   /**
    * Helix controller for job distribution. Effective only iff below two conditions are established:
    * 1. In {@link GobblinHelixMultiManager#dedicatedManagerCluster} mode.
@@ -95,17 +102,32 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
    */
   private Optional<HelixManager> jobClusterController = Optional.empty();
 
+  /**
+   * Helix controller for planning job distribution. Effective only iff below two conditions are established:
+   * 1. In {@link GobblinHelixMultiManager#dedicatedManagerCluster} mode.
+   * 2. {@link GobblinHelixMultiManager#dedicatedTaskDriverCluster} is turned on.
+   * Typically used for unit test and local deployment.
+   */
+  private Optional<HelixManager> taskDriverClusterController = Optional.empty();
+
   /**
    * Separate manager cluster and job distribution cluster iff this flag is turned on. Otherwise {@link GobblinHelixMultiManager#jobClusterHelixManager}
    * is same as {@link GobblinHelixMultiManager#managerClusterHelixManager}.
    */
   private boolean dedicatedManagerCluster = false;
 
+  private boolean dedicatedTaskDriverCluster = false;
+
   /**
    * Create a dedicated controller for job distribution.
    */
   private boolean dedicatedJobClusterController = true;
 
+  /**
+   * Create a dedicated controller for planning job distribution.
+   */
+  private boolean dedicatedTaskDriverClusterController = true;
+
   @Getter
   boolean isLeader = false;
   boolean isStandaloneMode = false;
@@ -131,6 +153,8 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
     this.metrics = new HelixManagerMetrics(this.metricContext, this.config);
     this.dedicatedManagerCluster = ConfigUtils.getBoolean(config,
        GobblinClusterConfigurationKeys.DEDICATED_MANAGER_CLUSTER_ENABLED,false);
+    this.dedicatedTaskDriverCluster = ConfigUtils.getBoolean(config,
+        GobblinClusterConfigurationKeys.DEDICATED_TASK_DRIVER_CLUSTER_ENABLED, false);
     this.userDefinedMessageHandlerFactory = messageHandlerFactoryFunction.apply(null);
     initialize();
   }
@@ -180,13 +204,35 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
             .buildHelixManager(this.config, zkConnectionString, GobblinClusterConfigurationKeys.HELIX_CLUSTER_NAME_KEY,
                 InstanceType.CONTROLLER));
       }
+
+      if (this.dedicatedTaskDriverCluster) {
+        // This will create a Helix administrator to dispatch jobs to ZooKeeper
+        this.taskDriverHelixManager = Optional.of(buildHelixManager(this.config,
+            zkConnectionString,
+            GobblinClusterConfigurationKeys.TASK_DRIVER_CLUSTER_NAME_KEY,
+            InstanceType.ADMINISTRATOR));
+
+        this.dedicatedTaskDriverClusterController = ConfigUtils.getBoolean(
+            this.config,
+            GobblinClusterConfigurationKeys.DEDICATED_TASK_DRIVER_CLUSTER_CONTROLLER_ENABLED,
+            true);
+
+        // This will creat a dedicated controller for planning job distribution
+        if (this.dedicatedTaskDriverClusterController) {
+          this.taskDriverClusterController = Optional.of(GobblinHelixMultiManager
+              .buildHelixManager(this.config, zkConnectionString, GobblinClusterConfigurationKeys.TASK_DRIVER_CLUSTER_NAME_KEY,
+                  InstanceType.CONTROLLER));
+        }
+      }
     } else {
       log.info("We will use same cluster to manage GobblinClusterManager and job distribution.");
       // This will create and register a Helix controller in ZooKeeper
+      boolean isHelixClusterManaged = ConfigUtils.getBoolean(this.config, GobblinClusterConfigurationKeys.IS_HELIX_CLUSTER_MANAGED,
+          GobblinClusterConfigurationKeys.DEFAULT_IS_HELIX_CLUSTER_MANAGED);
       this.managerClusterHelixManager = buildHelixManager(this.config,
           zkConnectionString,
           GobblinClusterConfigurationKeys.HELIX_CLUSTER_NAME_KEY,
-          InstanceType.CONTROLLER);
+          isHelixClusterManaged ? InstanceType.PARTICIPANT : InstanceType.CONTROLLER);
       this.jobClusterHelixManager = this.managerClusterHelixManager;
     }
   }
@@ -200,7 +246,15 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
         if (jobClusterController.isPresent()) {
           this.jobClusterController.get().connect();
         }
+        if (this.dedicatedTaskDriverCluster) {
+          if (taskDriverClusterController.isPresent()) {
+            this.taskDriverClusterController.get().connect();
+          }
+        }
         this.jobClusterHelixManager.connect();
+        if (this.taskDriverHelixManager.isPresent()) {
+          this.taskDriverHelixManager.get().connect();
+        }
       }
 
       this.jobClusterHelixManager.addLiveInstanceChangeListener(new GobblinLiveInstanceChangeListener());
@@ -239,10 +293,17 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
         this.jobClusterHelixManager.disconnect();
       }
 
+      if (taskDriverHelixManager.isPresent()) {
+        this.taskDriverHelixManager.get().disconnect();
+      }
+
       if (jobClusterController.isPresent() && jobClusterController.get().isConnected()) {
         this.jobClusterController.get().disconnect();
       }
 
+      if (taskDriverClusterController.isPresent() && taskDriverClusterController.get().isConnected()) {
+        this.taskDriverClusterController.get().disconnect();
+      }
     }
   }
 
@@ -269,7 +330,8 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
   void handleLeadershipChange(NotificationContext changeContext) {
     this.metrics.clusterLeadershipChange.update(1);
     if (this.managerClusterHelixManager.isLeader()) {
-      // can get multiple notifications on a leadership change, so only start the application launcher the first time
+      // can get multiple notifications on a leadership change,
+      // so only start the application launcher the first time
       // the notification is received
       log.info("Leader notification for {} isLeader {} HM.isLeader {}",
           managerClusterHelixManager.getInstanceName(),
@@ -279,21 +341,7 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
       if (!isLeader) {
         log.info("New Helix Controller leader {}", this.managerClusterHelixManager.getInstanceName());
 
-        // Clean up existing jobs
-        TaskDriver taskDriver = new TaskDriver(this.jobClusterHelixManager);
-        Map<String, WorkflowConfig> workflows = taskDriver.getWorkflows();
-
-        for (Map.Entry<String, WorkflowConfig> entry : workflows.entrySet()) {
-          String queueName = entry.getKey();
-          WorkflowConfig workflowConfig = entry.getValue();
-
-          // request delete if not already requested
-          if (workflowConfig.getTargetState() != TargetState.DELETE) {
-            taskDriver.delete(queueName);
-
-            log.info("Requested delete of queue {}", queueName);
-          }
-        }
+        cleanUpJobs();
 
         for (LeadershipChangeAwareComponent c: this.leadershipChangeAwareComponents) {
           c.becomeActive();
@@ -313,6 +361,51 @@ public class GobblinHelixMultiManager implements StandardMetricsBridge {
     }
   }
 
+  /**
+   * Delete jobs from the helix cluster
+   */
+  @VisibleForTesting
+  public void cleanUpJobs() {
+    cleanUpJobs(this.jobClusterHelixManager);
+
+    if (this.taskDriverHelixManager.isPresent()) {
+      cleanUpJobs(this.taskDriverHelixManager.get());
+    }
+  }
+
+  private void cleanUpJobs(HelixManager helixManager) {
+    // Clean up existing jobs
+    TaskDriver taskDriver = new TaskDriver(helixManager);
+
+    Map<String, WorkflowConfig> workflows = taskDriver.getWorkflows();
+
+    log.debug("cleanUpJobs workflow count {} workflows {}", workflows.size(), workflows.keySet());
+
+    boolean cleanupDistJobs = ConfigUtils.getBoolean(this.config,
+        GobblinClusterConfigurationKeys.CLEAN_ALL_DIST_JOBS,
+        GobblinClusterConfigurationKeys.DEFAULT_CLEAN_ALL_DIST_JOBS);
+
+    for (Map.Entry<String, WorkflowConfig> entry : workflows.entrySet()) {
+      String workflowName = entry.getKey();
+
+      if (workflowName.contains(GobblinClusterConfigurationKeys.PLANNING_JOB_NAME_PREFIX)
+          || workflowName.contains(GobblinClusterConfigurationKeys.ACTUAL_JOB_NAME_PREFIX)) {
+        if (!cleanupDistJobs) {
+          log.info("Distributed job {} won't be deleted.", workflowName);
+          continue;
+        }
+      }
+
+      WorkflowConfig workflowConfig = entry.getValue();
+
+      // request delete if not already requested
+      if (workflowConfig.getTargetState() != TargetState.DELETE) {
+        taskDriver.delete(workflowName);
+
+        log.info("Requested delete of workflowName {}", workflowName);
+      }
+    }
+  }
 
   /**
    * A custom {@link MessageHandlerFactory} for {@link MessageHandler}s that handle messages of type
