@@ -51,8 +51,6 @@ import org.apache.gobblin.util.ConfigUtils;
 @Slf4j
 @Alpha
 public abstract class TaskRunnerSuiteBase {
-  protected TaskFactory taskFactory;
-  protected GobblinHelixJobFactory jobFactory;
   protected MetricContext metricContext;
   protected String applicationId;
   protected String applicationName;
@@ -84,20 +82,25 @@ public abstract class TaskRunnerSuiteBase {
 
   @Getter
   public static class Builder {
-    private Config config;
-    private HelixManager helixManager;
+    private final Config config;
+    private final Config dynamicConfig;
+    private HelixManager jobHelixManager;
     private Optional<ContainerMetrics> containerMetrics;
     private FileSystem fs;
     private Path appWorkPath;
     private String applicationId;
     private String applicationName;
+    private String instanceName;
+    private String hostName;
+    private String containerId;
 
     public Builder(Config config) {
+      this.dynamicConfig = GobblinClusterUtils.getDynamicConfig(config);
       this.config = config;
     }
 
-    public Builder setHelixManager(HelixManager manager) {
-      this.helixManager = manager;
+    public Builder setJobHelixManager(HelixManager jobHelixManager) {
+      this.jobHelixManager = jobHelixManager;
       return this;
     }
 
@@ -106,6 +109,21 @@ public abstract class TaskRunnerSuiteBase {
       return this;
     }
 
+    public Builder setInstanceName(String instanceName) {
+      this.instanceName = instanceName;
+      return this;
+    }
+
+    public Builder setContainerId (String containerId) {
+      this.containerId = containerId;
+      return this;
+    }
+
+    public Builder setHostName(String hostName) {
+      this.hostName = hostName;
+      return this;
+    }
+
     public Builder setApplicationId(String applicationId) {
       this.applicationId = applicationId;
       return this;
