@@ -21,25 +21,11 @@ import java.io.File;
 import java.io.IOException;
 import java.util.Map;
 import java.util.Properties;
+import java.util.concurrent.CountDownLatch;
 
 import org.apache.avro.Schema;
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
-import org.apache.helix.HelixManager;
-import org.apache.helix.task.TaskCallbackContext;
-import org.apache.helix.task.TaskConfig;
-import org.apache.helix.task.TaskResult;
-import org.mockito.Mockito;
-import org.testng.Assert;
-import org.testng.annotations.AfterClass;
-import org.testng.annotations.BeforeClass;
-import org.testng.annotations.Test;
-
-import com.google.common.base.Optional;
-import com.google.common.collect.Maps;
-import com.typesafe.config.ConfigFactory;
 
+import org.apache.gobblin.broker.SharedResourcesBrokerFactory;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.example.simplejson.SimpleJsonConverter;
 import org.apache.gobblin.example.simplejson.SimpleJsonSource;
@@ -50,9 +36,41 @@ import org.apache.gobblin.runtime.TaskExecutor;
 import org.apache.gobblin.source.workunit.WorkUnit;
 import org.apache.gobblin.util.Id;
 import org.apache.gobblin.util.SerializationUtils;
+import org.apache.gobblin.util.event.ContainerHealthCheckFailureEvent;
+import org.apache.gobblin.util.eventbus.EventBusFactory;
+import org.apache.gobblin.util.retry.RetryerFactory;
 import org.apache.gobblin.writer.AvroDataWriterBuilder;
 import org.apache.gobblin.writer.Destination;
 import org.apache.gobblin.writer.WriterOutputFormat;
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.fs.Path;
+import org.apache.helix.HelixManager;
+import org.apache.helix.task.JobConfig;
+import org.apache.helix.task.JobContext;
+import org.apache.helix.task.TaskCallbackContext;
+import org.apache.helix.task.TaskConfig;
+import org.apache.helix.task.TaskDriver;
+import org.apache.helix.task.TaskResult;
+import org.mockito.Mockito;
+import org.testng.Assert;
+import org.testng.annotations.AfterClass;
+import org.testng.annotations.BeforeClass;
+import org.testng.annotations.Test;
+
+import com.google.common.base.Joiner;
+import com.google.common.base.Optional;
+import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.Maps;
+import com.google.common.eventbus.EventBus;
+import com.google.common.eventbus.Subscribe;
+import com.typesafe.config.ConfigFactory;
+import com.typesafe.config.ConfigValueFactory;
+
+import static org.apache.gobblin.cluster.GobblinHelixTaskStateTracker.IS_TASK_METRICS_SCHEDULING_FAILURE_FATAL;
+import static org.apache.gobblin.util.retry.RetryerFactory.RETRY_TIMES;
+import static org.apache.gobblin.util.retry.RetryerFactory.RETRY_TYPE;
+import static org.mockito.Mockito.when;
 
 
 /**
@@ -84,6 +102,8 @@ public class GobblinHelixTaskTest {
 
   private Path taskOutputDir;
 
+  private CountDownLatch countDownLatchForFailInTaskCreation;
+
   @BeforeClass
   public void setUp() throws IOException {
     Configuration configuration = new Configuration();
@@ -91,8 +111,10 @@ public class GobblinHelixTaskTest {
     this.taskExecutor = new TaskExecutor(configuration);
 
     this.helixManager = Mockito.mock(HelixManager.class);
-    Mockito.when(this.helixManager.getInstanceName()).thenReturn(GobblinHelixTaskTest.class.getSimpleName());
-    this.taskStateTracker = new GobblinHelixTaskStateTracker(new Properties());
+    when(this.helixManager.getInstanceName()).thenReturn(GobblinHelixTaskTest.class.getSimpleName());
+    Properties stateTrackerProp = new Properties();
+    stateTrackerProp.setProperty(IS_TASK_METRICS_SCHEDULING_FAILURE_FATAL, "true");
+    this.taskStateTracker = new GobblinHelixTaskStateTracker(stateTrackerProp);
 
     this.localFs = FileSystem.getLocal(configuration);
     this.appWorkDir = new Path(GobblinHelixTaskTest.class.getSimpleName());
@@ -100,7 +122,15 @@ public class GobblinHelixTaskTest {
   }
 
   @Test
-  public void testPrepareTask() throws IOException {
+  public void testPrepareTask()
+      throws IOException, InterruptedException {
+
+    EventBus eventBus = EventBusFactory.get(ContainerHealthCheckFailureEvent.CONTAINER_HEALTH_CHECK_EVENT_BUS_NAME,
+        SharedResourcesBrokerFactory.getImplicitBroker());
+    eventBus.register(this);
+
+    countDownLatchForFailInTaskCreation = new CountDownLatch(1);
+
     // Serialize the JobState that will be read later in GobblinHelixTask
     Path jobStateFilePath =
         new Path(appWorkDir, TestHelper.TEST_JOB_ID + "." + AbstractJobLauncher.JOB_STATE_FILE_NAME);
@@ -137,13 +167,75 @@ public class GobblinHelixTaskTest {
 
     TaskConfig taskConfig = new TaskConfig("", taskConfigMap, true);
     TaskCallbackContext taskCallbackContext = Mockito.mock(TaskCallbackContext.class);
-    Mockito.when(taskCallbackContext.getTaskConfig()).thenReturn(taskConfig);
-    Mockito.when(taskCallbackContext.getManager()).thenReturn(this.helixManager);
+    when(taskCallbackContext.getTaskConfig()).thenReturn(taskConfig);
+    when(taskCallbackContext.getManager()).thenReturn(this.helixManager);
+    TaskDriver taskDriver = createTaskDriverWithMockedAttributes(taskCallbackContext, taskConfig);
+
+    TaskRunnerSuiteBase.Builder builder = new TaskRunnerSuiteBase.Builder(ConfigFactory.empty()
+        .withValue(RETRY_TYPE, ConfigValueFactory.fromAnyRef(RetryerFactory.RetryType.FIXED_ATTEMPT.name()))
+        .withValue(RETRY_TIMES, ConfigValueFactory.fromAnyRef(2))
+    );
+    TaskRunnerSuiteBase sb = builder.setInstanceName("TestInstance")
+        .setApplicationName("TestApplication")
+        .setAppWorkPath(appWorkDir)
+        .setContainerMetrics(Optional.absent())
+        .setFileSystem(localFs)
+        .setJobHelixManager(this.helixManager)
+        .setApplicationId("TestApplication-1")
+        .build();
 
     GobblinHelixTaskFactory gobblinHelixTaskFactory =
-        new GobblinHelixTaskFactory(Optional.<ContainerMetrics>absent(), this.taskExecutor, this.taskStateTracker,
-            this.localFs, this.appWorkDir, ConfigFactory.empty(), this.helixManager);
+        new GobblinHelixTaskFactory(builder,
+                                    sb.metricContext,
+                                    this.taskStateTracker,
+                                    ConfigFactory.empty(),
+                                    Optional.of(taskDriver));
+
+    // Expect to go through.
     this.gobblinHelixTask = (GobblinHelixTask) gobblinHelixTaskFactory.createNewTask(taskCallbackContext);
+
+    // Mock the method getFs() which get called in SingleTask constructor, so that SingleTask could fail and trigger retry,
+    // which would also fail eventually with timeout.
+    TaskRunnerSuiteBase.Builder builderSpy = Mockito.spy(builder);
+    when(builderSpy.getFs()).thenThrow(new RuntimeException("failure on purpose"));
+    gobblinHelixTaskFactory =
+        new GobblinHelixTaskFactory(builderSpy,
+            sb.metricContext,
+            this.taskStateTracker,
+            ConfigFactory.empty(),
+            Optional.of(taskDriver));
+
+    // Expecting the eventBus containing the failure signal when run is called
+    try {
+      gobblinHelixTaskFactory.createNewTask(taskCallbackContext).run();
+    } catch (Throwable t){
+      return;
+    }
+    Assert.fail();
+  }
+
+  @Subscribe
+  public void handleContainerHealthCheckFailureEvent(ContainerHealthCheckFailureEvent event) {
+    this.countDownLatchForFailInTaskCreation.countDown();
+  }
+
+  /**
+   * To test against org.apache.gobblin.cluster.GobblinHelixTask#getPartitionForHelixTask(org.apache.helix.task.TaskDriver)
+   * we need to assign the right partition id for each helix task, which would be queried from taskDriver.
+   * This method encapsulate all mocking steps for taskDriver object to return expected value.
+   */
+  private TaskDriver createTaskDriverWithMockedAttributes(TaskCallbackContext taskCallbackContext,
+      TaskConfig taskConfig) {
+    String helixJobId = Joiner.on("_").join(TestHelper.TEST_JOB_ID, TestHelper.TEST_JOB_ID);
+    JobConfig jobConfig = Mockito.mock(JobConfig.class);
+    when(jobConfig.getJobId()).thenReturn(helixJobId);
+    when(taskCallbackContext.getJobConfig()).thenReturn(jobConfig);
+    JobContext mockJobContext = Mockito.mock(JobContext.class);
+    Map<String, Integer> taskIdPartitionMap = ImmutableMap.of(taskConfig.getId(), 0);
+    when(mockJobContext.getTaskIdPartitionMap()).thenReturn(taskIdPartitionMap);
+    TaskDriver taskDriver = Mockito.mock(TaskDriver.class);
+    when(taskDriver.getJobContext(Mockito.anyString())).thenReturn(mockJobContext);
+    return taskDriver;
   }
 
   @Test(dependsOnMethods = "testPrepareTask")
