@@ -20,7 +20,6 @@ package org.apache.gobblin.compaction.hivebasedconstructs;
 import java.io.IOException;
 
 import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.hive.ql.metadata.HiveException;
 import org.apache.thrift.TException;
 
 import org.apache.gobblin.configuration.WorkUnitState;
@@ -33,7 +32,7 @@ import org.apache.gobblin.data.management.conversion.hive.extractor.HiveBaseExtr
  */
 public class HiveMetadataForCompactionExtractorFactory implements HiveBaseExtractorFactory {
   public HiveBaseExtractor createExtractor(WorkUnitState state, FileSystem sourceFs)
-      throws IOException, TException, HiveException {
+      throws IOException, TException {
     return new HiveMetadataForCompactionExtractor(state, sourceFs);
   }
 }
\ No newline at end of file
