@@ -34,8 +34,9 @@ import org.apache.gobblin.compaction.parser.CompactionPathParser;
 import org.apache.gobblin.configuration.State;
 import org.apache.gobblin.dataset.FileSystemDataset;
 
+
 /**
- * Compare the source and destination avro records. Determine if a compaction is needed.
+ * Compare the source and destination file records' count. Determine if a compaction is needed.
  */
 @Slf4j
 public class CompactionThresholdVerifier implements CompactionVerifier<FileSystemDataset> {
@@ -61,31 +62,41 @@ public class CompactionThresholdVerifier implements CompactionVerifier<FileSyste
    *
    * @return true iff the difference exceeds the threshold or this is the first time compaction
    */
-  public Result verify (FileSystemDataset dataset) {
+  public Result verify(FileSystemDataset dataset) {
 
     Map<String, Double> thresholdMap = RecompactionConditionBasedOnRatio.
-            getDatasetRegexAndRecompactThreshold (state.getProp(MRCompactor.COMPACTION_LATEDATA_THRESHOLD_FOR_RECOMPACT_PER_DATASET,
-                    StringUtils.EMPTY));
+        getDatasetRegexAndRecompactThreshold(
+            state.getProp(MRCompactor.COMPACTION_LATEDATA_THRESHOLD_FOR_RECOMPACT_PER_DATASET, StringUtils.EMPTY));
 
     CompactionPathParser.CompactionParserResult result = new CompactionPathParser(state).parse(dataset);
 
-    double threshold = RecompactionConditionBasedOnRatio.getRatioThresholdByDatasetName (result.getDatasetName(), thresholdMap);
-    log.debug ("Threshold is {} for dataset {}", threshold, result.getDatasetName());
+    double threshold =
+        RecompactionConditionBasedOnRatio.getRatioThresholdByDatasetName(result.getDatasetName(), thresholdMap);
+    log.debug("Threshold is {} for dataset {}", threshold, result.getDatasetName());
 
     InputRecordCountHelper helper = new InputRecordCountHelper(state);
     try {
-      double newRecords = helper.calculateRecordCount (Lists.newArrayList(new Path(dataset.datasetURN())));
-      double oldRecords = helper.readRecordCount (new Path(result.getDstAbsoluteDir()));
+      double newRecords = 0;
+      if (!dataset.isVirtual()) {
+        newRecords = helper.calculateRecordCount(Lists.newArrayList(new Path(dataset.datasetURN())));
+      }
+      double oldRecords = helper.readRecordCount(new Path(result.getDstAbsoluteDir()));
 
       if (oldRecords == 0) {
         return new Result(true, "");
       }
+      if (newRecords < oldRecords) {
+        return new Result(false, "Illegal state: Current records count should old be smaller.");
+      }
+
       if ((newRecords - oldRecords) / oldRecords > threshold) {
-        log.debug ("Dataset {} records exceeded the threshold {}", dataset.datasetURN(), threshold);
+        log.debug("Dataset {} records exceeded the threshold {}", dataset.datasetURN(), threshold);
         return new Result(true, "");
       }
 
-      return new Result(false, String.format("%s is failed for dataset %s. Prev=%f, Cur=%f, not reaching to threshold %f", this.getName(), result.getDatasetName(), oldRecords, newRecords, threshold));
+      return new Result(false, String
+          .format("%s is failed for dataset %s. Prev=%f, Cur=%f, not reaching to threshold %f", this.getName(),
+              result.getDatasetName(), oldRecords, newRecords, threshold));
     } catch (IOException e) {
       return new Result(false, ExceptionUtils.getFullStackTrace(e));
     }
@@ -98,7 +109,7 @@ public class CompactionThresholdVerifier implements CompactionVerifier<FileSyste
     return this.getClass().getName();
   }
 
-  public boolean isRetriable () {
+  public boolean isRetriable() {
     return false;
   }
 }
