@@ -32,6 +32,7 @@ import org.apache.gobblin.compaction.audit.AuditCountClient;
 import org.apache.gobblin.compaction.dataset.TimeBasedSubDirDatasetsFinder;
 import org.apache.gobblin.compaction.mapreduce.MRCompactor;
 import org.apache.gobblin.configuration.State;
+import org.apache.gobblin.data.management.dataset.SimpleFileSystemDataset;
 import org.apache.gobblin.dataset.FileSystemDataset;
 
 /**
@@ -51,17 +52,8 @@ public class PinotAuditCountVerifierTest {
     final String inputSub = "hourly";
     final String outputSub = "hourly";
     TestAuditCountClient client = new TestAuditCountClient();
-    FileSystemDataset dataset = new FileSystemDataset() {
-      @Override
-      public Path datasetRoot() {
-        return new Path (input + topic + inputSub + "/2017/04/03/10");
-      }
-
-      @Override
-      public String datasetURN() {
-        return input + topic + inputSub + "/2017/04/03/10";
-      }
-    };
+    FileSystemDataset dataset = new SimpleFileSystemDataset(
+        new Path(input + topic + inputSub + "/2017/04/03/10"));
 
     State props = new State();
     props.setProp (CompactionAuditCountVerifier.PRODUCER_TIER, PRODUCER_TIER);
