@@ -21,18 +21,15 @@ import org.apache.avro.Schema;
 import org.apache.avro.generic.GenericRecord;
 
 import org.apache.gobblin.configuration.WorkUnitState;
+import org.apache.gobblin.instrumented.converter.InstrumentedConverter;
 
 
 /**
  * A base abstract {@link Converter} class for data transformation from Avro to Avro.
  */
-public abstract class AvroToAvroConverterBase extends Converter<Schema, Schema, GenericRecord, GenericRecord> {
+public abstract class AvroToAvroConverterBase extends InstrumentedConverter<Schema, Schema, GenericRecord, GenericRecord> {
 
   @Override
   public abstract Schema convertSchema(Schema inputSchema, WorkUnitState workUnit)
       throws SchemaConversionException;
-
-  @Override
-  public abstract Iterable<GenericRecord> convertRecord(Schema outputSchema, GenericRecord inputRecord, WorkUnitState workUnit)
-      throws DataConversionException;
 }
