@@ -142,18 +142,6 @@ public class InstrumentedDataWriterDecorator<D> extends InstrumentedDataWriterBa
     return watermarkAwareWriter.isPresent() && watermarkAwareWriter.get().isWatermarkCapable();
   }
 
-  @Override
-  public Map<String, CheckpointableWatermark> getCommittableWatermark() {
-    Preconditions.checkState(isWatermarkCapable());
-    return watermarkAwareWriter.get().getCommittableWatermark();
-  }
-
-  @Override
-  public Map<String, CheckpointableWatermark> getUnacknowledgedWatermark() {
-    Preconditions.checkState(isWatermarkCapable());
-    return watermarkAwareWriter.get().getUnacknowledgedWatermark();
-  }
-
   @Override
   public ControlMessageHandler getMessageHandler() {
     return this.embeddedWriter.getMessageHandler();
