@@ -142,7 +142,7 @@ public class JsonElementConversionFactory {
         return new BooleanConverter(schemaNode);
 
       case ARRAY:
-        return new ArrayConverter(schemaNode, state);
+        return new ArrayConverter(schemaNode, state, namespace);
 
       case MAP:
         return new MapConverter(schemaNode, state);
@@ -501,7 +501,7 @@ public class JsonElementConversionFactory {
       return this.elementConverter;
     }
 
-    protected void processNestedItems(JsonSchema schema, WorkUnitState state)
+    protected void processNestedItems(JsonSchema schema, WorkUnitState state, String namespace)
         throws UnsupportedDateTypeException {
       JsonSchema nestedItem = null;
       if (schema.isType(ARRAY)) {
@@ -510,16 +510,16 @@ public class JsonElementConversionFactory {
       if (schema.isType(MAP)) {
         nestedItem = schema.getValuesWithinDataType();
       }
-      this.setElementConverter(getConvertor(nestedItem, null, state));
+      this.setElementConverter(getConvertor(nestedItem, namespace, state));
     }
   }
 
   public static class ArrayConverter extends ComplexConverter {
 
-    public ArrayConverter(JsonSchema schema, WorkUnitState state)
+    public ArrayConverter(JsonSchema schema, WorkUnitState state, String namespace)
         throws UnsupportedDateTypeException {
       super(schema);
-      processNestedItems(schema, state);
+      processNestedItems(schema, state, namespace);
     }
 
     @Override
@@ -530,7 +530,7 @@ public class JsonElementConversionFactory {
       List<Object> list = new ArrayList<>();
 
       for (JsonElement elem : (JsonArray) value) {
-        list.add(getElementConverter().convertField(elem));
+        list.add(getElementConverter().convert(elem));
       }
 
       return new GenericData.Array<>(arraySchema(), list);
@@ -558,7 +558,7 @@ public class JsonElementConversionFactory {
     public MapConverter(JsonSchema schema, WorkUnitState state)
         throws UnsupportedDateTypeException {
       super(schema);
-      processNestedItems(schema, state);
+      processNestedItems(schema, state, null);
     }
 
     @Override
@@ -566,7 +566,7 @@ public class JsonElementConversionFactory {
       Map<String, Object> map = new HashMap<>();
 
       for (Map.Entry<String, JsonElement> entry : ((JsonObject) value).entrySet()) {
-        map.put(entry.getKey(), getElementConverter().convertField(entry.getValue()));
+        map.put(entry.getKey(), getElementConverter().convert(entry.getValue()));
       }
 
       return map;
