@@ -290,6 +290,7 @@ public class BaseDataPublisher extends SingleTaskDataPublisher {
       for (Path path : this.publisherOutputDirs) {
         this.state.appendToSetProp(ConfigurationKeys.PUBLISHER_DIRS, path.toString());
       }
+      this.state.setProp(ConfigurationKeys.PUBLISHER_LATEST_FILE_ARRIVAL_TIMESTAMP, System.currentTimeMillis());
     } finally {
       this.closer.close();
     }
@@ -364,20 +365,13 @@ public class BaseDataPublisher extends SingleTaskDataPublisher {
     }
 
     this.parallelRunnerCloser.close();
-
-    for (WorkUnitState workUnitState : states) {
-      // Upon successfully committing the data to the final output directory, set states
-      // of successful tasks to COMMITTED. leaving states of unsuccessful ones unchanged.
-      // This makes sense to the COMMIT_ON_PARTIAL_SUCCESS policy.
-      workUnitState.setWorkingState(WorkUnitState.WorkingState.COMMITTED);
-    }
   }
 
   /**
    * This method publishes task output data for the given {@link WorkUnitState}, but if there are output data of
    * other tasks in the same folder, it may also publish those data.
    */
-  private void publishMultiTaskData(WorkUnitState state, int branchId, Set<Path> writerOutputPathsMoved)
+  protected void publishMultiTaskData(WorkUnitState state, int branchId, Set<Path> writerOutputPathsMoved)
       throws IOException {
     publishData(state, branchId, false, writerOutputPathsMoved);
     addLineageInfo(state, branchId);
