@@ -117,13 +117,16 @@ public class DatePartitionedNestedRetriever implements PartitionAwareFileRetriev
     for (DateTime date = lowWaterMarkDate; !date.isAfter(currentDay) && filesToProcess.size() < maxFilesToReturn;
         date = date.withFieldAdded(incrementalUnit, 1)) {
 
+      // Constructs the partition path - e.g. prefix/2015/01/01/suffix
+      String partitionPath = constructPartitionPath(date);
       // Constructs the path folder - e.g. /my/data/prefix/2015/01/01/suffix
-      Path sourcePath = constructSourcePath(date);
+      Path sourcePath = new Path(sourceDir, partitionPath);
 
       if (this.fs.exists(sourcePath)) {
         for (FileStatus fileStatus : getFilteredFileStatuses(sourcePath, getFileFilter())) {
           LOG.info("Will process file " + fileStatus.getPath());
-          filesToProcess.add(new FileInfo(fileStatus.getPath().toString(), fileStatus.getLen(), date.getMillis()));
+          filesToProcess.add(
+              new FileInfo(fileStatus.getPath().toString(), fileStatus.getLen(), date.getMillis(), partitionPath));
         }
       }
     }
@@ -186,7 +189,7 @@ public class DatePartitionedNestedRetriever implements PartitionAwareFileRetriev
     this.incrementalUnit = partitionType.getDateTimeFieldType().getDurationType();
   }
 
-  private Path constructSourcePath(DateTime date) {
+  private String constructPartitionPath(DateTime date) {
     StringBuilder pathBuilder = new StringBuilder();
 
     if (!this.sourcePartitionPrefix.isEmpty()) {
@@ -201,7 +204,7 @@ public class DatePartitionedNestedRetriever implements PartitionAwareFileRetriev
       pathBuilder.append(this.sourcePartitionSuffix);
     }
 
-    return new Path(this.sourceDir, pathBuilder.toString());
+    return pathBuilder.toString();
   }
 
   /**
