@@ -104,7 +104,9 @@ public abstract class FsDataWriter<D> implements DataWriter<D>, FinalState, Meta
     // Add all job configuration properties so they are picked up by Hadoop
     JobConfigurationUtils.putStateIntoConfiguration(properties, conf);
     this.fs = WriterUtils.getWriterFS(properties, this.numBranches, this.branchId);
-    this.fileContext = FileContext.getFileContext(conf);
+    this.fileContext = FileContext.getFileContext(
+            WriterUtils.getWriterFsUri(properties, this.numBranches, this.branchId),
+            conf);
 
     // Initialize staging/output directory
     Path writerStagingDir = this.writerAttemptIdOptional.isPresent() ? WriterUtils
@@ -260,7 +262,22 @@ public abstract class FsDataWriter<D> implements DataWriter<D>, FinalState, Meta
     LOG.info(String.format("Moving data from %s to %s", this.stagingFile, this.outputFile));
     // For the same reason as deleting the staging file if it already exists, overwrite
     // the output file if it already exists to prevent task retry from being blocked.
-    HadoopUtils.renamePath(this.fileContext, this.stagingFile, this.outputFile, true);
+    HadoopUtils.renamePath(this.fs, this.stagingFile, this.outputFile, true);
+
+    // The staging file is moved to the output path in commit, so rename to add record count after that
+    if (this.shouldIncludeRecordCountInFileName) {
+      String filePathWithRecordCount = addRecordCountToFileName();
+      this.properties.appendToSetProp(this.allOutputFilesPropName, filePathWithRecordCount);
+    } else {
+      this.properties.appendToSetProp(this.allOutputFilesPropName, getOutputFilePath());
+    }
+
+    FsWriterMetrics metrics = new FsWriterMetrics(
+        this.id,
+        new PartitionIdentifier(this.partitionKey, this.branchId),
+        ImmutableSet.of(new FsWriterMetrics.FileInfo(this.outputFile.getName(), recordsWritten()))
+    );
+    this.properties.setProp(FS_WRITER_METRICS_KEY, metrics.toJson());
  }
 
   /**
@@ -285,20 +302,6 @@ public abstract class FsDataWriter<D> implements DataWriter<D>, FinalState, Meta
   public void close()
       throws IOException {
     this.closer.close();
-
-    if (this.shouldIncludeRecordCountInFileName) {
-      String filePathWithRecordCount = addRecordCountToFileName();
-      this.properties.appendToSetProp(this.allOutputFilesPropName, filePathWithRecordCount);
-    } else {
-      this.properties.appendToSetProp(this.allOutputFilesPropName, getOutputFilePath());
-    }
-
-    FsWriterMetrics metrics = new FsWriterMetrics(
-        this.id,
-        new PartitionIdentifier(this.partitionKey, this.branchId),
-        ImmutableSet.of(new FsWriterMetrics.FileInfo(this.outputFile.getName(), recordsWritten()))
-    );
-    this.properties.setProp(FS_WRITER_METRICS_KEY, metrics.toJson());
   }
 
   private synchronized String addRecordCountToFileName()
@@ -306,7 +309,7 @@ public abstract class FsDataWriter<D> implements DataWriter<D>, FinalState, Meta
     String filePath = getOutputFilePath();
     String filePathWithRecordCount = IngestionRecordCountProvider.constructFilePath(filePath, recordsWritten());
     LOG.info("Renaming " + filePath + " to " + filePathWithRecordCount);
-    HadoopUtils.renamePath(this.fileContext, new Path(filePath), new Path(filePathWithRecordCount), true);
+    HadoopUtils.renamePath(this.fs, new Path(filePath), new Path(filePathWithRecordCount), true);
     this.outputFile = new Path(filePathWithRecordCount);
     return filePathWithRecordCount;
   }
@@ -344,6 +347,10 @@ public abstract class FsDataWriter<D> implements DataWriter<D>, FinalState, Meta
     return this.fs.makeQualified(this.outputFile).toString();
   }
 
+  /**
+   * Classes that extends this method needs to determine if writerAttemptIdOptional is present and to avoid
+   * problems of overriding, adding another checking on class type.
+   */
   @Override
   public boolean isSpeculativeAttemptSafe() {
     return this.writerAttemptIdOptional.isPresent() && this.getClass() == FsDataWriter.class;
