@@ -22,6 +22,9 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ExecutionException;
+import java.util.concurrent.Executors;
+import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Supplier;
 
 import org.apache.avro.SchemaBuilder;
@@ -29,15 +32,19 @@ import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericRecord;
 import org.apache.commons.lang3.reflect.ConstructorUtils;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 import com.google.common.cache.CacheBuilder;
 import com.google.common.cache.CacheLoader;
 import com.google.common.cache.LoadingCache;
+import com.google.common.cache.RemovalListener;
+import com.google.common.cache.RemovalNotification;
 import com.google.common.collect.Lists;
 import com.google.common.io.Closer;
 
+import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.commit.SpeculativeAttemptAwareConstruct;
@@ -48,12 +55,13 @@ import org.apache.gobblin.dataset.PartitionDescriptor;
 import org.apache.gobblin.instrumented.writer.InstrumentedDataWriterDecorator;
 import org.apache.gobblin.instrumented.writer.InstrumentedPartitionedDataWriterDecorator;
 import org.apache.gobblin.records.ControlMessageHandler;
-import org.apache.gobblin.source.extractor.CheckpointableWatermark;
 import org.apache.gobblin.stream.ControlMessage;
+import org.apache.gobblin.stream.FlushControlMessage;
 import org.apache.gobblin.stream.MetadataUpdateControlMessage;
 import org.apache.gobblin.stream.RecordEnvelope;
 import org.apache.gobblin.stream.StreamEntity;
 import org.apache.gobblin.util.AvroUtils;
+import org.apache.gobblin.util.ExecutorsUtils;
 import org.apache.gobblin.util.FinalState;
 import org.apache.gobblin.writer.partitioner.WriterPartitioner;
 
@@ -67,6 +75,13 @@ import org.apache.gobblin.writer.partitioner.WriterPartitioner;
 @Slf4j
 public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements FinalState, SpeculativeAttemptAwareConstruct, WatermarkAwareWriter<D> {
 
+  public static final String WRITER_LATEST_SCHEMA = "writer.latest.schema";
+  //Config to control when a writer is evicted from Partitioned data writer cache.
+  // NOTE: this config must be set only in streaming mode. For batch mode, setting this config will result
+  // in incorrect behavior.
+  public static final String PARTITIONED_WRITER_CACHE_TTL_SECONDS = "partitionedDataWriter.cache.ttl.seconds";
+  public static final Long DEFAULT_PARTITIONED_WRITER_CACHE_TTL_SECONDS = Long.MAX_VALUE;
+
   private static final GenericRecord NON_PARTITIONED_WRITER_KEY =
       new GenericData.Record(SchemaBuilder.record("Dummy").fields().endRecord());
 
@@ -76,6 +91,8 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   private final int branchId;
 
   private final Optional<WriterPartitioner> partitioner;
+  @Getter
+  @VisibleForTesting
   private final LoadingCache<GenericRecord, DataWriter<D>> partitionWriters;
   private final Optional<PartitionAwareDataWriterBuilder> builder;
   private final DataWriterBuilder writerBuilder;
@@ -85,6 +102,17 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   private boolean isSpeculativeAttemptSafe;
   private boolean isWatermarkCapable;
 
+  private ScheduledExecutorService cacheCleanUpExecutor;
+
+  //Counters to keep track of records and bytes of writers which have been evicted from cache.
+  @Getter
+  @VisibleForTesting
+  private long totalRecordsFromEvictedWriters;
+  @Getter
+  @VisibleForTesting
+  private long totalBytesFromEvictedWriters;
+
+
   public PartitionedDataWriter(DataWriterBuilder<S, D> builder, final State state)
       throws IOException {
     this.state = state;
@@ -96,13 +124,38 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
     this.closer = Closer.create();
     this.writerBuilder = builder;
     this.controlMessageHandler = new PartitionDataWriterMessageHandler();
-    this.partitionWriters = CacheBuilder.newBuilder().build(new CacheLoader<GenericRecord, DataWriter<D>>() {
+    if(builder.schema != null) {
+      this.state.setProp(WRITER_LATEST_SCHEMA, builder.getSchema());
+    }
+    long cacheExpiryInterval = this.state.getPropAsLong(PARTITIONED_WRITER_CACHE_TTL_SECONDS, DEFAULT_PARTITIONED_WRITER_CACHE_TTL_SECONDS);
+    log.debug("PartitionedDataWriter: Setting cache expiry interval to {} seconds", cacheExpiryInterval);
+
+    this.partitionWriters = CacheBuilder.newBuilder()
+        .expireAfterAccess(cacheExpiryInterval, TimeUnit.SECONDS)
+        .removalListener(new RemovalListener<GenericRecord, DataWriter<D>>() {
+      @Override
+      public void onRemoval(RemovalNotification<GenericRecord, DataWriter<D>> notification) {
+        synchronized (PartitionedDataWriter.this) {
+          if (notification.getValue() != null) {
+            try {
+              DataWriter<D> writer = notification.getValue();
+              totalRecordsFromEvictedWriters += writer.recordsWritten();
+              totalBytesFromEvictedWriters += writer.bytesWritten();
+              writer.close();
+            } catch (IOException e) {
+              log.error("Exception {} encountered when closing data writer on cache eviction", e);
+              //Should propagate the exception to avoid committing/publishing corrupt files.
+              throw new RuntimeException(e);
+            }
+          }
+        }
+      }
+    }).build(new CacheLoader<GenericRecord, DataWriter<D>>() {
       @Override
       public DataWriter<D> load(final GenericRecord key)
           throws Exception {
         /* wrap the data writer to allow the option to close the writer on flush */
-        return PartitionedDataWriter.this.closer
-            .register(new InstrumentedPartitionedDataWriterDecorator<>(
+        return new InstrumentedPartitionedDataWriterDecorator<>(
                 new CloseOnFlushWriterWrapper<D>(new Supplier<DataWriter<D>>() {
                   @Override
                   public DataWriter<D> get() {
@@ -112,25 +165,38 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
                       throw new RuntimeException("Error creating writer", e);
                     }
                   }
-                }, state), state, key));
+                }, state), state, key);
       }
     });
 
+    //Schedule a DataWriter cache clean up operation, since LoadingCache may keep the object
+    // in memory even after it has been evicted from the cache.
+    if (cacheExpiryInterval < Long.MAX_VALUE) {
+      this.cacheCleanUpExecutor = Executors.newSingleThreadScheduledExecutor(
+          ExecutorsUtils.newThreadFactory(Optional.of(log), Optional.of("CacheCleanupExecutor")));
+      this.cacheCleanUpExecutor.scheduleAtFixedRate(new Runnable() {
+        @Override
+        public void run() {
+          PartitionedDataWriter.this.partitionWriters.cleanUp();
+        }
+      }, 0, cacheExpiryInterval, TimeUnit.SECONDS);
+    }
+
     if (state.contains(ConfigurationKeys.WRITER_PARTITIONER_CLASS)) {
       Preconditions.checkArgument(builder instanceof PartitionAwareDataWriterBuilder, String
-              .format("%s was specified but the writer %s does not support partitioning.",
-                  ConfigurationKeys.WRITER_PARTITIONER_CLASS, builder.getClass().getCanonicalName()));
+          .format("%s was specified but the writer %s does not support partitioning.",
+              ConfigurationKeys.WRITER_PARTITIONER_CLASS, builder.getClass().getCanonicalName()));
 
       try {
         this.shouldPartition = true;
         this.builder = Optional.of(PartitionAwareDataWriterBuilder.class.cast(builder));
         this.partitioner = Optional.of(WriterPartitioner.class.cast(ConstructorUtils
-                .invokeConstructor(Class.forName(state.getProp(ConfigurationKeys.WRITER_PARTITIONER_CLASS)), state,
-                    builder.getBranches(), builder.getBranch())));
+            .invokeConstructor(Class.forName(state.getProp(ConfigurationKeys.WRITER_PARTITIONER_CLASS)), state,
+                builder.getBranches(), builder.getBranch())));
         Preconditions
             .checkArgument(this.builder.get().validatePartitionSchema(this.partitioner.get().partitionSchema()), String
-                    .format("Writer %s does not support schema from partitioner %s",
-                        builder.getClass().getCanonicalName(), this.partitioner.getClass().getCanonicalName()));
+                .format("Writer %s does not support schema from partitioner %s",
+                    builder.getClass().getCanonicalName(), this.partitioner.getClass().getCanonicalName()));
       } catch (ReflectiveOperationException roe) {
         throw new IOException(roe);
       }
@@ -184,7 +250,7 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   }
 
   @Override
-  public void commit()
+  public synchronized void commit()
       throws IOException {
     int writersCommitted = 0;
     for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
@@ -201,7 +267,7 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   }
 
   @Override
-  public void cleanup()
+  public synchronized void cleanup()
       throws IOException {
     int writersCleanedUp = 0;
     for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
@@ -209,7 +275,7 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
         entry.getValue().cleanup();
         writersCleanedUp++;
       } catch (Throwable throwable) {
-        log.error(String.format("Failed to cleanup writer for partition %s.", entry.getKey()));
+        log.error(String.format("Failed to cleanup writer for partition %s.", entry.getKey()), throwable);
       }
     }
     if (writersCleanedUp < this.partitionWriters.asMap().size()) {
@@ -218,34 +284,41 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   }
 
   @Override
-  public long recordsWritten() {
+  public synchronized long recordsWritten() {
     long totalRecords = 0;
     for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
       totalRecords += entry.getValue().recordsWritten();
     }
-    return totalRecords;
+    return totalRecords + this.totalRecordsFromEvictedWriters;
   }
 
   @Override
-  public long bytesWritten()
+  public synchronized long bytesWritten()
       throws IOException {
     long totalBytes = 0;
     for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
       totalBytes += entry.getValue().bytesWritten();
     }
-    return totalBytes;
+    return totalBytes + this.totalBytesFromEvictedWriters;
   }
 
   @Override
-  public void close()
+  public synchronized void close()
       throws IOException {
     try {
       serializePartitionInfoToState();
     } finally {
+      closeWritersInCache();
       this.closer.close();
     }
   }
 
+  private void closeWritersInCache() throws IOException {
+    for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
+      entry.getValue().close();
+    }
+  }
+
   private DataWriter<D> createPartitionWriter(GenericRecord partition)
       throws IOException {
     if (!this.builder.isPresent()) {
@@ -259,7 +332,7 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   }
 
   @Override
-  public State getFinalState() {
+  public synchronized State getFinalState() {
 
     State state = new State();
     try {
@@ -304,43 +377,6 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
     return this.isWatermarkCapable;
   }
 
-  @Override
-  public Map<String, CheckpointableWatermark> getCommittableWatermark() {
-    // The committable watermark from a collection of commitable and unacknowledged watermarks is the highest
-    // committable watermark that is less than the lowest unacknowledged watermark
-
-    WatermarkTracker watermarkTracker = new MultiWriterWatermarkTracker();
-    for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
-      if (entry.getValue() instanceof WatermarkAwareWriter) {
-        Map<String, CheckpointableWatermark> commitableWatermarks =
-            ((WatermarkAwareWriter) entry.getValue()).getCommittableWatermark();
-        if (!commitableWatermarks.isEmpty()) {
-          watermarkTracker.committedWatermarks(commitableWatermarks);
-        }
-
-        Map<String, CheckpointableWatermark> unacknowledgedWatermark =
-            ((WatermarkAwareWriter) entry.getValue()).getUnacknowledgedWatermark();
-        if (!unacknowledgedWatermark.isEmpty()) {
-          watermarkTracker.unacknowledgedWatermarks(unacknowledgedWatermark);
-        }
-      }
-    }
-    return watermarkTracker.getAllCommitableWatermarks(); //TODO: Change this to use List of committables instead
-  }
-
-  @Override
-  public Map<String, CheckpointableWatermark> getUnacknowledgedWatermark() {
-    WatermarkTracker watermarkTracker = new MultiWriterWatermarkTracker();
-    for (Map.Entry<GenericRecord, DataWriter<D>> entry : this.partitionWriters.asMap().entrySet()) {
-      Map<String, CheckpointableWatermark> unacknowledgedWatermark =
-          ((WatermarkAwareWriter) entry.getValue()).getUnacknowledgedWatermark();
-      if (!unacknowledgedWatermark.isEmpty()) {
-        watermarkTracker.unacknowledgedWatermarks(unacknowledgedWatermark);
-      }
-    }
-    return watermarkTracker.getAllUnacknowledgedWatermarks();
-  }
-
   @Override
   public ControlMessageHandler getMessageHandler() {
     return this.controlMessageHandler;
@@ -358,11 +394,18 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
       if (message instanceof MetadataUpdateControlMessage) {
         PartitionedDataWriter.this.writerBuilder.withSchema(((MetadataUpdateControlMessage) message)
             .getGlobalMetadata().getSchema());
+        state.setProp(WRITER_LATEST_SCHEMA, ((MetadataUpdateControlMessage) message)
+            .getGlobalMetadata().getSchema());
+      } else if (message instanceof FlushControlMessage){
+        //Add Partition info to state to report partition level lineage events on Flush
+        serializePartitionInfoToState();
       }
 
-      for (DataWriter writer : PartitionedDataWriter.this.partitionWriters.asMap().values()) {
-        ControlMessage cloned = (ControlMessage) cloner.getClone();
-        writer.getMessageHandler().handleMessage(cloned);
+      synchronized (PartitionedDataWriter.this) {
+        for (DataWriter writer : PartitionedDataWriter.this.partitionWriters.asMap().values()) {
+          ControlMessage cloned = (ControlMessage) cloner.getClone();
+          writer.getMessageHandler().handleMessage(cloned);
+        }
       }
 
       cloner.close();
@@ -372,7 +415,7 @@ public class PartitionedDataWriter<S, D> extends WriterWrapper<D> implements Fin
   /**
    * Get the serialized key to partitions info in {@link #state}
    */
-  private static String getPartitionsKey(int branchId) {
+  public static String getPartitionsKey(int branchId) {
     return String.format("writer.%d.partitions", branchId);
   }
 
