@@ -19,19 +19,36 @@ package org.apache.gobblin.data.management.copy;
 
 import java.io.InputStream;
 
-import lombok.AllArgsConstructor;
+import lombok.Builder;
+import lombok.NonNull;
 import lombok.Getter;
 
+import com.google.common.base.Optional;
+
+import org.apache.gobblin.data.management.copy.splitter.DistcpFileSplitter;
+
+
 /**
  * A wrapper to {@link InputStream} that represents an entity to be copied. The enclosed {@link CopyableFile} instance
  * contains file Metadata like permission, destination path etc. required by the writers and converters.
+ * The enclosed {@link DistcpFileSplitter.Split} object indicates whether the {@link InputStream} to be copied is a
+ * block of the {@link CopyableFile} or not. If it is present, the {@link InputStream} should already be at the start
+ * position of the specified split/block.
  */
-@AllArgsConstructor
 @Getter
 public class FileAwareInputStream {
 
   private CopyableFile file;
   private InputStream inputStream;
+  private Optional<DistcpFileSplitter.Split> split = Optional.absent();
+
+  @Builder(toBuilder = true)
+  public FileAwareInputStream(@NonNull CopyableFile file, @NonNull InputStream inputStream,
+      Optional<DistcpFileSplitter.Split> split) {
+    this.file = file;
+    this.inputStream = inputStream;
+    this.split = split == null ? Optional.<DistcpFileSplitter.Split>absent() : split;
+  }
 
   @Override
   public String toString() {
