@@ -27,15 +27,21 @@ import java.util.Map;
 
 import lombok.extern.slf4j.Slf4j;
 
+import org.apache.gobblin.data.management.copy.PreserveAttributes;
+import org.apache.gobblin.util.ConfigUtils;
+import org.apache.gobblin.util.filesystem.DataFileVersionStrategy;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ArrayListMultimap;
+import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Multimap;
+import com.typesafe.config.Config;
+import com.typesafe.config.ConfigFactory;
 
 import org.apache.gobblin.commit.CommitStep;
 import org.apache.gobblin.configuration.ConfigurationKeys;
@@ -79,11 +85,13 @@ public class CopyDataPublisher extends DataPublisher implements UnpublishedHandl
   public boolean isThreadSafe() {
     return this.getClass() == CopyDataPublisher.class;
   }
-
+  private final FileSystem srcFs;
   private final FileSystem fs;
   protected final EventSubmitter eventSubmitter;
   protected final RecoveryHelper recoveryHelper;
   protected final Optional<LineageInfo> lineageInfo;
+  protected final DataFileVersionStrategy srcDataFileVersionStrategy;
+  protected final DataFileVersionStrategy dstDataFileVersionStrategy;
 
   /**
    * Build a new {@link CopyDataPublisher} from {@link State}. The constructor expects the following to be set in the
@@ -119,6 +127,12 @@ public class CopyDataPublisher extends DataPublisher implements UnpublishedHandl
 
     this.recoveryHelper = new RecoveryHelper(this.fs, state);
     this.recoveryHelper.purgeOldPersistedFile();
+
+    Config config = ConfigUtils.propertiesToConfig(state.getProperties());
+
+    this.srcFs = HadoopUtils.getSourceFileSystem(state);
+    this.srcDataFileVersionStrategy = DataFileVersionStrategy.instantiateDataFileVersionStrategy(this.srcFs, config);
+    this.dstDataFileVersionStrategy = DataFileVersionStrategy.instantiateDataFileVersionStrategy(this.fs, config);
   }
 
   @Override
@@ -171,9 +185,38 @@ public class CopyDataPublisher extends DataPublisher implements UnpublishedHandl
     return datasetRoots;
   }
 
+  /**
+   * Unlike other preserving attributes of files (ownership, group, etc.), which is preserved in writer,
+   * some of the attributes have to be set during publish phase like ModTime,
+   * and versionStrategy (usually relevant to modTime as well), since they are subject to change with Publish(rename)
+   */
+  private void preserveFileAttrInPublisher(CopyableFile copyableFile) throws IOException {
+    // Preserving File ModTime, and set the access time to an initializing value when ModTime is declared to be preserved.
+    if (copyableFile.getPreserve().preserve(PreserveAttributes.Option.MOD_TIME)) {
+      fs.setTimes(copyableFile.getDestination(), copyableFile.getOriginTimestamp(), -1);
+    }
+
+    // Preserving File Version.
+    DataFileVersionStrategy srcVS = this.srcDataFileVersionStrategy;
+    DataFileVersionStrategy dstVS = this.dstDataFileVersionStrategy;
+
+    // Prefer to use copyableFile's specific version strategy
+    if (copyableFile.getDataFileVersionStrategy() != null) {
+      Config versionStrategyConfig = ConfigFactory.parseMap(ImmutableMap.of(
+          DataFileVersionStrategy.DATA_FILE_VERSION_STRATEGY_KEY, copyableFile.getDataFileVersionStrategy()));
+      srcVS = DataFileVersionStrategy.instantiateDataFileVersionStrategy(this.srcFs, versionStrategyConfig);
+      dstVS = DataFileVersionStrategy.instantiateDataFileVersionStrategy(this.fs, versionStrategyConfig);
+    }
+
+    if (copyableFile.getPreserve().preserve(PreserveAttributes.Option.VERSION)
+        && dstVS.hasCharacteristic(DataFileVersionStrategy.Characteristic.SETTABLE)) {
+      dstVS.setVersion(copyableFile.getDestination(),
+          srcVS.getVersion(copyableFile.getOrigin().getPath()));
+    }
+  }
+
   /**
    * Publish data for a {@link CopyableDataset}.
-   *
    */
   private void publishFileSet(CopyEntity.DatasetAndPartition datasetAndPartition,
       Collection<WorkUnitState> datasetWorkUnitStates) throws IOException {
@@ -219,6 +262,7 @@ public class CopyDataPublisher extends DataPublisher implements UnpublishedHandl
       CopyEntity copyEntity = CopySource.deserializeCopyEntity(wus);
       if (copyEntity instanceof CopyableFile) {
         CopyableFile copyableFile = (CopyableFile) copyEntity;
+        preserveFileAttrInPublisher(copyableFile);
         if (wus.getWorkingState() == WorkingState.COMMITTED) {
           CopyEventSubmitterHelper.submitSuccessfulFilePublish(this.eventSubmitter, copyableFile, wus);
           // Dataset Output path is injected in each copyableFile.
@@ -256,7 +300,8 @@ public class CopyDataPublisher extends DataPublisher implements UnpublishedHandl
     additionalMetadata.put(SlaEventKeys.DATASET_OUTPUT_PATH, fileSetRoot.or("Unknown"));
     CopyEventSubmitterHelper.submitSuccessfulDatasetPublish(this.eventSubmitter, datasetAndPartition,
         Long.toString(datasetOriginTimestamp), Long.toString(datasetUpstreamTimestamp), additionalMetadata);
-    }
+  }
+
 
   private static boolean hasCopyableFiles(Collection<WorkUnitState> workUnits) throws IOException {
     for (WorkUnitState wus : workUnits) {
