@@ -20,12 +20,14 @@ package org.apache.gobblin.data.management.copy.hive;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.fs.LocalFileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.ql.metadata.Partition;
 import org.apache.hadoop.hive.ql.metadata.Table;
@@ -39,6 +41,8 @@ import com.google.common.base.Optional;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 
+import lombok.AllArgsConstructor;
+
 import org.apache.gobblin.configuration.State;
 import org.apache.gobblin.data.management.copy.CopyEntity;
 import org.apache.gobblin.data.management.copy.entities.PostPublishStep;
@@ -77,6 +81,7 @@ public class HiveCopyEntityHelperTest {
 
     MultiTimingEvent timer = Mockito.mock(MultiTimingEvent.class);
     HiveCopyEntityHelper helper = Mockito.mock(HiveCopyEntityHelper.class);
+    Mockito.when(helper.isEnforceFileSizeMatch()).thenReturn(true);
     HiveTargetPathHelper targetPathHelper = Mockito.mock(HiveTargetPathHelper.class);
     Mockito.when(targetPathHelper
         .getTargetPath(Mockito.any(Path.class), Mockito.any(FileSystem.class), Mockito.any(Optional.class),
@@ -139,6 +144,7 @@ public class HiveCopyEntityHelperTest {
     HiveDataset hiveDataset = Mockito.mock(HiveDataset.class);
     MultiTimingEvent timer = Mockito.mock(MultiTimingEvent.class);
     HiveCopyEntityHelper helper = Mockito.mock(HiveCopyEntityHelper.class);
+    Mockito.when(helper.isEnforceFileSizeMatch()).thenReturn(true);
     HiveTargetPathHelper targetPathHelper = Mockito.mock(HiveTargetPathHelper.class);
     Mockito.when(helper.getDataset()).thenReturn(hiveDataset);
     Mockito.when(hiveDataset.getTable()).thenReturn(table);
@@ -200,6 +206,7 @@ public class HiveCopyEntityHelperTest {
     HiveDataset hiveDataset = Mockito.mock(HiveDataset.class);
     MultiTimingEvent timer = Mockito.mock(MultiTimingEvent.class);
     HiveCopyEntityHelper helper = Mockito.mock(HiveCopyEntityHelper.class);
+    Mockito.when(helper.isEnforceFileSizeMatch()).thenReturn(true);
     HiveTargetPathHelper targetPathHelper = Mockito.mock(HiveTargetPathHelper.class);
     Mockito.when(helper.getDataset()).thenReturn(hiveDataset);
     Mockito.when(hiveDataset.getTable()).thenReturn(table);
@@ -341,6 +348,23 @@ public class HiveCopyEntityHelperTest {
     Assert.assertEquals(HiveCopyEntityHelper.replacedPrefix(sourcePath, prefixTobeReplaced, prefixReplacement), expected);
   }
 
+  @Test
+  public void testAddMetadataToTargetTable() throws Exception {
+    org.apache.hadoop.hive.ql.metadata.Table meta_table =
+        new Table(Table.getEmptyTable("testDB", "testTable"));
+
+    Map<String, String> storageParams = new HashMap<>();
+    storageParams.put("path", "randomPath");
+    meta_table.getSd().getSerdeInfo().setParameters(storageParams);
+    HiveCopyEntityHelper.addMetadataToTargetTable(meta_table, new Path("newPath"), "testDB", 10L);
+    Assert.assertEquals(meta_table.getSd().getSerdeInfo().getParameters().get("path"), "newPath");
+
+    storageParams.clear();
+    meta_table.getSd().getSerdeInfo().setParameters(storageParams);
+    HiveCopyEntityHelper.addMetadataToTargetTable(meta_table, new Path("newPath"), "testDB", 10L);
+    Assert.assertFalse(meta_table.getSd().getSerdeInfo().getParameters().containsKey("path"));
+  }
+
   private boolean containsPath(Collection<FileStatus> statuses, Path path) {
     for (FileStatus status : statuses) {
       if (status.getPath().equals(path)) {
@@ -358,7 +382,7 @@ public class HiveCopyEntityHelperTest {
     Map<Path, FileStatus> paths;
 
     public TestLocationDescriptor(Map<Path, FileStatus> paths) {
-      super(null, null, null, null);
+      super(null, null, new TestLocationFs(paths), new Properties());
       this.paths = paths;
     }
 
@@ -369,4 +393,11 @@ public class HiveCopyEntityHelperTest {
     }
   }
 
+  @AllArgsConstructor
+  class TestLocationFs extends LocalFileSystem {
+    private Map<Path, FileStatus> paths;
+    public FileStatus getFileStatus(Path f) throws IOException {
+      return paths.get(f);
+    }
+  }
 }
