@@ -36,6 +36,7 @@ import org.apache.hadoop.hive.metastore.api.Partition;
 import org.apache.hadoop.hive.metastore.api.SerDeInfo;
 import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 import org.apache.hadoop.hive.metastore.api.Table;
+import org.apache.hadoop.hive.ql.io.avro.AvroContainerInputFormat;
 import org.apache.hadoop.hive.serde2.Deserializer;
 import org.apache.hadoop.hive.serde2.SerDeException;
 import org.apache.hadoop.hive.serde2.SerDeUtils;
@@ -188,7 +189,7 @@ public class HiveMetaStoreUtils {
     return hivePartition;
   }
 
-  private static Map<String, String> getParameters(State props) {
+  public static Map<String, String> getParameters(State props) {
     Map<String, String> parameters = Maps.newHashMap();
     if (props.contains(RUNTIME_PROPS)) {
       String runtimePropsString = props.getProp(RUNTIME_PROPS);
@@ -211,7 +212,11 @@ public class HiveMetaStoreUtils {
     State props = unit.getStorageProps();
     StorageDescriptor sd = new StorageDescriptor();
     sd.setParameters(getParameters(props));
-    sd.setCols(getFieldSchemas(unit));
+    //Treat AVRO and other formats differently. Details can be found in GOBBLIN-877
+    if (unit.isRegisterSchema() ||
+        (unit.getInputFormat().isPresent() && !unit.getInputFormat().get().equals(AvroContainerInputFormat.class.getName()))) {
+      sd.setCols(getFieldSchemas(unit));
+    }
     if (unit.getLocation().isPresent()) {
       sd.setLocation(unit.getLocation().get());
     }
@@ -237,7 +242,7 @@ public class HiveMetaStoreUtils {
     return sd;
   }
 
-  private static SerDeInfo getSerDeInfo(HiveRegistrationUnit unit) {
+  public static SerDeInfo getSerDeInfo(HiveRegistrationUnit unit) {
     State props = unit.getSerDeProps();
     SerDeInfo si = new SerDeInfo();
     si.setParameters(getParameters(props));
@@ -248,7 +253,7 @@ public class HiveMetaStoreUtils {
     return si;
   }
 
-  private static State getTableProps(Table table) {
+  public static State getTableProps(Table table) {
     State tableProps = new State();
     for (Map.Entry<String, String> entry : table.getParameters().entrySet()) {
       tableProps.setProp(entry.getKey(), entry.getValue());
