@@ -29,10 +29,6 @@ public class AzkabanExecuteFlowStatus extends AzkabanClientStatus<AzkabanExecute
     super(executeId);
   }
 
-  public AzkabanExecuteFlowStatus(String failMsg, Throwable throwable) {
-    super(failMsg, throwable);
-  }
-
   @Getter
   @AllArgsConstructor
   public static class ExecuteId {
