@@ -21,6 +21,7 @@ import java.io.IOException;
 import java.util.Properties;
 import java.util.concurrent.Future;
 
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.kafka.clients.producer.Callback;
 import org.apache.kafka.clients.producer.KafkaProducer;
 import org.apache.kafka.clients.producer.Producer;
@@ -39,6 +40,7 @@ import lombok.extern.slf4j.Slf4j;
 import org.I0Itec.zkclient.ZkClient;
 import org.I0Itec.zkclient.ZkConnection;
 
+import org.apache.gobblin.configuration.ConfigurationException;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.writer.AsyncDataWriter;
 import org.apache.gobblin.writer.WriteCallback;
@@ -54,10 +56,10 @@ import org.apache.gobblin.writer.WriteResponseMapper;
  *
  */
 @Slf4j
-public class Kafka09DataWriter<D> implements AsyncDataWriter<D> {
+public class Kafka09DataWriter<K, V> implements KafkaDataWriter<K, V> {
 
-  
-  private static final WriteResponseMapper<RecordMetadata> WRITE_RESPONSE_WRAPPER =
+
+  public static final WriteResponseMapper<RecordMetadata> WRITE_RESPONSE_WRAPPER =
       new WriteResponseMapper<RecordMetadata>() {
 
         @Override
@@ -81,8 +83,9 @@ public class Kafka09DataWriter<D> implements AsyncDataWriter<D> {
         }
       };
 
-  private final Producer<String, D> producer;
+  private final Producer<K, V> producer;
   private final String topic;
+  private final KafkaWriterCommonConfig commonConfig;
 
   public static Producer getKafkaProducer(Properties props) {
     Object producerObject = KafkaWriterHelper.getKafkaProducer(props);
@@ -96,14 +99,17 @@ public class Kafka09DataWriter<D> implements AsyncDataWriter<D> {
     }
   }
 
-  public Kafka09DataWriter(Properties props) {
+  public Kafka09DataWriter(Properties props)
+      throws ConfigurationException {
     this(getKafkaProducer(props), ConfigFactory.parseProperties(props));
   }
 
-  public Kafka09DataWriter(Producer producer, Config config) {
+  public Kafka09DataWriter(Producer producer, Config config)
+      throws ConfigurationException {
     this.topic = config.getString(KafkaWriterConfigurationKeys.KAFKA_TOPIC);
     provisionTopic(topic,config);
     this.producer = producer;
+    this.commonConfig = new KafkaWriterCommonConfig(config);
   }
 
   @Override
@@ -114,17 +120,31 @@ public class Kafka09DataWriter<D> implements AsyncDataWriter<D> {
   }
 
   @Override
-  public Future<WriteResponse> write(final D record, final WriteCallback callback) {
-    return new WriteResponseFuture<>(this.producer.send(new ProducerRecord<String, D>(topic, record), new Callback() {
-      @Override
-      public void onCompletion(final RecordMetadata metadata, Exception exception) {
-        if (exception != null) {
-          callback.onFailure(exception);
-        } else {
-          callback.onSuccess(WRITE_RESPONSE_WRAPPER.wrap(metadata));
-        }
-      }
-    }), WRITE_RESPONSE_WRAPPER);
+  public Future<WriteResponse> write(final V record, final WriteCallback callback) {
+    try {
+      Pair<K, V> keyValuePair = KafkaWriterHelper.getKeyValuePair(record, this.commonConfig);
+      return write(keyValuePair, callback);
+    } catch (Exception e) {
+      throw new RuntimeException("Failed to create a Kafka write request", e);
+    }
+  }
+
+  public Future<WriteResponse> write(Pair<K, V> keyValuePair, final WriteCallback callback) {
+    try {
+      return new WriteResponseFuture<>(this.producer
+          .send(new ProducerRecord<>(topic, keyValuePair.getKey(), keyValuePair.getValue()), new Callback() {
+            @Override
+            public void onCompletion(final RecordMetadata metadata, Exception exception) {
+              if (exception != null) {
+                callback.onFailure(exception);
+              } else {
+                callback.onSuccess(WRITE_RESPONSE_WRAPPER.wrap(metadata));
+              }
+            }
+          }), WRITE_RESPONSE_WRAPPER);
+    } catch (Exception e) {
+      throw new RuntimeException("Failed to create a Kafka write request", e);
+    }
   }
 
   @Override
@@ -132,35 +152,43 @@ public class Kafka09DataWriter<D> implements AsyncDataWriter<D> {
       throws IOException {
 	  this.producer.flush();
   }
-  
-  private void provisionTopic(String topicName,Config config) {
+
+  private void provisionTopic(String topicName, Config config) {
     String zooKeeperPropKey = KafkaWriterConfigurationKeys.CLUSTER_ZOOKEEPER;
-    if(!config.hasPath(zooKeeperPropKey)) {
-     log.debug("Topic "+topicName+" is configured without the partition and replication");
-     return;
+    if (!config.hasPath(zooKeeperPropKey)) {
+      log.debug("Topic " + topicName + " is configured without the partition and replication");
+      return;
     }
     String zookeeperConnect = config.getString(zooKeeperPropKey);
-    int sessionTimeoutMs = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.ZOOKEEPER_SESSION_TIMEOUT, KafkaWriterConfigurationKeys.ZOOKEEPER_SESSION_TIMEOUT_DEFAULT);
-    int connectionTimeoutMs = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.ZOOKEEPER_CONNECTION_TIMEOUT, KafkaWriterConfigurationKeys.ZOOKEEPER_CONNECTION_TIMEOUT_DEFAULT);
+    int sessionTimeoutMs = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.ZOOKEEPER_SESSION_TIMEOUT,
+        KafkaWriterConfigurationKeys.ZOOKEEPER_SESSION_TIMEOUT_DEFAULT);
+    int connectionTimeoutMs = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.ZOOKEEPER_CONNECTION_TIMEOUT,
+        KafkaWriterConfigurationKeys.ZOOKEEPER_CONNECTION_TIMEOUT_DEFAULT);
     // Note: You must initialize the ZkClient with ZKStringSerializer.  If you don't, then
     // createTopic() will only seem to work (it will return without error).  The topic will exist in
     // only ZooKeeper and will be returned when listing topics, but Kafka itself does not create the
     // topic.
-    ZkClient zkClient = new ZkClient(zookeeperConnect, sessionTimeoutMs, connectionTimeoutMs, ZKStringSerializer$.MODULE$);
+    ZkClient zkClient =
+        new ZkClient(zookeeperConnect, sessionTimeoutMs, connectionTimeoutMs, ZKStringSerializer$.MODULE$);
     // Security for Kafka was added in Kafka 0.9.0.0
     ZkUtils zkUtils = new ZkUtils(zkClient, new ZkConnection(zookeeperConnect), false);
-    int partitions = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.PARTITION_COUNT, KafkaWriterConfigurationKeys.PARTITION_COUNT_DEFAULT);
-    int replication = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.REPLICATION_COUNT, KafkaWriterConfigurationKeys.PARTITION_COUNT_DEFAULT);
-    Properties topicConfig = new Properties(); 
-    if(AdminUtils.topicExists(zkUtils, topicName)) {
-	   log.debug("Topic"+topicName+" already Exists with replication: "+replication+" and partitions :"+partitions);
-       return;
-    } 
-    try {
-       AdminUtils.createTopic(zkUtils, topicName, partitions, replication, topicConfig);
-    } catch (RuntimeException e) {
-       throw new RuntimeException(e);
-    }
-       log.info("Created Topic "+topicName+" with replication: "+replication+" and partitions :"+partitions);
+    int partitions = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.PARTITION_COUNT,
+        KafkaWriterConfigurationKeys.PARTITION_COUNT_DEFAULT);
+    int replication = ConfigUtils.getInt(config, KafkaWriterConfigurationKeys.REPLICATION_COUNT,
+        KafkaWriterConfigurationKeys.PARTITION_COUNT_DEFAULT);
+    Properties topicConfig = new Properties();
+    if (AdminUtils.topicExists(zkUtils, topicName)) {
+      log.debug("Topic {} already exists with replication: {} and partitions: {}", topicName, replication, partitions);
+      boolean deleteTopicIfExists = ConfigUtils.getBoolean(config, KafkaWriterConfigurationKeys.DELETE_TOPIC_IF_EXISTS,
+          KafkaWriterConfigurationKeys.DEFAULT_DELETE_TOPIC_IF_EXISTS);
+      if (!deleteTopicIfExists) {
+        return;
+      } else {
+        log.debug("Deleting topic {}", topicName);
+        AdminUtils.deleteTopic(zkUtils, topicName);
+      }
     }
+    AdminUtils.createTopic(zkUtils, topicName, partitions, replication, topicConfig);
+    log.info("Created topic {} with replication: {} and partitions : {}", topicName, replication, partitions);
+  }
 }
\ No newline at end of file
