@@ -17,6 +17,10 @@
 
 package org.apache.gobblin.kafka.writer;
 
+import org.apache.gobblin.types.AvroGenericRecordTypeMapper;
+import org.apache.gobblin.types.TypeMapper;
+
+
 /**
  * Configuration keys for a KafkaWriter.
  */
@@ -33,6 +37,14 @@ public class KafkaWriterConfigurationKeys {
   static final String FAILURE_ALLOWANCE_PCT_CONFIG = "writer.kafka.failureAllowancePercentage";
   static final double FAILURE_ALLOWANCE_PCT_DEFAULT = 20.0;
 
+  public static final String WRITER_KAFKA_KEYED_CONFIG = "writer.kafka.keyed";
+  public static final boolean WRITER_KAFKA_KEYED_DEFAULT = false;
+  public static final String WRITER_KAFKA_KEYFIELD_CONFIG = "writer.kafka.keyField";
+  public static final String WRITER_KAFKA_KEYFIELD_DEFAULT = null;
+  public static final String WRITER_KAFKA_TYPEMAPPERCLASS_CONFIG = "writer.kafka.typeMapperClass";
+  public static final String WRITER_KAFKA_TYPEMAPPERCLASS_DEFAULT = AvroGenericRecordTypeMapper.class.getName();
+  public static final String WRITER_KAFKA_VALUEFIELD_CONFIG = "writer.kafka.valueField";
+  public static final String WRITER_KAFKA_VALUEFIELD_DEFAULT = TypeMapper.FIELD_PATH_ALL;
 
   /**
    * Kafka producer configurations will be passed through as is as long as they are prefixed
@@ -44,7 +56,7 @@ public class KafkaWriterConfigurationKeys {
   /** Kafka producer scoped configuration keys go here **/
   static final String KEY_SERIALIZER_CONFIG = "key.serializer";
   static final String DEFAULT_KEY_SERIALIZER = "org.apache.kafka.common.serialization.StringSerializer";
-  static final String VALUE_SERIALIZER_CONFIG = "value.serializer";
+  public static final String VALUE_SERIALIZER_CONFIG = "value.serializer";
   static final String DEFAULT_VALUE_SERIALIZER = "org.apache.kafka.common.serialization.ByteArraySerializer";
   static final String CLIENT_ID_CONFIG = "client.id";
   static final String CLIENT_ID_DEFAULT = "gobblin";
@@ -56,8 +68,10 @@ public class KafkaWriterConfigurationKeys {
   public static final String CLUSTER_ZOOKEEPER = KAFKA_TOPIC_CONFIG + "zookeeper";
   static final String REPLICATION_COUNT = KAFKA_TOPIC_CONFIG + "replicationCount";
   static final int REPLICATION_COUNT_DEFAULT = 1;
-  static final String PARTITION_COUNT = KAFKA_TOPIC_CONFIG + "partitionCount";
+  public static final String PARTITION_COUNT = KAFKA_TOPIC_CONFIG + "partitionCount";
   static final int PARTITION_COUNT_DEFAULT = 1;
+  public static final String DELETE_TOPIC_IF_EXISTS = KAFKA_TOPIC_CONFIG + "deleteTopicIfExists";
+  static final Boolean DEFAULT_DELETE_TOPIC_IF_EXISTS = false;
   public static final String ZOOKEEPER_SESSION_TIMEOUT = CLUSTER_ZOOKEEPER + ".sto";
   static final int ZOOKEEPER_SESSION_TIMEOUT_DEFAULT = 10000; // 10 seconds
   public static final String ZOOKEEPER_CONNECTION_TIMEOUT = CLUSTER_ZOOKEEPER + ".cto";
