@@ -16,9 +16,24 @@
  */
 package org.apache.gobblin.source.extractor.extract.kafka;
 
+import java.net.URI;
+import java.net.URISyntaxException;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.List;
+import java.util.Properties;
+import java.util.Set;
+import java.util.stream.Collectors;
+
+import org.apache.commons.lang.StringUtils;
+import org.apache.hadoop.fs.Path;
+
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.typesafe.config.Config;
+
+import lombok.extern.slf4j.Slf4j;
+
 import org.apache.gobblin.config.client.ConfigClient;
 import org.apache.gobblin.config.client.ConfigClientUtils;
 import org.apache.gobblin.config.client.api.ConfigStoreFactoryDoesNotExistsException;
@@ -30,23 +45,16 @@ import org.apache.gobblin.kafka.client.GobblinKafkaConsumerClient;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.util.DatasetFilterUtils;
 import org.apache.gobblin.util.PathUtils;
-import java.net.URI;
-import java.net.URISyntaxException;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.List;
-import java.util.Properties;
-import java.util.Set;
-import java.util.stream.Collectors;
-import lombok.extern.slf4j.Slf4j;
-import org.apache.commons.lang.StringUtils;
-import org.apache.hadoop.fs.Path;
 
 
 @Slf4j
 public class ConfigStoreUtils {
+  /**
+   * Used as the grouping functionality in config-store to filter certain data nodes.
+   */
   public static final String GOBBLIN_CONFIG_TAGS_WHITELIST = "gobblin.config.tags.whitelist";
   public static final String GOBBLIN_CONFIG_TAGS_BLACKLIST = "gobblin.config.tags.blacklist";
+
   public static final String GOBBLIN_CONFIG_FILTER = "gobblin.config.filter";
   public static final String GOBBLIN_CONFIG_COMMONPATH = "gobblin.config.commonPath";
 
@@ -56,9 +64,9 @@ public class ConfigStoreUtils {
   public static Collection<URI> getTopicsURIFromConfigStore(ConfigClient configClient, Path tagUri, String filterString,
       Optional<Config> runtimeConfig) {
     try {
-      Collection<URI> importedBy = configClient.getImportedBy(new URI(tagUri.toString()), true, runtimeConfig);
+      Collection<URI> importedBy = configClient.getImportedBy(tagUri.toUri(), true, runtimeConfig);
       return importedBy.stream().filter((URI u) -> u.toString().contains(filterString)).collect(Collectors.toList());
-    } catch (URISyntaxException | ConfigStoreFactoryDoesNotExistsException | ConfigStoreCreationException e) {
+    } catch (ConfigStoreFactoryDoesNotExistsException | ConfigStoreCreationException e) {
       throw new Error(e);
     }
   }
@@ -81,15 +89,27 @@ public class ConfigStoreUtils {
 
   public static URI getUriStringForTopic(String topicName, String commonPath, String configStoreUri)
       throws URISyntaxException {
-    URI storeUri = new URI(configStoreUri);
-    Path path = PathUtils.mergePaths(new Path(storeUri.getPath()), PathUtils.mergePaths(new Path(commonPath), new Path(topicName)));
-    URI topicUri = new URI(storeUri.getScheme(), storeUri.getAuthority(), path.toString(), storeUri.getQuery(), storeUri.getFragment());
+    Path fullTopicPathInConfigStore = PathUtils.mergePaths(new Path(commonPath), new Path(topicName));
+    URI topicUri = getUriFromPath(fullTopicPathInConfigStore, configStoreUri);
     log.info("URI for topic is : " + topicUri.toString());
     return topicUri;
   }
 
+  /**
+   * Used when topic name needs to be fetched from Properties object, assuming we knew the topicKey.
+   */
   public static Optional<Config> getConfigForTopic(Properties properties, String topicKey, ConfigClient configClient) {
-    Optional<String> configStoreUri = getConfigStoreUri(properties);
+    Preconditions.checkArgument(properties.containsKey(topicKey), "Missing required property " + topicKey);
+    String topicName = properties.getProperty(topicKey);
+
+    return getConfigForTopicWithName(properties, topicName, configClient);
+  }
+
+  /**
+   * Used when topic name is known.
+   */
+  public static Optional<Config> getConfigForTopicWithName(Properties properties, String topicName, ConfigClient configClient) {
+    Optional<String> configStoreUri = ConfigStoreUtils.getConfigStoreUri(properties);
     Optional<Config> config = Optional.<Config>absent();
     if (!configStoreUri.isPresent()) {
       return config;
@@ -97,11 +117,9 @@ public class ConfigStoreUtils {
     try {
       Preconditions.checkArgument(properties.containsKey(GOBBLIN_CONFIG_COMMONPATH),
           "Missing required property " + GOBBLIN_CONFIG_COMMONPATH);
-      Preconditions.checkArgument(properties.containsKey(topicKey), "Missing required property " + topicKey);
-      String topicName = properties.getProperty(topicKey);
       String commonPath = properties.getProperty(GOBBLIN_CONFIG_COMMONPATH);
       config = Optional.fromNullable(
-          getConfig(configClient, getUriStringForTopic(topicName, commonPath, configStoreUri.get()),
+          ConfigStoreUtils.getConfig(configClient, ConfigStoreUtils.getUriStringForTopic(topicName, commonPath, configStoreUri.get()),
               ConfigClientUtils.getOptionalRuntimeConfig(properties)));
     } catch (URISyntaxException e) {
       log.error("Unable to get config", e);
@@ -124,10 +142,8 @@ public class ConfigStoreUtils {
   /**
    * Get topics from config store.
    * Topics will either be whitelisted or blacklisted using tag.
-   * After filtering out topics via tag, their config property is checked.
-   * For each shortlisted topic, config must contain either property topic.blacklist or topic.whitelist
    *
-   * If tags are not provided, it will return all topics
+   * If tags are not provided, it will return all topics.
    */
   public static List<KafkaTopic> getTopicsFromConfigStore(Properties properties, String configStoreUri,
       GobblinKafkaConsumerClient kafkaConsumerClient) {
@@ -141,33 +157,14 @@ public class ConfigStoreUtils {
     Optional<Config> runtimeConfig = ConfigClientUtils.getOptionalRuntimeConfig(properties);
 
     if (properties.containsKey(GOBBLIN_CONFIG_TAGS_WHITELIST)) {
-      Preconditions.checkArgument(properties.containsKey(GOBBLIN_CONFIG_FILTER),
-          "Missing required property " + GOBBLIN_CONFIG_FILTER);
-      String filterString = properties.getProperty(GOBBLIN_CONFIG_FILTER);
-      Path whiteListTagUri = PathUtils.mergePaths(new Path(configStoreUri),
-          new Path(properties.getProperty(GOBBLIN_CONFIG_TAGS_WHITELIST)));
-      List<String> whitelistedTopics = new ArrayList<>();
-      ConfigStoreUtils.getTopicsURIFromConfigStore(configClient, whiteListTagUri, filterString, runtimeConfig)
-          .stream()
-          .filter((URI u) -> ConfigUtils.getBoolean(ConfigStoreUtils.getConfig(configClient, u, runtimeConfig),
-              KafkaSource.TOPIC_WHITELIST, false))
-          .forEach(((URI u) -> whitelistedTopics.add(ConfigStoreUtils.getTopicNameFromURI(u))));
-
+      List<String> whitelistedTopics = getListOfTopicNamesByFilteringTag(properties, configClient, runtimeConfig,
+          configStoreUri, GOBBLIN_CONFIG_TAGS_WHITELIST);
       return allTopics.stream()
           .filter((KafkaTopic p) -> whitelistedTopics.contains(p.getName()))
           .collect(Collectors.toList());
     } else if (properties.containsKey(GOBBLIN_CONFIG_TAGS_BLACKLIST)) {
-      Preconditions.checkArgument(properties.containsKey(GOBBLIN_CONFIG_FILTER),
-          "Missing required property " + GOBBLIN_CONFIG_FILTER);
-      String filterString = properties.getProperty(GOBBLIN_CONFIG_FILTER);
-      Path blackListTagUri = PathUtils.mergePaths(new Path(configStoreUri),
-          new Path(properties.getProperty(GOBBLIN_CONFIG_TAGS_BLACKLIST)));
-      List<String> blacklistedTopics = new ArrayList<>();
-      ConfigStoreUtils.getTopicsURIFromConfigStore(configClient, blackListTagUri, filterString, runtimeConfig)
-          .stream()
-          .filter((URI u) -> ConfigUtils.getBoolean(ConfigStoreUtils.getConfig(configClient, u, runtimeConfig),
-              KafkaSource.TOPIC_BLACKLIST, false))
-          .forEach(((URI u) -> blacklistedTopics.add(ConfigStoreUtils.getTopicNameFromURI(u))));
+      List<String> blacklistedTopics = getListOfTopicNamesByFilteringTag(properties, configClient, runtimeConfig,
+          configStoreUri, GOBBLIN_CONFIG_TAGS_BLACKLIST);
       return allTopics.stream()
           .filter((KafkaTopic p) -> !blacklistedTopics.contains(p.getName()))
           .collect(Collectors.toList());
@@ -177,12 +174,59 @@ public class ConfigStoreUtils {
     }
   }
 
+  /**
+   * Using the tag feature provided by Config-Store for grouping, getting a list of topics (case-sensitive,
+   * need to be matched with what would be returned from kafka broker) tagged by the tag value specified
+   * in job configuration.
+   */
+  public static List<String> getListOfTopicNamesByFilteringTag(Properties properties, ConfigClient configClient,
+      Optional<Config> runtimeConfig, String configStoreUri, String tagConfName) {
+    Preconditions.checkArgument(properties.containsKey(GOBBLIN_CONFIG_FILTER),
+        "Missing required property " + GOBBLIN_CONFIG_FILTER);
+    String filterString = properties.getProperty(GOBBLIN_CONFIG_FILTER);
+
+    Path tagUri = new Path("/");
+    try {
+      tagUri = new Path(getUriFromPath(new Path(properties.getProperty(tagConfName)), configStoreUri));
+    } catch (URISyntaxException ue) {
+      log.error("Cannot construct a Tag URI due to the exception:", ue);
+    }
+
+    List<String> taggedTopics = new ArrayList<>();
+    ConfigStoreUtils.getTopicsURIFromConfigStore(configClient, tagUri, filterString, runtimeConfig)
+        .forEach(((URI u) -> taggedTopics.add(ConfigStoreUtils.getTopicNameFromURI(u))));
+
+    return taggedTopics;
+  }
+
+  /**
+   * Construct the URI for a Config-Store node given a path.
+   * The implementation will be based on scheme, while the signature of this method will not be subject to
+   * different implementation.
+   *
+   * The implementation will be different since Fs-based config-store simply append dataNode's path in the end,
+   * while ivy-based config-store will require query to store those information.
+   *
+   * @param path The relative path of a node inside Config-Store.
+   * @param configStoreUri The config store URI.
+   * @return The URI to inspect a data node represented by path inside Config Store.
+   * @throws URISyntaxException
+   */
+  private static URI getUriFromPath(Path path, String configStoreUri) throws URISyntaxException {
+    URI storeUri = new URI(configStoreUri);
+    return new URI(storeUri.getScheme(), storeUri.getAuthority(),
+        PathUtils.mergePaths(new Path(storeUri.getPath()), path).toString(), storeUri.getQuery(), storeUri.getFragment());
+
+  }
+
   /**
    * Shortlist topics from config store based on whitelist/blacklist tags and
    * add it to {@param whitelist}/{@param blacklist}
    *
    * If tags are not provided, blacklist and whitelist won't be modified
+   * @deprecated Since this method contains implementation-specific way to construct TagURI inside Config-Store.
    */
+  @Deprecated
   public static void setTopicsFromConfigStore(Properties properties, Set<String> blacklist, Set<String> whitelist,
       final String _blacklistTopicKey, final String _whitelistTopicKey) {
     Optional<String> configStoreUri = getConfigStoreUri(properties);
