@@ -19,16 +19,19 @@ package org.apache.gobblin.service;
 
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.Collection;
 import java.util.Properties;
+import java.util.stream.Collectors;
 
 import org.apache.commons.lang.StringUtils;
 
-import com.google.common.collect.Maps;
-import com.linkedin.data.template.StringMap;
+import com.codahale.metrics.MetricRegistry;
 import com.linkedin.restli.common.ComplexResourceKey;
 import com.linkedin.restli.common.EmptyRecord;
 import com.linkedin.restli.common.HttpStatus;
+import com.linkedin.restli.common.PatchRequest;
 import com.linkedin.restli.server.CreateResponse;
+import com.linkedin.restli.server.RestLiServiceException;
 import com.linkedin.restli.server.UpdateResponse;
 import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
@@ -38,64 +41,50 @@ import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.config.ConfigBuilder;
 import org.apache.gobblin.configuration.ConfigurationKeys;
+import org.apache.gobblin.instrumented.Instrumented;
+import org.apache.gobblin.metrics.ContextAwareMeter;
+import org.apache.gobblin.metrics.MetricContext;
+import org.apache.gobblin.metrics.ServiceMetricNames;
 import org.apache.gobblin.runtime.api.FlowSpec;
+import org.apache.gobblin.runtime.api.FlowSpecSearchObject;
 import org.apache.gobblin.runtime.api.SpecNotFoundException;
 import org.apache.gobblin.runtime.spec_catalog.FlowCatalog;
+import org.apache.gobblin.util.ConfigUtils;
+
 
 /**
  * A {@link FlowConfigsResourceHandler} that handles Restli locally.
  */
 @Slf4j
 public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandler {
+  public static final Schedule NEVER_RUN_CRON_SCHEDULE = new Schedule().setCronSchedule("0 0 0 ? 1 1 2050");
   @Getter
   protected FlowCatalog flowCatalog;
+  protected final ContextAwareMeter createFlow;
+  protected final ContextAwareMeter deleteFlow;
+  protected final ContextAwareMeter runImmediatelyFlow;
+
   public FlowConfigResourceLocalHandler(FlowCatalog flowCatalog) {
     this.flowCatalog = flowCatalog;
+    MetricContext metricContext = Instrumented.getMetricContext(ConfigUtils.configToState(ConfigFactory.empty()), getClass());
+    this.createFlow = metricContext.contextAwareMeter(
+        MetricRegistry.name(ServiceMetricNames.GOBBLIN_SERVICE_PREFIX, ServiceMetricNames.CREATE_FLOW_METER));
+    this.deleteFlow = metricContext.contextAwareMeter(
+        MetricRegistry.name(ServiceMetricNames.GOBBLIN_SERVICE_PREFIX, ServiceMetricNames.DELETE_FLOW_METER));
+    this.runImmediatelyFlow = metricContext.contextAwareMeter(
+        MetricRegistry.name(ServiceMetricNames.GOBBLIN_SERVICE_PREFIX, ServiceMetricNames.RUN_IMMEDIATELY_FLOW_METER));
   }
 
   /**
-   * Get flow config
+   * Get flow config given a {@link FlowId}
    */
   public FlowConfig getFlowConfig(FlowId flowId) throws FlowConfigLoggedException {
     log.info("[GAAS-REST] Get called with flowGroup {} flowName {}", flowId.getFlowGroup(), flowId.getFlowName());
 
     try {
-      URI flowCatalogURI = new URI("gobblin-flow", null, "/", null, null);
-      URI flowUri = new URI(flowCatalogURI.getScheme(), flowCatalogURI.getAuthority(),
-          "/" + flowId.getFlowGroup() + "/" + flowId.getFlowName(), null, null);
-      FlowSpec spec = (FlowSpec) flowCatalog.getSpec(flowUri);
-      FlowConfig flowConfig = new FlowConfig();
-      Properties flowProps = spec.getConfigAsProperties();
-      Schedule schedule = null;
-
-      if (flowProps.containsKey(ConfigurationKeys.JOB_SCHEDULE_KEY)) {
-        schedule = new Schedule();
-        schedule.setCronSchedule(flowProps.getProperty(ConfigurationKeys.JOB_SCHEDULE_KEY));
-      }
-      if (flowProps.containsKey(ConfigurationKeys.JOB_TEMPLATE_PATH)) {
-        flowConfig.setTemplateUris(flowProps.getProperty(ConfigurationKeys.JOB_TEMPLATE_PATH));
-      } else if (spec.getTemplateURIs().isPresent()) {
-        flowConfig.setTemplateUris(StringUtils.join(spec.getTemplateURIs().get(), ","));
-      } else {
-        flowConfig.setTemplateUris("NA");
-      }
-      if (schedule != null) {
-        if (flowProps.containsKey(ConfigurationKeys.FLOW_RUN_IMMEDIATELY)) {
-          schedule.setRunImmediately(Boolean.valueOf(flowProps.getProperty(ConfigurationKeys.FLOW_RUN_IMMEDIATELY)));
-        }
-
-        flowConfig.setSchedule(schedule);
-      }
-
-      // remove keys that were injected as part of flowSpec creation
-      flowProps.remove(ConfigurationKeys.JOB_SCHEDULE_KEY);
-      flowProps.remove(ConfigurationKeys.JOB_TEMPLATE_PATH);
-
-      StringMap flowPropsAsStringMap = new StringMap();
-      flowPropsAsStringMap.putAll(Maps.fromProperties(flowProps));
-
-      return flowConfig.setId(new FlowId().setFlowGroup(flowId.getFlowGroup()).setFlowName(flowId.getFlowName()))
-          .setProperties(flowPropsAsStringMap);
+      URI flowUri = FlowSpec.Utils.createFlowSpecUri(flowId);
+      FlowSpec spec = (FlowSpec) flowCatalog.getSpecs(flowUri);
+      return FlowSpec.Utils.toFlowConfig(spec);
     } catch (URISyntaxException e) {
       throw new FlowConfigLoggedException(HttpStatus.S_400_BAD_REQUEST, "bad URI " + flowId.getFlowName(), e);
     } catch (SpecNotFoundException e) {
@@ -103,11 +92,38 @@ public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandle
     }
   }
 
+  /**
+   * Get flow config given a {@link FlowSpecSearchObject}
+   * @return all the {@link FlowConfig}s that satisfy the {@link FlowSpecSearchObject}
+   */
+  public Collection<FlowConfig> getFlowConfig(FlowSpecSearchObject flowSpecSearchObject) throws FlowConfigLoggedException {
+    log.info("[GAAS-REST] Get called with flowSpecSearchObject {}", flowSpecSearchObject);
+    return flowCatalog.getSpecs(flowSpecSearchObject).stream().map(FlowSpec.Utils::toFlowConfig).collect(Collectors.toList());
+  }
+
+  /**
+   * Get all flow configs
+   */
+  public Collection<FlowConfig> getAllFlowConfigs() {
+    log.info("[GAAS-REST] GetAll called");
+    return flowCatalog.getAllSpecs().stream().map(FlowSpec.Utils::toFlowConfig).collect(Collectors.toList());
+  }
+
   /**
    * Add flowConfig locally and trigger all listeners iff @param triggerListener is set to true
    */
   public CreateResponse createFlowConfig(FlowConfig flowConfig, boolean triggerListener) throws FlowConfigLoggedException {
     log.info("[GAAS-REST] Create called with flowGroup " + flowConfig.getId().getFlowGroup() + " flowName " + flowConfig.getId().getFlowName());
+    this.createFlow.mark();
+    if (!flowConfig.hasSchedule() || StringUtils.isEmpty(flowConfig.getSchedule().getCronSchedule())) {
+      this.runImmediatelyFlow.mark();
+    }
+
+    if (flowConfig.hasExplain()) {
+      //Return Error if FlowConfig has explain set. Explain request is only valid for v2 FlowConfig.
+      return new CreateResponse(new RestLiServiceException(HttpStatus.S_400_BAD_REQUEST, "FlowConfig with explain not supported."));
+    }
+
     FlowSpec flowSpec = createFlowSpecForConfig(flowConfig);
     // Existence of a flow spec in the flow catalog implies that the flow is currently running.
     // If the new flow spec has a schedule we should allow submission of the new flow to accept the new schedule.
@@ -138,10 +154,24 @@ public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandle
           "flowName and flowGroup cannot be changed in update", null);
     }
 
+    // Carry forward the requester list property since it is added at time of creation
+    FlowConfig originalFlowConfig = getFlowConfig(flowId);
+    flowConfig.getProperties().put(RequesterService.REQUESTER_LIST, originalFlowConfig.getProperties().get(RequesterService.REQUESTER_LIST));
+
+    if (isUnscheduleRequest(flowConfig)) {
+      // flow config is not changed if it is just a request to un-schedule
+      originalFlowConfig.setSchedule(NEVER_RUN_CRON_SCHEDULE);
+      flowConfig = originalFlowConfig;
+    }
+
     this.flowCatalog.put(createFlowSpecForConfig(flowConfig), triggerListener);
     return new UpdateResponse(HttpStatus.S_200_OK);
   }
 
+  private boolean isUnscheduleRequest(FlowConfig flowConfig) {
+    return Boolean.parseBoolean(flowConfig.getProperties().getOrDefault(ConfigurationKeys.FLOW_UNSCHEDULE_KEY, "false"));
+  }
+
   /**
    * Update flowConfig locally and trigger all listeners
    */
@@ -149,16 +179,22 @@ public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandle
     return updateFlowConfig(flowId, flowConfig, true);
   }
 
+  @Override
+  public UpdateResponse partialUpdateFlowConfig(FlowId flowId, PatchRequest<FlowConfig> flowConfigPatch) throws FlowConfigLoggedException {
+    throw new UnsupportedOperationException("Partial update only supported by GobblinServiceFlowConfigResourceHandler");
+  }
+
   /**
    * Delete flowConfig locally and trigger all listeners iff @param triggerListener is set to true
    */
   public UpdateResponse deleteFlowConfig(FlowId flowId, Properties header, boolean triggerListener) throws FlowConfigLoggedException {
 
     log.info("[GAAS-REST] Delete called with flowGroup {} flowName {}", flowId.getFlowGroup(), flowId.getFlowName());
+    this.deleteFlow.mark();
     URI flowUri = null;
 
     try {
-      flowUri = createFlowSpecUri(flowId);
+      flowUri = FlowSpec.Utils.createFlowSpecUri(flowId);
       this.flowCatalog.remove(flowUri, header, triggerListener);
       return new UpdateResponse(HttpStatus.S_200_OK);
     } catch (URISyntaxException e) {
@@ -173,13 +209,6 @@ public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandle
     return deleteFlowConfig(flowId, header, true);
   }
 
-  public static URI createFlowSpecUri (FlowId flowId) throws URISyntaxException {
-    URI flowCatalogURI = new URI("gobblin-flow", null, "/", null, null);
-    URI flowUri = new URI(flowCatalogURI.getScheme(), flowCatalogURI.getAuthority(),
-        "/" + flowId.getFlowGroup() + "/" + flowId.getFlowName(), null, null);
-    return flowUri;
-  }
-
   /**
    * Build a {@link FlowSpec} from a {@link FlowConfig}
    * @param flowConfig flow configuration
@@ -201,13 +230,38 @@ public class FlowConfigResourceLocalHandler implements FlowConfigsResourceHandle
       // If it is not a run-once job, we should not add flow execution id here,
       // because execution id is generated for every scheduled execution of the flow and cannot be materialized to
       // the flow catalog. In this case, this id is added during flow compilation.
-      configBuilder.addPrimitive(ConfigurationKeys.FLOW_EXECUTION_ID_KEY, String.valueOf(System.currentTimeMillis()));
+      String flowExecutionId;
+      if (flowConfig.getProperties().containsKey(ConfigurationKeys.FLOW_EXECUTION_ID_KEY)) {
+        flowExecutionId = flowConfig.getProperties().get(ConfigurationKeys.FLOW_EXECUTION_ID_KEY);
+        // FLOW_EXECUTION_ID may already be present in FlowSpec in cases
+        // where the FlowSpec is forwarded by a slave to the master.
+        log.info("Using the existing flowExecutionId {} for {},{}", flowExecutionId, flowConfig.getId().getFlowGroup(), flowConfig.getId().getFlowName());
+      } else {
+        flowExecutionId = String.valueOf(System.currentTimeMillis());
+        log.info("Created a flowExecutionId {} for {},{}", flowExecutionId, flowConfig.getId().getFlowGroup(), flowConfig.getId().getFlowName());
+      }
+      flowConfig.getProperties().put(ConfigurationKeys.FLOW_EXECUTION_ID_KEY, flowExecutionId);
+      configBuilder.addPrimitive(ConfigurationKeys.FLOW_EXECUTION_ID_KEY, flowExecutionId);
+    }
+
+    if (flowConfig.hasExplain()) {
+      configBuilder.addPrimitive(ConfigurationKeys.FLOW_EXPLAIN_KEY, flowConfig.isExplain());
     }
 
     Config config = configBuilder.build();
-    //We allow for flowConfig to be in HOCON format. We first convert the StringMap object to a String object and
-    // then use ConfigFactory#parseString() to parse the HOCON string.
-    Config configWithFallback = config.withFallback(ConfigFactory.parseString(flowConfig.getProperties().toString()).resolve());
+
+    Config configWithFallback;
+    // We first attempt to process the REST.li request as a HOCON string. If the request is not a valid HOCON string
+    // (e.g. when certain special characters such as ":" or "*" are not properly escaped), we catch the Typesafe ConfigException and
+    // fallback to assuming that values are literal strings.
+    try {
+      // We first convert the StringMap object to a String object and then use ConfigFactory#parseString() to parse the
+      // HOCON string.
+      configWithFallback = config.withFallback(ConfigFactory.parseString(flowConfig.getProperties().toString()).resolve());
+    } catch (Exception e) {
+      configWithFallback = config.withFallback(ConfigFactory.parseMap(flowConfig.getProperties()));
+    }
+
     try {
       URI templateURI = new URI(flowConfig.getTemplateUris());
       return FlowSpec.builder().withConfig(configWithFallback).withTemplate(templateURI).build();
