@@ -476,7 +476,9 @@ public class JobContext implements Closeable {
 
       if (!IteratorExecutor.verifyAllSuccessful(result)) {
         this.jobState.setState(JobState.RunningState.FAILED);
-        throw new IOException("Failed to commit dataset state for some dataset(s) of job " + this.jobId);
+        String errMsg = "Failed to commit dataset state for some dataset(s) of job " + this.jobId;
+        this.jobState.setJobFailureMessage(errMsg);
+        throw new IOException(errMsg);
       }
     } catch (InterruptedException exc) {
       throw new IOException(exc);
