@@ -28,6 +28,8 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.function.Predicate;
+import java.util.stream.Collectors;
 
 import org.apache.commons.lang3.BooleanUtils;
 import org.slf4j.Logger;
@@ -45,7 +47,6 @@ import com.google.common.io.Closer;
 import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 
-import javax.annotation.Nullable;
 import lombok.NoArgsConstructor;
 
 import org.apache.gobblin.Constructs;
@@ -74,6 +75,7 @@ import org.apache.gobblin.publisher.SingleTaskDataPublisher;
 import org.apache.gobblin.qualitychecker.row.RowLevelPolicyCheckResults;
 import org.apache.gobblin.qualitychecker.row.RowLevelPolicyChecker;
 import org.apache.gobblin.records.RecordStreamProcessor;
+import org.apache.gobblin.runtime.api.TaskEventMetadataGenerator;
 import org.apache.gobblin.runtime.fork.AsynchronousFork;
 import org.apache.gobblin.runtime.fork.Fork;
 import org.apache.gobblin.runtime.fork.SynchronousFork;
@@ -85,10 +87,10 @@ import org.apache.gobblin.source.extractor.StreamingExtractor;
 import org.apache.gobblin.state.ConstructState;
 import org.apache.gobblin.stream.RecordEnvelope;
 import org.apache.gobblin.util.ConfigUtils;
+import org.apache.gobblin.util.TaskEventMetadataUtils;
 import org.apache.gobblin.writer.AcknowledgableWatermark;
 import org.apache.gobblin.writer.DataWriter;
 import org.apache.gobblin.writer.FineGrainedWatermarkTracker;
-import org.apache.gobblin.writer.MultiWriterWatermarkManager;
 import org.apache.gobblin.writer.TrackerBasedWatermarkManager;
 import org.apache.gobblin.writer.WatermarkAwareWriter;
 import org.apache.gobblin.writer.WatermarkManager;
@@ -150,13 +152,13 @@ public class Task implements TaskIFace {
   private final InstrumentedExtractorBase extractor;
   private final RowLevelPolicyChecker rowChecker;
   private final ExecutionModel taskMode;
-  private final String watermarkingStrategy;
   private final Optional<WatermarkManager> watermarkManager;
   private final Optional<FineGrainedWatermarkTracker> watermarkTracker;
   private final Optional<WatermarkStorage> watermarkStorage;
   private final List<RecordStreamProcessor<?,?,?,?>> recordStreamProcessors;
 
   private final Closer closer;
+  private final TaskEventMetadataGenerator taskEventMetadataGenerator;
 
   private long startTime;
   private volatile long lastRecordPulledTimestampMillis;
@@ -165,7 +167,7 @@ public class Task implements TaskIFace {
   private final AtomicBoolean shutdownRequested;
   private volatile long shutdownRequestedTime = Long.MAX_VALUE;
   private final CountDownLatch shutdownLatch;
-  private Future<?> taskFuture;
+  protected Future<?> taskFuture;
 
   /**
    * Instantiate a new {@link Task}.
@@ -235,9 +237,6 @@ public class Task implements TaskIFace {
     this.shutdownLatch = new CountDownLatch(1);
 
     // Setup Streaming constructs
-
-    this.watermarkingStrategy = "FineGrain"; // TODO: Configure
-
     if (isStreamingTask()) {
       Extractor underlyingExtractor = this.taskContext.getRawSourceExtractor();
       if (!(underlyingExtractor instanceof StreamingExtractor)) {
@@ -260,23 +259,17 @@ public class Task implements TaskIFace {
       long commitIntervalMillis = ConfigUtils.getLong(config,
           TaskConfigurationKeys.STREAMING_WATERMARK_COMMIT_INTERVAL_MILLIS,
           TaskConfigurationKeys.DEFAULT_STREAMING_WATERMARK_COMMIT_INTERVAL_MILLIS);
-      if (watermarkingStrategy.equals("FineGrain")) { // TODO: Configure
-        this.watermarkTracker = Optional.of(this.closer.register(new FineGrainedWatermarkTracker(config)));
-        this.watermarkManager = Optional.of((WatermarkManager) this.closer.register(
-            new TrackerBasedWatermarkManager(this.watermarkStorage.get(), this.watermarkTracker.get(),
-                commitIntervalMillis, Optional.of(this.LOG))));
+      this.watermarkTracker = Optional.of(this.closer.register(new FineGrainedWatermarkTracker(config)));
+      this.watermarkManager = Optional.of((WatermarkManager) this.closer.register(
+          new TrackerBasedWatermarkManager(this.watermarkStorage.get(), this.watermarkTracker.get(),
+              commitIntervalMillis, Optional.of(this.LOG))));
 
-      } else {
-        // writer-based watermarking
-        this.watermarkManager = Optional.of((WatermarkManager) this.closer.register(
-            new MultiWriterWatermarkManager(this.watermarkStorage.get(), commitIntervalMillis, Optional.of(this.LOG))));
-        this.watermarkTracker = Optional.absent();
-      }
     } else {
       this.watermarkManager = Optional.absent();
       this.watermarkTracker = Optional.absent();
       this.watermarkStorage = Optional.absent();
     }
+    this.taskEventMetadataGenerator = TaskEventMetadataUtils.getTaskEventMetadataGenerator(taskState);
   }
 
   /**
@@ -369,7 +362,7 @@ public class Task implements TaskIFace {
       } else {
         new StreamModelTaskRunner(this, this.taskState, this.closer, this.taskContext, this.extractor,
             this.converter, this.recordStreamProcessors, this.rowChecker, this.taskExecutor, this.taskMode, this.shutdownRequested,
-            this.watermarkTracker, this.watermarkManager, this.watermarkStorage, this.forks, this.watermarkingStrategy).run();
+            this.watermarkTracker, this.watermarkManager, this.watermarkStorage, this.forks).run();
       }
 
       LOG.info("Extracted " + this.recordsPulled + " data records");
@@ -378,8 +371,17 @@ public class Task implements TaskIFace {
       this.taskState.setProp(ConfigurationKeys.EXTRACTOR_ROWS_EXTRACTED, this.recordsPulled);
       this.taskState.setProp(ConfigurationKeys.EXTRACTOR_ROWS_EXPECTED, extractor.getExpectedRecordCount());
 
+      // If there are folks not successfully being executed, get the aggregated exceptions and throw RuntimeException.
       if (!this.forks.keySet().stream().map(Optional::get).allMatch(Fork::isSucceeded)) {
-        throw new RuntimeException("Some forks failed.");
+        List<Integer> failedForksId = this.forks.keySet().stream().map(Optional::get).
+            filter(not(Fork::isSucceeded)).map(x -> x.getIndex()).collect(Collectors.toList());
+        ForkThrowableHolder holder = Task.getForkThrowableHolder(this.taskState.getTaskBroker());
+
+        Exception e = null;
+        if (!holder.isEmpty()) {
+          e = holder.getAggregatedException(failedForksId, this.taskId);
+        }
+        throw e == null ? new RuntimeException("Some forks failed") : new RuntimeException("Forks failed with exception:", e);
       }
 
       //TODO: Move these to explicit shutdown phase
@@ -404,6 +406,13 @@ public class Task implements TaskIFace {
     }
   }
 
+  /**
+   * TODO: Remove this method after Java-11 as JDK offers similar built-in solution.
+   */
+  public static <T> Predicate<T> not(Predicate<T> t) {
+    return t.negate();
+  }
+
   @Deprecated
   private void runSynchronousModel() throws Exception {
     // Get the fork operator. By default IdentityForkOperator is used with a single branch.
@@ -435,7 +444,7 @@ public class Task implements TaskIFace {
           AsynchronousFork fork = closer.register(
               new AsynchronousFork(this.taskContext, schema instanceof Copyable ? ((Copyable) schema).copy() : schema,
                   branches, i, this.taskMode));
-          configureStreamingFork(fork, watermarkingStrategy);
+          configureStreamingFork(fork);
           // Run the Fork
           this.forks.put(Optional.<Fork>of(fork), Optional.<Future<?>>of(this.taskExecutor.submit(fork)));
         } else {
@@ -446,10 +455,11 @@ public class Task implements TaskIFace {
       SynchronousFork fork = closer.register(
           new SynchronousFork(this.taskContext, schema instanceof Copyable ? ((Copyable) schema).copy() : schema,
               branches, 0, this.taskMode));
-      configureStreamingFork(fork, watermarkingStrategy);
+      configureStreamingFork(fork);
       this.forks.put(Optional.<Fork>of(fork), Optional.<Future<?>> of(this.taskExecutor.submit(fork)));
     }
 
+    LOG.info("Task mode streaming = " + isStreamingTask());
     if (isStreamingTask()) {
 
       // Start watermark manager and tracker
@@ -463,7 +473,7 @@ public class Task implements TaskIFace {
 
       RecordEnvelope recordEnvelope;
       // Extract, convert, and fork one source record at a time.
-      while (!shutdownRequested() && (recordEnvelope = extractor.readRecordEnvelope()) != null) {
+      while ((recordEnvelope = extractor.readRecordEnvelope()) != null) {
         onRecordExtract();
         AcknowledgableWatermark ackableWatermark = new AcknowledgableWatermark(recordEnvelope.getWatermark());
         if (watermarkTracker.isPresent()) {
@@ -474,6 +484,9 @@ public class Task implements TaskIFace {
               ackableWatermark.incrementAck());
         }
         ackableWatermark.ack();
+        if (shutdownRequested()) {
+          extractor.shutdown();
+        }
       }
     } else {
       RecordEnvelope record;
@@ -496,6 +509,9 @@ public class Task implements TaskIFace {
             throw new RuntimeException(e);
           }
         }
+        if (shutdownRequested()) {
+          extractor.shutdown();
+        }
       }
     }
 
@@ -526,14 +542,13 @@ public class Task implements TaskIFace {
     }
   }
 
-  protected void configureStreamingFork(Fork fork, String watermarkingStrategy) throws IOException {
+  protected void configureStreamingFork(Fork fork) throws IOException {
     if (isStreamingTask()) {
       DataWriter forkWriter = fork.getWriter();
-      if (forkWriter instanceof WatermarkAwareWriter) {
-        if (watermarkingStrategy.equals("WriterBased")) {
-          ((MultiWriterWatermarkManager) this.watermarkManager.get()).registerWriter((WatermarkAwareWriter) forkWriter);
-        }
-      } else {
+      boolean isWaterMarkAwareWriter = (forkWriter instanceof WatermarkAwareWriter)
+          && ((WatermarkAwareWriter) forkWriter).isWatermarkCapable();
+
+      if (!isWaterMarkAwareWriter) {
         String errorMessage = String.format("The Task is configured to run in continuous mode, "
             + "but the writer %s is not a WatermarkAwareWriter", forkWriter.getClass().getName());
         LOG.error(errorMessage);
@@ -556,6 +571,7 @@ public class Task implements TaskIFace {
     FailureEventBuilder failureEvent = new FailureEventBuilder(FAILED_TASK_EVENT);
     failureEvent.setRootCause(t);
     failureEvent.addMetadata(TASK_STATE, this.taskState.toString());
+    failureEvent.addAdditionalMetadata(this.taskEventMetadataGenerator.getMetadata(this.taskState, failureEvent.getName()));
     failureEvent.submit(taskContext.getTaskMetrics().getMetricContext());
   }
 
@@ -973,9 +989,12 @@ public class Task implements TaskIFace {
   protected void submitTaskCommittedEvent() {
     MetricContext taskMetricContext = TaskMetrics.get(this.taskState).getMetricContext();
     EventSubmitter eventSubmitter = new EventSubmitter.Builder(taskMetricContext, "gobblin.runtime.task").build();
-    eventSubmitter.submit(TaskEvent.TASK_COMMITTED_EVENT_NAME, ImmutableMap
+    Map<String, String> metadataMap = Maps.newHashMap();
+    metadataMap.putAll(this.taskEventMetadataGenerator.getMetadata(this.taskState, TaskEvent.TASK_COMMITTED_EVENT_NAME));
+    metadataMap.putAll(ImmutableMap
         .of(TaskEvent.METADATA_TASK_ID, this.taskId, TaskEvent.METADATA_TASK_ATTEMPT_ID,
             this.taskState.getTaskAttemptId().or("")));
+    eventSubmitter.submit(TaskEvent.TASK_COMMITTED_EVENT_NAME, metadataMap);
   }
 
   /**
