@@ -40,10 +40,15 @@ import org.apache.hadoop.mapreduce.Counter;
 import org.apache.hadoop.mapreduce.CounterGroup;
 import org.apache.hadoop.mapreduce.Counters;
 import org.apache.hadoop.mapreduce.Job;
+import org.apache.hadoop.mapreduce.JobStatus;
 import org.apache.hadoop.mapreduce.Mapper;
+import org.apache.hadoop.mapreduce.TaskAttemptID;
+import org.apache.hadoop.mapreduce.TaskType;
 import org.apache.hadoop.mapreduce.filecache.DistributedCache;
 import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
+import org.apache.hadoop.mapreduce.lib.map.WrappedMapper;
 import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
+import org.apache.hadoop.mapreduce.task.MapContextImpl;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -59,6 +64,8 @@ import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 import com.typesafe.config.ConfigValue;
 
+import lombok.extern.slf4j.Slf4j;
+
 import org.apache.gobblin.broker.SharedResourcesBrokerFactory;
 import org.apache.gobblin.broker.gobblin_scopes.GobblinScopeTypes;
 import org.apache.gobblin.broker.gobblin_scopes.JobScopeInstance;
@@ -66,11 +73,18 @@ import org.apache.gobblin.broker.iface.SharedResourcesBroker;
 import org.apache.gobblin.commit.CommitStep;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.configuration.DynamicConfigGenerator;
+import org.apache.gobblin.configuration.State;
+import org.apache.gobblin.fsm.FiniteStateMachine;
 import org.apache.gobblin.metastore.FsStateStore;
 import org.apache.gobblin.metastore.StateStore;
 import org.apache.gobblin.metrics.GobblinMetrics;
+import org.apache.gobblin.metrics.MultiReporterException;
 import org.apache.gobblin.metrics.Tag;
+import org.apache.gobblin.metrics.event.CountEventBuilder;
+import org.apache.gobblin.metrics.event.JobEvent;
+import org.apache.gobblin.metrics.event.JobStateEventBuilder;
 import org.apache.gobblin.metrics.event.TimingEvent;
+import org.apache.gobblin.metrics.reporter.util.MetricReportUtils;
 import org.apache.gobblin.password.PasswordManager;
 import org.apache.gobblin.runtime.AbstractJobLauncher;
 import org.apache.gobblin.runtime.DynamicConfigGeneratorFactory;
@@ -82,6 +96,9 @@ import org.apache.gobblin.runtime.TaskExecutor;
 import org.apache.gobblin.runtime.TaskState;
 import org.apache.gobblin.runtime.TaskStateCollectorService;
 import org.apache.gobblin.runtime.TaskStateTracker;
+import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine;
+import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine.JobFSMState;
+import org.apache.gobblin.runtime.job.GobblinJobFiniteStateMachine.StateType;
 import org.apache.gobblin.runtime.util.JobMetrics;
 import org.apache.gobblin.runtime.util.MetricGroup;
 import org.apache.gobblin.source.workunit.MultiWorkUnit;
@@ -92,8 +109,7 @@ import org.apache.gobblin.util.JobConfigurationUtils;
 import org.apache.gobblin.util.JobLauncherUtils;
 import org.apache.gobblin.util.ParallelRunner;
 import org.apache.gobblin.util.SerializationUtils;
-
-
+import org.apache.gobblin.util.reflection.RestrictedFieldAccessingUtils;
 /**
  * An implementation of {@link JobLauncher} that launches a Gobblin job as a Hadoop MR job.
  *
@@ -108,8 +124,12 @@ import org.apache.gobblin.util.SerializationUtils;
  *
  * @author Yinan Li
  */
+@Slf4j
 public class MRJobLauncher extends AbstractJobLauncher {
 
+  private static final String INTERRUPT_JOB_FILE_NAME = "_INTERRUPT_JOB";
+  private static final String GOBBLIN_JOB_INTERRUPT_PATH_KEY = "gobblin.jobInterruptPath";
+
   private static final Logger LOG = LoggerFactory.getLogger(MRJobLauncher.class);
 
   private static final String JOB_NAME_PREFIX = "Gobblin-";
@@ -118,15 +138,25 @@ public class MRJobLauncher extends AbstractJobLauncher {
   private static final String FILES_DIR_NAME = "_files";
   static final String INPUT_DIR_NAME = "input";
   private static final String OUTPUT_DIR_NAME = "output";
-  private static final String WORK_UNIT_LIST_FILE_EXTENSION = ".wulist";
   private static final String SERIALIZE_PREVIOUS_WORKUNIT_STATES_KEY = "MRJobLauncher.serializePreviousWorkunitStates";
   private static final boolean DEFAULT_SERIALIZE_PREVIOUS_WORKUNIT_STATES = true;
 
+  /**
+   * In MR-mode, it is necessary to enable customized progress if speculative execution is required.
+   */
+  private static final String ENABLED_CUSTOMIZED_PROGRESS = "MRJobLauncher.enabledCustomizedProgress";
+
   // Configuration that make uploading of jar files more reliable,
   // since multiple Gobblin Jobs are sharing the same jar directory.
   private static final int MAXIMUM_JAR_COPY_RETRY_TIMES_DEFAULT = 5;
   private static final int WAITING_TIME_ON_IMCOMPLETE_UPLOAD = 3000;
 
+  public static final String MR_TYPE_KEY = ConfigurationKeys.METRICS_CONFIGURATIONS_PREFIX + "mr.type";
+  public static final String MAPPER_TASK_NUM_KEY = ConfigurationKeys.METRICS_CONFIGURATIONS_PREFIX + "reporting.mapper.task.num";
+  public static final String MAPPER_TASK_ATTEMPT_NUM_KEY = ConfigurationKeys.METRICS_CONFIGURATIONS_PREFIX + "reporting.mapper.task.attempt.num";
+  public static final String REDUCER_TASK_NUM_KEY = ConfigurationKeys.METRICS_CONFIGURATIONS_PREFIX + "reporting.reducer.task.num";
+  public static final String REDUCER_TASK_ATTEMPT_NUM_KEY = ConfigurationKeys.METRICS_CONFIGURATIONS_PREFIX + "reporting.reducer.task.attempt.num";
+
   private static final Splitter SPLITTER = Splitter.on(',').omitEmptyStrings().trimResults();
 
   private final Configuration conf;
@@ -148,6 +178,8 @@ public class MRJobLauncher extends AbstractJobLauncher {
   private final StateStore<TaskState> taskStateStore;
 
   private final int jarFileMaximumRetry;
+  private final Path interruptPath;
+  private final GobblinJobFiniteStateMachine fsm;
 
   public MRJobLauncher(Properties jobProps) throws Exception {
     this(jobProps, null);
@@ -170,6 +202,9 @@ public class MRJobLauncher extends AbstractJobLauncher {
       throws Exception {
     super(jobProps, metadataTags);
 
+    this.fsm = GobblinJobFiniteStateMachine.builder().jobState(jobContext.getJobState())
+        .interruptGracefully(this::interruptGracefully).killJob(this::killJob).build();
+
     this.conf = conf;
     // Put job configuration properties into the Hadoop configuration so they are available in the mappers
     JobConfigurationUtils.putPropertiesIntoConfiguration(this.jobProps, this.conf);
@@ -185,6 +220,7 @@ public class MRJobLauncher extends AbstractJobLauncher {
     this.mrJobDir = new Path(
         new Path(this.jobProps.getProperty(ConfigurationKeys.MR_JOB_ROOT_DIR_KEY), this.jobContext.getJobName()),
         this.jobContext.getJobId());
+    this.interruptPath = new Path(this.mrJobDir, INTERRUPT_JOB_FILE_NAME);
     if (this.fs.exists(this.mrJobDir)) {
       LOG.warn("Job working directory already exists for job " + this.jobContext.getJobName());
       this.fs.delete(this.mrJobDir, true);
@@ -220,6 +256,9 @@ public class MRJobLauncher extends AbstractJobLauncher {
             jobProps.getProperty(ConfigurationKeys.MAXIMUM_JAR_COPY_RETRY_TIMES_KEY))
             : MAXIMUM_JAR_COPY_RETRY_TIMES_DEFAULT;
 
+    // One of the most common user mistakes is mis-configuring the FileSystem scheme (e.g. file versus hdfs)
+    log.info("Configured fs:{}", fs);
+    log.debug("Configuration: {}", conf);
     startCancellationExecutor();
   }
 
@@ -246,38 +285,71 @@ public class MRJobLauncher extends AbstractJobLauncher {
     JobState jobState = this.jobContext.getJobState();
 
     try {
+      CountEventBuilder countEventBuilder = new CountEventBuilder(JobEvent.WORK_UNITS_CREATED, workUnits.size());
+      this.eventSubmitter.submit(countEventBuilder);
+      LOG.info("Emitting WorkUnitsCreated Count: " + countEventBuilder.getCount());
+
       prepareHadoopJob(workUnits);
 
       // Start the output TaskState collector service
       this.taskStateCollectorService.startAsync().awaitRunning();
 
       LOG.info("Launching Hadoop MR job " + this.job.getJobName());
-      this.job.submit();
-      this.hadoopJobSubmitted = true;
+      try (FiniteStateMachine<JobFSMState>.Transition t = this.fsm.startTransition(this.fsm.getEndStateForType(StateType.RUNNING))) {
+        try {
+          this.job.submit();
+        } catch (Throwable exc) {
+          t.changeEndState(this.fsm.getEndStateForType(StateType.FAILED));
+          throw exc;
+        }
+        this.hadoopJobSubmitted = true;
 
-      // Set job tracking URL to the Hadoop job tracking URL if it is not set yet
-      if (!jobState.contains(ConfigurationKeys.JOB_TRACKING_URL_KEY)) {
-        jobState.setProp(ConfigurationKeys.JOB_TRACKING_URL_KEY, this.job.getTrackingURL());
+        // Set job tracking URL to the Hadoop job tracking URL if it is not set yet
+        if (!jobState.contains(ConfigurationKeys.JOB_TRACKING_URL_KEY)) {
+          jobState.setProp(ConfigurationKeys.JOB_TRACKING_URL_KEY, this.job.getTrackingURL());
+        }
+        /**
+         * Catch {@link UnallowedTransitionException} only, leaving other failure while submitting MR jobs to catch
+         * block afterwards.
+         */
+      } catch (FiniteStateMachine.UnallowedTransitionException unallowed) {
+        LOG.error("Cannot start MR job.", unallowed);
       }
 
-      TimingEvent mrJobRunTimer = this.eventSubmitter.getTimingEvent(TimingEvent.RunJobTimings.MR_JOB_RUN);
-      LOG.info(String.format("Waiting for Hadoop MR job %s to complete", this.job.getJobID()));
-      this.job.waitForCompletion(true);
-      mrJobRunTimer.stop(ImmutableMap.of("hadoopMRJobId", this.job.getJobID().toString()));
+      if (this.fsm.getCurrentState().getStateType().equals(StateType.RUNNING)) {
+        TimingEvent mrJobRunTimer = this.eventSubmitter.getTimingEvent(TimingEvent.RunJobTimings.MR_JOB_RUN);
+        LOG.info(String.format("Waiting for Hadoop MR job %s to complete", this.job.getJobID()));
 
-      if (this.cancellationRequested) {
-        // Wait for the cancellation execution if it has been requested
-        synchronized (this.cancellationExecution) {
-          if (this.cancellationExecuted) {
-            return;
-          }
-        }
+        this.job.waitForCompletion(true);
+        this.fsm.transitionIfAllowed(fsm.getEndStateForType(StateType.SUCCESS));
+
+        mrJobRunTimer.stop(ImmutableMap.of("hadoopMRJobId", this.job.getJobID().toString()));
+      }
+
+      if (this.fsm.getCurrentState().getStateType().equals(StateType.CANCELLED)) {
+        return;
       }
 
       // Create a metrics set for this job run from the Hadoop counters.
       // The metrics set is to be persisted to the metrics store later.
       countersToMetrics(JobMetrics.get(jobName, this.jobProps.getProperty(ConfigurationKeys.JOB_ID_KEY)));
+    } catch (Throwable t) {
+      throw new RuntimeException("The MR job cannot be submitted due to:", t);
     } finally {
+      JobStateEventBuilder eventBuilder = new JobStateEventBuilder(JobStateEventBuilder.MRJobState.MR_JOB_STATE);
+
+      if (!hadoopJobSubmitted) {
+        eventBuilder.jobTrackingURL = "";
+        eventBuilder.status = JobStateEventBuilder.Status.FAILED;
+      } else {
+        eventBuilder.jobTrackingURL = this.job.getTrackingURL();
+        eventBuilder.status = JobStateEventBuilder.Status.SUCCEEDED;
+        if (this.job.getJobState() != JobStatus.State.SUCCEEDED) {
+          eventBuilder.status = JobStateEventBuilder.Status.FAILED;
+        }
+      }
+      this.eventSubmitter.submit(eventBuilder);
+
       // The last iteration of output TaskState collecting will run when the collector service gets stopped
       this.taskStateCollectorService.stopAsync().awaitTerminated();
       cleanUpWorkingDirectory();
@@ -286,20 +358,54 @@ public class MRJobLauncher extends AbstractJobLauncher {
 
   @Override
   protected void executeCancellation() {
-    try {
-      if (this.hadoopJobSubmitted && !this.job.isComplete()) {
-        LOG.info("Killing the Hadoop MR job for job " + this.jobContext.getJobId());
-        this.job.killJob();
-        // Collect final task states.
-        this.taskStateCollectorService.stopAsync().awaitTerminated();
+    try (FiniteStateMachine<JobFSMState>.Transition transition =
+        this.fsm.startTransition(this.fsm.getEndStateForType(StateType.CANCELLED))) {
+      if (transition.getStartState().getStateType().equals(StateType.RUNNING)) {
+        try {
+          killJob();
+        } catch (IOException ioe) {
+          LOG.error("Failed to kill the Hadoop MR job for job " + this.jobContext.getJobId());
+          transition.changeEndState(this.fsm.getEndStateForType(StateType.FAILED));
+        }
       }
-    } catch (IllegalStateException ise) {
-      LOG.error("The Hadoop MR job has not started for job " + this.jobContext.getJobId());
-    } catch (IOException ioe) {
-      LOG.error("Failed to kill the Hadoop MR job for job " + this.jobContext.getJobId());
+    } catch (GobblinJobFiniteStateMachine.FailedTransitionCallbackException exc) {
+      exc.getTransition().switchEndStateToErrorState();
+      exc.getTransition().closeWithoutCallbacks();
+    } catch (FiniteStateMachine.UnallowedTransitionException | InterruptedException exc) {
+      LOG.error("Failed to cancel job " + this.jobContext.getJobId(), exc);
     }
   }
 
+  /**
+   * Attempt a gracious interruption of the running job
+   */
+  private void interruptGracefully() throws IOException {
+    LOG.info("Attempting graceful interruption of job " + this.jobContext.getJobId());
+
+    this.fs.createNewFile(this.interruptPath);
+
+    long waitTimeStart = System.currentTimeMillis();
+    while (!this.job.isComplete() && System.currentTimeMillis() < waitTimeStart + 30 * 1000) {
+      try {
+        Thread.sleep(1000);
+      } catch (InterruptedException ie) {
+        break;
+      }
+    }
+
+    if (!this.job.isComplete()) {
+      LOG.info("Interrupted job did not shut itself down after timeout. Killing job.");
+      this.job.killJob();
+    }
+  }
+
+  private void killJob() throws IOException {
+    LOG.info("Killing the Hadoop MR job for job " + this.jobContext.getJobId());
+    this.job.killJob();
+    // Collect final task states.
+    this.taskStateCollectorService.stopAsync().awaitTerminated();
+  }
+
   /**
    * Add dependent jars and files.
    */
@@ -381,6 +487,8 @@ public class MRJobLauncher extends AbstractJobLauncher {
           Integer.parseInt(this.jobProps.getProperty(ConfigurationKeys.MR_JOB_MAX_MAPPERS_KEY)));
     }
 
+    this.job.getConfiguration().set(GOBBLIN_JOB_INTERRUPT_PATH_KEY, this.interruptPath.toString());
+
     mrJobSetupTimer.stop();
   }
 
@@ -389,6 +497,11 @@ public class MRJobLauncher extends AbstractJobLauncher {
         props.getProperty(JobContext.MAP_SPECULATIVE, ConfigurationKeys.DEFAULT_ENABLE_MR_SPECULATIVE_EXECUTION));
   }
 
+  static boolean isCustomizedProgressReportEnabled(Properties properties) {
+    return properties.containsKey(ENABLED_CUSTOMIZED_PROGRESS)
+        && Boolean.parseBoolean(properties.getProperty(ENABLED_CUSTOMIZED_PROGRESS));
+  }
+
   @VisibleForTesting
   static void serializeJobState(FileSystem fs, Path mrJobDir, Configuration conf, JobState jobState, Job job)
       throws IOException {
@@ -611,16 +724,32 @@ public class MRJobLauncher extends AbstractJobLauncher {
     private ServiceManager serviceManager;
     private Optional<JobMetrics> jobMetrics = Optional.absent();
     private boolean isSpeculativeEnabled;
+    private boolean customizedProgressEnabled;
     private final JobState jobState = new JobState();
+    private CustomizedProgresser customizedProgresser;
+
+    private static final String CUSTOMIZED_PROGRESSER_FACTORY_CLASS = "customizedProgresser.factoryClass";
+    private static final String DEFAULT_CUSTOMIZED_PROGRESSER_FACTORY_CLASS =
+        "org.apache.gobblin.runtime.mapreduce.CustomizedProgresserBase$BaseFactory";
 
     // A list of WorkUnits (flattened for MultiWorkUnits) to be run by this mapper
     private final List<WorkUnit> workUnits = Lists.newArrayList();
 
     @Override
     protected void setup(Context context) {
+      final State gobblinJobState = HadoopUtils.getStateFromConf(context.getConfiguration());
+      TaskAttemptID taskAttemptID = context.getTaskAttemptID();
+
       try (Closer closer = Closer.create()) {
-        this.isSpeculativeEnabled =
-            isSpeculativeExecutionEnabled(HadoopUtils.getStateFromConf(context.getConfiguration()).getProperties());
+        // Default for customizedProgressEnabled is false.
+        this.customizedProgressEnabled = isCustomizedProgressReportEnabled(gobblinJobState.getProperties());
+        this.isSpeculativeEnabled = isSpeculativeExecutionEnabled(gobblinJobState.getProperties());
+
+        String factoryClassName = gobblinJobState.getProperties().getProperty(
+            CUSTOMIZED_PROGRESSER_FACTORY_CLASS, DEFAULT_CUSTOMIZED_PROGRESSER_FACTORY_CLASS);
+        this.customizedProgresser = Class.forName(factoryClassName).asSubclass(CustomizedProgresser.Factory.class)
+            .newInstance().createCustomizedProgresser(context);
+
         this.fs = FileSystem.get(context.getConfiguration());
         this.taskStateStore =
             new FsStateStore<>(this.fs, FileOutputFormat.getOutputPath(context).toUri().getPath(), TaskState.class);
@@ -638,8 +767,8 @@ public class MRJobLauncher extends AbstractJobLauncher {
         if (!foundStateFile) {
           throw new IOException("Job state file not found.");
         }
-      } catch (IOException ioe) {
-        throw new RuntimeException("Failed to setup the mapper task", ioe);
+      } catch (IOException | ReflectiveOperationException e) {
+        throw new RuntimeException("Failed to setup the mapper task", e);
       }
 
 
@@ -654,6 +783,24 @@ public class MRJobLauncher extends AbstractJobLauncher {
       for (Map.Entry<String, ConfigValue> entry : dynamicConfig.entrySet()) {
         this.jobState.setProp(entry.getKey(), entry.getValue().unwrapped().toString());
         configuration.set(entry.getKey(), entry.getValue().unwrapped().toString());
+        gobblinJobState.setProp(entry.getKey(), entry.getValue().unwrapped().toString());
+      }
+
+      // add some more MR task related configs
+
+      String[] tokens = taskAttemptID.toString().split("_");
+      TaskType taskType = taskAttemptID.getTaskType();
+      gobblinJobState.setProp(MR_TYPE_KEY, taskType.name());
+
+      // a task attempt id should be like 'attempt_1592863931636_2371636_m_000003_4'
+      if (tokens.length == 6) {
+        if (taskType.equals(TaskType.MAP)) {
+          gobblinJobState.setProp(MAPPER_TASK_NUM_KEY, tokens[tokens.length - 2]);
+          gobblinJobState.setProp(MAPPER_TASK_ATTEMPT_NUM_KEY, tokens[tokens.length - 1]);
+        } else if (taskType.equals(TaskType.REDUCE)) {
+          gobblinJobState.setProp(REDUCER_TASK_NUM_KEY, tokens[tokens.length - 2]);
+          gobblinJobState.setProp(REDUCER_TASK_ATTEMPT_NUM_KEY, tokens[tokens.length - 1]);
+        }
       }
 
       this.taskExecutor = new TaskExecutor(configuration);
@@ -667,24 +814,46 @@ public class MRJobLauncher extends AbstractJobLauncher {
       }
 
       // Setup and start metrics reporting if metric reporting is enabled
-      if (Boolean.valueOf(
-          configuration.get(ConfigurationKeys.METRICS_ENABLED_KEY, ConfigurationKeys.DEFAULT_METRICS_ENABLED))) {
+      if (Boolean.parseBoolean(configuration.get(ConfigurationKeys.METRICS_ENABLED_KEY, ConfigurationKeys.DEFAULT_METRICS_ENABLED))) {
         this.jobMetrics = Optional.of(JobMetrics.get(this.jobState));
-        this.jobMetrics.get()
-            .startMetricReportingWithFileSuffix(HadoopUtils.getStateFromConf(configuration),
-                context.getTaskAttemptID().toString());
+        try {
+          this.jobMetrics.get().startMetricReportingWithFileSuffix(gobblinJobState, taskAttemptID.toString());
+        } catch (MultiReporterException ex) {
+          //Fail the task if metric/event reporting failure is configured to be fatal.
+          boolean isMetricReportingFailureFatal = configuration.getBoolean(ConfigurationKeys.GOBBLIN_TASK_METRIC_REPORTING_FAILURE_FATAL,
+              ConfigurationKeys.DEFAULT_GOBBLIN_TASK_METRIC_REPORTING_FAILURE_FATAL);
+          boolean isEventReportingFailureFatal = configuration.getBoolean(ConfigurationKeys.GOBBLIN_TASK_EVENT_REPORTING_FAILURE_FATAL,
+                  ConfigurationKeys.DEFAULT_GOBBLIN_TASK_EVENT_REPORTING_FAILURE_FATAL);
+          if (MetricReportUtils.shouldThrowException(LOG, ex, isMetricReportingFailureFatal, isEventReportingFailureFatal)) {
+            throw new RuntimeException(ex);
+          }
+        }
       }
     }
 
     @Override
     public void run(Context context) throws IOException, InterruptedException {
       this.setup(context);
+
+      Path interruptPath = new Path(context.getConfiguration().get(GOBBLIN_JOB_INTERRUPT_PATH_KEY));
+      if (this.fs.exists(interruptPath)) {
+        LOG.info(String.format("Found interrupt path %s indicating the driver has interrupted the job, aborting mapper.", interruptPath));
+        return;
+      }
+
       GobblinMultiTaskAttempt gobblinMultiTaskAttempt = null;
       try {
         // De-serialize and collect the list of WorkUnits to run
         while (context.nextKeyValue()) {
           this.map(context.getCurrentKey(), context.getCurrentValue(), context);
         }
+
+        // org.apache.hadoop.util.Progress.complete will set the progress to 1.0f eventually so we don't have to
+        // set it in finally block.
+        if (customizedProgressEnabled) {
+          setProgressInMapper(customizedProgresser.getCustomizedProgress(), context);
+        }
+
         GobblinMultiTaskAttempt.CommitPolicy multiTaskAttemptCommitPolicy =
             isSpeculativeEnabled ? GobblinMultiTaskAttempt.CommitPolicy.CUSTOMIZED
                 : GobblinMultiTaskAttempt.CommitPolicy.IMMEDIATE;
@@ -701,7 +870,13 @@ public class MRJobLauncher extends AbstractJobLauncher {
         gobblinMultiTaskAttempt =
             GobblinMultiTaskAttempt.runWorkUnits(this.jobState.getJobId(), context.getTaskAttemptID().toString(),
                 this.jobState, this.workUnits, this.taskStateTracker, this.taskExecutor, this.taskStateStore,
-                multiTaskAttemptCommitPolicy, jobBroker);
+                multiTaskAttemptCommitPolicy, jobBroker, (gmta) -> {
+                  try {
+                    return this.fs.exists(interruptPath);
+                  } catch (IOException ioe) {
+                    return false;
+                  }
+                });
 
         if (this.isSpeculativeEnabled) {
           LOG.info("will not commit in task attempt");
@@ -760,5 +935,26 @@ public class MRJobLauncher extends AbstractJobLauncher {
         this.workUnits.add(workUnit);
       }
     }
+
+    /**
+     * Setting progress within implementation of {@link Mapper} for reporting progress.
+     * Gobblin (when running in MR mode) used to report progress only in {@link GobblinWorkUnitsInputFormat} while
+     * deserializing {@link WorkUnit} in MapReduce job. In that scenario, whenever workunit is deserialized (but not yet
+     * executed) the progress will be reported as 1.0f. This could implicitly disable the feature of speculative-execution
+     * provided by MR-framework as the latter is looking at the progress to determine if speculative-execution is necessary
+     * to trigger or not.
+     *
+     * Different application of Gobblin should have customized logic on calculating progress.
+     */
+    void setProgressInMapper(float progress, Context context) {
+      try {
+        WrappedMapper.Context wrappedContext = ((WrappedMapper.Context) context);
+        Object contextImpl = RestrictedFieldAccessingUtils.getRestrictedFieldByReflection(wrappedContext, "mapContext", wrappedContext.getClass());
+        ((org.apache.hadoop.mapred.Task.TaskReporter)RestrictedFieldAccessingUtils
+            .getRestrictedFieldByReflectionRecursively(contextImpl, "reporter", MapContextImpl.class)).setProgress(progress);
+      } catch (NoSuchFieldException | IllegalAccessException e) {
+        throw new RuntimeException(e);
+      }
+    }
   }
 }
