@@ -22,10 +22,12 @@ import java.lang.reflect.InvocationTargetException;
 import java.net.URI;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 
-import org.apache.commons.lang3.SerializationUtils;
 import org.apache.commons.lang3.reflect.ConstructorUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -34,11 +36,11 @@ import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.google.common.util.concurrent.AbstractIdleService;
 import com.typesafe.config.Config;
+import com.typesafe.config.ConfigException;
 
 import javax.annotation.Nonnull;
+import lombok.Getter;
 
-import org.apache.gobblin.annotation.Alpha;
-import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.instrumented.Instrumented;
 import org.apache.gobblin.metrics.MetricContext;
 import org.apache.gobblin.metrics.Tag;
@@ -49,22 +51,42 @@ import org.apache.gobblin.runtime.api.Spec;
 import org.apache.gobblin.runtime.api.SpecCatalog;
 import org.apache.gobblin.runtime.api.SpecCatalogListener;
 import org.apache.gobblin.runtime.api.SpecNotFoundException;
+import org.apache.gobblin.runtime.api.SpecSearchObject;
 import org.apache.gobblin.runtime.api.SpecSerDe;
 import org.apache.gobblin.runtime.api.SpecStore;
+import org.apache.gobblin.runtime.spec_serde.JavaSpecSerDe;
 import org.apache.gobblin.runtime.spec_store.FSSpecStore;
+import org.apache.gobblin.service.ServiceConfigKeys;
 import org.apache.gobblin.util.ClassAliasResolver;
+import org.apache.gobblin.util.ConfigUtils;
+import org.apache.gobblin.util.callbacks.CallbackResult;
+import org.apache.gobblin.util.callbacks.CallbacksDispatcher;
 
 
-@Alpha
-public class FlowCatalog extends AbstractIdleService implements SpecCatalog, MutableSpecCatalog, SpecSerDe {
+/**
+ * A service that interact with FlowSpec storage.
+ * The FlowSpec storage, a.k.a. {@link SpecStore} should be plugable with different implementation.
+ */
+public class FlowCatalog extends AbstractIdleService implements SpecCatalog, MutableSpecCatalog {
 
+  /***
+   * Configuration properties related to FlowSpec Store
+   */
+  public static final String FLOWSPEC_STORE_CLASS_KEY = "flowSpec.store.class";
+  public static final String FLOWSPEC_STORE_DIR_KEY = "flowSpec.store.dir";
   public static final String DEFAULT_FLOWSPEC_STORE_CLASS = FSSpecStore.class.getCanonicalName();
+  public static final String FLOWSPEC_SERDE_CLASS_KEY = "flowSpec.serde.class";
+  public static final String DEFAULT_FLOWSPEC_SERDE_CLASS = JavaSpecSerDe.class.getCanonicalName();
 
   protected final SpecCatalogListenersList listeners;
   protected final Logger log;
   protected final MetricContext metricContext;
   protected final MutableStandardMetrics metrics;
+  @Getter
   protected final SpecStore specStore;
+  // a map which keeps a handle of condition variables for each spec being added to the flow catalog
+  // to provide synchronization needed for flow specs
+  private final Map<String, Object> specSyncObjects = new HashMap<>();
 
   private final ClassAliasResolver<SpecStore> aliasResolver;
 
@@ -99,17 +121,19 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
     this.aliasResolver = new ClassAliasResolver<>(SpecStore.class);
     try {
       Config newConfig = config;
-      if (config.hasPath(ConfigurationKeys.FLOWSPEC_STORE_DIR_KEY)) {
-        newConfig = config.withValue(ConfigurationKeys.SPECSTORE_FS_DIR_KEY,
-            config.getValue(ConfigurationKeys.FLOWSPEC_STORE_DIR_KEY));
+      if (config.hasPath(FLOWSPEC_STORE_DIR_KEY)) {
+        newConfig = config.withValue(FSSpecStore.SPECSTORE_FS_DIR_KEY,
+            config.getValue(FLOWSPEC_STORE_DIR_KEY));
       }
-      String specStoreClassName = DEFAULT_FLOWSPEC_STORE_CLASS;
-      if (config.hasPath(ConfigurationKeys.FLOWSPEC_STORE_CLASS_KEY)) {
-        specStoreClassName = config.getString(ConfigurationKeys.FLOWSPEC_STORE_CLASS_KEY);
-      }
-      this.log.info("Using audit sink class name/alias " + specStoreClassName);
+      String specStoreClassName = ConfigUtils.getString(config, FLOWSPEC_STORE_CLASS_KEY, DEFAULT_FLOWSPEC_STORE_CLASS);
+      this.log.info(String.format("Using class name/alias [%s] for specstore", specStoreClassName));
+      String specSerDeClassName = ConfigUtils.getString(config, FLOWSPEC_SERDE_CLASS_KEY, DEFAULT_FLOWSPEC_SERDE_CLASS);
+      this.log.info(String.format("Using class name/alias [%s] for spec serde", specSerDeClassName));
+
+      SpecSerDe specSerDe = (SpecSerDe) ConstructorUtils.invokeConstructor(Class.forName(
+          new ClassAliasResolver<>(SpecSerDe.class).resolve(specSerDeClassName)));
       this.specStore = (SpecStore) ConstructorUtils.invokeConstructor(Class.forName(this.aliasResolver.resolve(
-          specStoreClassName)), newConfig, this);
+          specStoreClassName)), newConfig, specSerDe);
     } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException
         | ClassNotFoundException e) {
       throw new RuntimeException(e);
@@ -135,8 +159,13 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
    /**************************************************/
 
   protected void notifyAllListeners() {
-    for (Spec spec : getSpecsWithTimeUpdate()) {
-      this.listeners.onAddSpec(spec);
+    try {
+      Iterator<URI> uriIterator = getSpecURIs();
+      while (uriIterator.hasNext()) {
+        this.listeners.onAddSpec(getSpecWrapper(uriIterator.next()));
+      }
+    } catch (IOException e) {
+      log.error("Cannot retrieve specs from catalog:", e);
     }
   }
 
@@ -146,9 +175,15 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
     this.listeners.addListener(specListener);
 
     if (state() == State.RUNNING) {
-      for (Spec spec : getSpecsWithTimeUpdate()) {
-        SpecCatalogListener.AddSpecCallback addJobCallback = new SpecCatalogListener.AddSpecCallback(spec);
-        this.listeners.callbackOneListener(addJobCallback, specListener);
+      try {
+        Iterator<URI> uriIterator = getSpecURIs();
+        while (uriIterator.hasNext()) {
+          SpecCatalogListener.AddSpecCallback addJobCallback =
+              new SpecCatalogListener.AddSpecCallback(getSpecWrapper(uriIterator.next()));
+          this.listeners.callbackOneListener(addJobCallback, specListener);
+        }
+      } catch (IOException e) {
+        log.error("Cannot retrieve specs from catalog:", e);
       }
     }
   }
@@ -202,21 +237,25 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
   /* Catalog core functionality                     *
   /**************************************************/
 
+  public Iterator<URI> getSpecURIs() throws IOException {
+    return specStore.getSpecURIs();
+  }
+
+  public Iterator<URI> getSpecURISWithTag(String tag) throws IOException {
+    return specStore.getSpecURIsWithTag(tag);
+  }
+
+  /**
+   * Get all specs from {@link SpecStore}
+   * Not suggested for {@link FlowCatalog} where the total amount of space that all {@link FlowSpec}s occupied
+   * would be large and loading process is slow.
+   */
+  @Deprecated
   @Override
   public Collection<Spec> getSpecs() {
     try {
       return specStore.getSpecs();
-    } catch (IOException e) {
-      throw new RuntimeException("Cannot retrieve Specs from Spec store", e);
-    }
-  }
-
-  public Collection<Spec> getSpecsWithTimeUpdate() {
-    try {
-      long startTime = System.currentTimeMillis();
-      Collection<Spec> specs = specStore.getSpecs();
-      this.metrics.updateGetSpecTime(startTime);
-      return specs;
+      // TODO: Have kind of metrics keeping track of specs that failed to be deserialized.
     } catch (IOException e) {
       throw new RuntimeException("Cannot retrieve Specs from Spec store", e);
     }
@@ -231,7 +270,7 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
   }
 
   @Override
-  public Spec getSpec(URI uri) throws SpecNotFoundException {
+  public Spec getSpecs(URI uri) throws SpecNotFoundException {
     try {
       return specStore.getSpec(uri);
     } catch (IOException e) {
@@ -239,27 +278,108 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
     }
   }
 
-  public void put(Spec spec, boolean triggerListener) {
+  @Override
+  public Collection<Spec> getSpecs(SpecSearchObject specSearchObject) {
     try {
-      Preconditions.checkState(state() == State.RUNNING, String.format("%s is not running.", this.getClass().getName()));
-      Preconditions.checkNotNull(spec);
-
-      long startTime = System.currentTimeMillis();
-      log.info(String.format("Adding FlowSpec with URI: %s and Config: %s", spec.getUri(),
-          ((FlowSpec) spec).getConfigAsProperties()));
-      specStore.addSpec(spec);
-      metrics.updatePutSpecTime(startTime);
-      if (triggerListener) {
-        this.listeners.onAddSpec(spec);
-      }
+      return specStore.getSpecs(specSearchObject);
     } catch (IOException e) {
-      throw new RuntimeException("Cannot add Spec to Spec store: " + spec, e);
+      throw new RuntimeException("Cannot retrieve Spec from Spec store for URI: " + specSearchObject, e);
     }
   }
 
+  public Collection<Spec> getAllSpecs() {
+    try {
+      return specStore.getSpecs();
+    } catch (IOException e) {
+      throw new RuntimeException("Cannot retrieve all specs from Spec stores", e);
+    }
+  }
+
+  /**
+   * A wrapper of getSpecs that handles {@link SpecNotFoundException} properly.
+   * This is the most common way to fetch {@link Spec}. For customized way to deal with exception, one will
+   * need to implement specific catch-block logic.
+   */
+  public Spec getSpecWrapper(URI uri) {
+    Spec spec = null;
+    try {
+      spec = getSpecs(uri);
+    } catch (SpecNotFoundException snfe) {
+      log.error(String.format("The URI %s discovered in SpecStore is missing in FlowCatalog"
+          + ", suspecting current modification on SpecStore", uri), snfe);
+    }
+    return spec;
+  }
+
+  /**
+   * Persist {@link Spec} into {@link SpecStore} and notify {@link SpecCatalogListener} if triggerListener
+   * is set to true.
+   * If the {@link Spec} is a {@link FlowSpec} it is persisted if it can be compiled at the time this method received
+   * the spec. `explain` specs are not persisted. The logic of this method is tightly coupled with the logic of
+   * {@link GobblinServiceJobScheduler#onAddSpec()}, which is one of the listener of {@link FlowCatalog}.
+   * We use condition variables {@link #specSyncObjects} to achieve synchronization between
+   * {@link GobblinServiceJobScheduler#NonScheduledJobRunner} thread and this thread to ensure deletion of
+   * {@link FlowSpec} happens after the corresponding run once flow is submitted to the orchestrator.
+   *
+   * @param spec The Spec to be added
+   * @param triggerListener True if listeners should be notified.
+   * @return a map of listeners and their {@link AddSpecResponse}s
+   */
+  public Map<String, AddSpecResponse> put(Spec spec, boolean triggerListener) {
+    Map<String, AddSpecResponse> responseMap = new HashMap<>();
+    FlowSpec flowSpec = (FlowSpec) spec;
+    Preconditions.checkState(state() == State.RUNNING, String.format("%s is not running.", this.getClass().getName()));
+    Preconditions.checkNotNull(flowSpec);
+
+    log.info(String.format("Adding FlowSpec with URI: %s and Config: %s", flowSpec.getUri(), flowSpec.getConfigAsProperties()));
+
+    Object syncObject = new Object();
+    specSyncObjects.put(flowSpec.getUri().toString(), syncObject);
+
+    if (triggerListener) {
+      AddSpecResponse<CallbacksDispatcher.CallbackResults<SpecCatalogListener, AddSpecResponse>> response = this.listeners.onAddSpec(flowSpec);
+      for (Map.Entry<SpecCatalogListener, CallbackResult<AddSpecResponse>> entry: response.getValue().getSuccesses().entrySet()) {
+        responseMap.put(entry.getKey().getName(), entry.getValue().getResult());
+      }
+    }
+
+    if (isCompileSuccessful(responseMap)) {
+      synchronized (syncObject) {
+        try {
+          if (!flowSpec.isExplain()) {
+            long startTime = System.currentTimeMillis();
+            specStore.addSpec(spec);
+            metrics.updatePutSpecTime(startTime);
+          }
+          responseMap.put(ServiceConfigKeys.COMPILATION_SUCCESSFUL, new AddSpecResponse<>("true"));
+        } catch (IOException e) {
+          throw new RuntimeException("Cannot add Spec to Spec store: " + flowSpec, e);
+        } finally {
+          syncObject.notifyAll();
+          this.specSyncObjects.remove(flowSpec.getUri().toString());
+        }
+      }
+    } else {
+      responseMap.put(ServiceConfigKeys.COMPILATION_SUCCESSFUL, new AddSpecResponse<>("false"));
+    }
+
+    return responseMap;
+  }
+
+  public static boolean isCompileSuccessful(Map<String, AddSpecResponse> responseMap) {
+    AddSpecResponse<String> addSpecResponse = responseMap.getOrDefault(
+        ServiceConfigKeys.GOBBLIN_SERVICE_JOB_SCHEDULER_LISTENER_CLASS, new AddSpecResponse<>(""));
+
+    return isCompileSuccessful(addSpecResponse.getValue());
+  }
+
+  public static boolean isCompileSuccessful(String dag) {
+    return dag != null && !dag.contains(ConfigException.class.getSimpleName());
+  }
+
   @Override
-  public void put(Spec spec) {
-    put(spec, true);
+  public Map<String, AddSpecResponse> put(Spec spec) {
+    return put(spec, true);
   }
 
   public void remove(URI uri) {
@@ -287,13 +407,7 @@ public class FlowCatalog extends AbstractIdleService implements SpecCatalog, Mut
     }
   }
 
-  @Override
-  public byte[] serialize(Spec spec) {
-    return SerializationUtils.serialize(spec);
-  }
-
-  @Override
-  public Spec deserialize(byte[] spec) {
-    return SerializationUtils.deserialize(spec);
+  public Object getSyncObject(String specUri) {
+    return this.specSyncObjects.getOrDefault(specUri, null);
   }
 }
