@@ -18,8 +18,14 @@
 package org.apache.gobblin.service.modules.core;
 
 import java.io.IOException;
+import java.net.URI;
+import java.net.URISyntaxException;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.locks.ReadWriteLock;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.hadoop.fs.Path;
@@ -27,6 +33,7 @@ import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.diff.DiffEntry;
 
 import com.google.common.base.Joiner;
+import com.google.common.base.Optional;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.io.Files;
 import com.typesafe.config.Config;
@@ -36,12 +43,14 @@ import com.typesafe.config.ConfigValueFactory;
 import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.configuration.ConfigurationKeys;
+import org.apache.gobblin.runtime.api.SpecExecutor;
+import org.apache.gobblin.runtime.api.TopologySpec;
 import org.apache.gobblin.service.modules.flowgraph.DataNode;
 import org.apache.gobblin.service.modules.flowgraph.FlowEdge;
 import org.apache.gobblin.service.modules.flowgraph.FlowEdgeFactory;
 import org.apache.gobblin.service.modules.flowgraph.FlowGraph;
 import org.apache.gobblin.service.modules.flowgraph.FlowGraphConfigurationKeys;
-import org.apache.gobblin.service.modules.template_catalog.FSFlowCatalog;
+import org.apache.gobblin.service.modules.template_catalog.FSFlowTemplateCatalog;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 
@@ -59,8 +68,8 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
   public static final String GIT_FLOWGRAPH_MONITOR_PREFIX = "gobblin.service.gitFlowGraphMonitor";
 
   private static final String PROPERTIES_EXTENSIONS = "properties";
-  private static final String CONF_EXTENSIONS = StringUtils.EMPTY;
-  private static final String FLOW_EDGE_LABEL_JOINER_CHAR = ":";
+  private static final String CONF_EXTENSIONS = "conf";
+  private static final String FLOW_EDGE_LABEL_JOINER_CHAR = "_";
   private static final String DEFAULT_GIT_FLOWGRAPH_MONITOR_REPO_DIR = "git-flowgraph";
   private static final String DEFAULT_GIT_FLOWGRAPH_MONITOR_FLOWGRAPH_DIR = "gobblin-flowgraph";
   private static final String DEFAULT_GIT_FLOWGRAPH_MONITOR_BRANCH_NAME = "master";
@@ -80,14 +89,19 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
           .put(SHOULD_CHECKPOINT_HASHES, false)
           .build());
 
-  private FSFlowCatalog flowCatalog;
+  private Optional<? extends FSFlowTemplateCatalog> flowTemplateCatalog;
   private FlowGraph flowGraph;
+  private final Map<URI, TopologySpec> topologySpecMap;
   private final Config emptyConfig = ConfigFactory.empty();
+  private final CountDownLatch initComplete;
 
-  public GitFlowGraphMonitor(Config config, FSFlowCatalog flowCatalog, FlowGraph graph) {
+  public GitFlowGraphMonitor(Config config, Optional<? extends FSFlowTemplateCatalog> flowTemplateCatalog,
+      FlowGraph graph, Map<URI, TopologySpec> topologySpecMap, CountDownLatch initComplete) {
     super(config.getConfig(GIT_FLOWGRAPH_MONITOR_PREFIX).withFallback(DEFAULT_FALLBACK));
-    this.flowCatalog = flowCatalog;
+    this.flowTemplateCatalog = flowTemplateCatalog;
     this.flowGraph = graph;
+    this.topologySpecMap = topologySpecMap;
+    this.initComplete = initComplete;
   }
 
   /**
@@ -96,7 +110,7 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
    */
   @Override
   public boolean shouldPollGit() {
-    return true;
+    return this.isActive;
   }
 
   /**
@@ -114,6 +128,11 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
    */
   @Override
   void processGitConfigChanges() throws GitAPIException, IOException {
+    if (flowTemplateCatalog.isPresent() && flowTemplateCatalog.get().getAndSetShouldRefreshFlowGraph(false)) {
+      log.info("Change to template catalog detected, refreshing FlowGraph");
+      this.gitRepo.initRepository();
+    }
+
     List<DiffEntry> changes = this.gitRepo.getChanges();
     Collections.sort(changes, (o1, o2) -> {
       Integer o1Depth = (o1.getNewPath() != null) ? (new Path(o1.getNewPath())).depth() : (new Path(o1.getOldPath())).depth();
@@ -121,7 +140,11 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
       return o1Depth.compareTo(o2Depth);
     });
     processGitConfigChangesHelper(changes);
+    //Decrements the latch count. The countdown latch is initialized to 1. So after the first time the latch is decremented,
+    // the following operation should be a no-op.
+    this.initComplete.countDown();
   }
+
   /**
    * Add an element (i.e., a {@link DataNode}, or a {@link FlowEdge} to
    * the {@link FlowGraph} for an added, updated or modified node or edge file.
@@ -171,7 +194,7 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
           log.info("Added Datanode {} to FlowGraph", dataNode.getId());
         }
       } catch (Exception e) {
-        log.warn("Could not add DataNode defined in {} due to exception {}", change.getNewPath(), e.getMessage());
+        log.warn("Could not add DataNode defined in {} due to exception {}", change.getNewPath(), e);
       }
     }
   }
@@ -203,15 +226,20 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
     if (checkFilePath(change.getNewPath(), EDGE_FILE_DEPTH)) {
       Path edgeFilePath = new Path(this.repositoryDir, change.getNewPath());
       try {
-        Config config = loadEdgeFileWithOverrides(edgeFilePath);
-        Class flowEdgeFactoryClass = Class.forName(ConfigUtils.getString(config, FlowGraphConfigurationKeys.FLOW_EDGE_FACTORY_CLASS,
+        Config edgeConfig = loadEdgeFileWithOverrides(edgeFilePath);
+        List<SpecExecutor> specExecutors = getSpecExecutors(edgeConfig);
+        Class flowEdgeFactoryClass = Class.forName(ConfigUtils.getString(edgeConfig, FlowGraphConfigurationKeys.FLOW_EDGE_FACTORY_CLASS,
             FlowGraphConfigurationKeys.DEFAULT_FLOW_EDGE_FACTORY_CLASS));
-        FlowEdgeFactory flowEdgeFactory = (FlowEdgeFactory) GobblinConstructorUtils.invokeLongestConstructor(flowEdgeFactoryClass, config);
-        FlowEdge edge = flowEdgeFactory.createFlowEdge(config, flowCatalog);
-        if (!this.flowGraph.addFlowEdge(edge)) {
-          log.warn("Could not add edge {} to FlowGraph; skipping", edge.getId());
+        FlowEdgeFactory flowEdgeFactory = (FlowEdgeFactory) GobblinConstructorUtils.invokeLongestConstructor(flowEdgeFactoryClass, edgeConfig);
+        if (flowTemplateCatalog.isPresent()) {
+          FlowEdge edge = flowEdgeFactory.createFlowEdge(edgeConfig, flowTemplateCatalog.get(), specExecutors);
+          if (!this.flowGraph.addFlowEdge(edge)) {
+            log.warn("Could not add edge {} to FlowGraph; skipping", edge.getId());
+          } else {
+            log.info("Added edge {} to FlowGraph", edge.getId());
+          }
         } else {
-          log.info("Added edge {} to FlowGraph", edge.getId());
+          log.warn("Could not add edge defined in {} to FlowGraph as FlowTemplateCatalog is absent", change.getNewPath());
         }
       } catch (Exception e) {
         log.warn("Could not add edge defined in {} due to exception {}", change.getNewPath(), e.getMessage());
@@ -255,7 +283,7 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
     Path filePath = new Path(file);
     String fileExtension = Files.getFileExtension(filePath.getName());
     if (filePath.depth() != depth || !checkFileLevelRelativeToRoot(filePath, depth)
-        || !(this.javaPropsExtensions.contains(fileExtension))) {
+        || !(this.javaPropsExtensions.contains(fileExtension) || this.hoconFileExtensions.contains(fileExtension))) {
       log.warn("Changed file does not conform to directory structure and file name format, skipping: "
           + filePath);
       return false;
@@ -304,11 +332,31 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
     String source = edgeFilePath.getParent().getParent().getName();
     String destination = edgeFilePath.getParent().getName();
     String edgeName = Files.getNameWithoutExtension(edgeFilePath.getName());
+
     return edgeConfig.withValue(FlowGraphConfigurationKeys.FLOW_EDGE_SOURCE_KEY, ConfigValueFactory.fromAnyRef(source))
         .withValue(FlowGraphConfigurationKeys.FLOW_EDGE_DESTINATION_KEY, ConfigValueFactory.fromAnyRef(destination))
         .withValue(FlowGraphConfigurationKeys.FLOW_EDGE_ID_KEY, ConfigValueFactory.fromAnyRef(getEdgeId(source, destination, edgeName)));
   }
 
+  /**
+   * This method first retrieves  the logical names of all the {@link org.apache.gobblin.runtime.api.SpecExecutor}s
+   * for this edge and returns the SpecExecutors from the {@link TopologySpec} map.
+   * @param edgeConfig containing the logical names of SpecExecutors for this edge.
+   * @return a {@link List<SpecExecutor>}s for this edge.
+   */
+  private List<SpecExecutor> getSpecExecutors(Config edgeConfig)
+      throws URISyntaxException {
+    //Get the logical names of SpecExecutors where the FlowEdge can be executed.
+    List<String> specExecutorNames = ConfigUtils.getStringList(edgeConfig, FlowGraphConfigurationKeys.FLOW_EDGE_SPEC_EXECUTORS_KEY);
+    //Load all the SpecExecutor configurations for this FlowEdge from the SpecExecutor Catalog.
+    List<SpecExecutor> specExecutors = new ArrayList<>();
+    for (String specExecutorName: specExecutorNames) {
+      URI specExecutorUri = new URI(specExecutorName);
+      specExecutors.add(this.topologySpecMap.get(specExecutorUri).getSpecExecutor());
+    }
+    return specExecutors;
+  }
+
   /**
    * Load the node file.
    * @param filePath path of the node file relative to the repository root
@@ -316,7 +364,7 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
    * @throws IOException
    */
   private Config loadNodeFileWithOverrides(Path filePath) throws IOException {
-    Config nodeConfig = this.pullFileLoader.loadPullFile(filePath, emptyConfig, false);
+    Config nodeConfig = this.pullFileLoader.loadPullFile(filePath, emptyConfig, false, false);
     return getNodeConfigWithOverrides(nodeConfig, filePath);
   }
 
@@ -327,7 +375,7 @@ public class GitFlowGraphMonitor extends GitMonitoringService {
    * @throws IOException
    */
   private Config loadEdgeFileWithOverrides(Path filePath) throws IOException {
-    Config edgeConfig = this.pullFileLoader.loadPullFile(filePath, emptyConfig, false);
+    Config edgeConfig = this.pullFileLoader.loadPullFile(filePath, emptyConfig, false, false);
     return getEdgeConfigWithOverrides(edgeConfig, filePath);
   }
 
