@@ -41,6 +41,7 @@ import org.apache.helix.model.Message;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.codahale.metrics.MetricRegistry;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.base.Throwables;
@@ -60,13 +61,16 @@ import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 
 import lombok.Getter;
+import lombok.Setter;
 
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.instrumented.Instrumented;
 import org.apache.gobblin.instrumented.StandardMetricsBridge;
+import org.apache.gobblin.metrics.ContextAwareGauge;
 import org.apache.gobblin.metrics.ContextAwareHistogram;
 import org.apache.gobblin.metrics.MetricContext;
+import org.apache.gobblin.metrics.ServiceMetricNames;
 import org.apache.gobblin.restli.EmbeddedRestliServer;
 import org.apache.gobblin.runtime.api.TopologySpec;
 import org.apache.gobblin.runtime.app.ApplicationException;
@@ -93,19 +97,32 @@ import org.apache.gobblin.service.modules.restli.GobblinServiceFlowConfigResourc
 import org.apache.gobblin.service.modules.scheduler.GobblinServiceJobScheduler;
 import org.apache.gobblin.service.modules.topology.TopologySpecFactory;
 import org.apache.gobblin.service.modules.utils.HelixUtils;
+import org.apache.gobblin.service.monitoring.FlowStatusGenerator;
+import org.apache.gobblin.service.monitoring.FsJobStatusRetriever;
+import org.apache.gobblin.service.monitoring.JobStatusRetriever;
+import org.apache.gobblin.service.monitoring.KafkaJobStatusMonitor;
+import org.apache.gobblin.service.monitoring.KafkaJobStatusMonitorFactory;
 import org.apache.gobblin.util.ClassAliasResolver;
 import org.apache.gobblin.util.ConfigUtils;
+import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 
 
 @Alpha
 public class GobblinServiceManager implements ApplicationLauncher, StandardMetricsBridge {
 
+  // Command line options
+  // These two options are required to launch GobblinServiceManager.
+  public static final String SERVICE_NAME_OPTION_NAME = "service_name";
+  public static final String SERVICE_ID_OPTION_NAME = "service_id";
+
   private static final Logger LOGGER = LoggerFactory.getLogger(GobblinServiceManager.class);
+  private static final String JOB_STATUS_RETRIEVER_CLASS_KEY = "jobStatusRetriever.class";
 
   protected final ServiceBasedAppLauncher serviceLauncher;
   private volatile boolean stopInProgress = false;
 
   // An EventBus used for communications between services running in the ApplicationMaster
+  @Getter
   protected final EventBus eventBus = new EventBus(GobblinServiceManager.class.getSimpleName());
 
   protected final FileSystem fs;
@@ -120,6 +137,7 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
   protected final boolean isTopologySpecFactoryEnabled;
   protected final boolean isGitConfigMonitorEnabled;
   protected final boolean isDagManagerEnabled;
+  protected final boolean isJobStatusMonitorEnabled;
 
   protected TopologyCatalog topologyCatalog;
   @Getter
@@ -132,6 +150,7 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
   protected GobblinServiceFlowConfigResourceHandler v2ResourceHandler;
 
   protected boolean flowCatalogLocalCommit;
+  @Getter
   protected Orchestrator orchestrator;
   protected EmbeddedRestliServer restliServer;
   protected TopologySpecFactory topologySpecFactory;
@@ -144,6 +163,11 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
 
   protected DagManager dagManager;
 
+  protected KafkaJobStatusMonitor jobStatusMonitor;
+
+  protected Optional<HelixLeaderState> helixLeaderGauges;
+
+
   @Getter
   protected Config config;
   private final MetricContext metricContext;
@@ -152,17 +176,18 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
   public GobblinServiceManager(String serviceName, String serviceId, Config config,
       Optional<Path> serviceWorkDirOptional) throws Exception {
 
+    Properties appLauncherProperties = ConfigUtils.configToProperties(ConfigUtils.getConfigOrEmpty(config,
+        ServiceConfigKeys.GOBBLIN_SERVICE_APP_LAUNCHER_PREFIX).withFallback(config));
     // Done to preserve backwards compatibility with the previously hard-coded timeout of 5 minutes
-    Properties properties = ConfigUtils.configToProperties(config);
-    if (!properties.contains(ServiceBasedAppLauncher.APP_STOP_TIME_SECONDS)) {
-      properties.setProperty(ServiceBasedAppLauncher.APP_STOP_TIME_SECONDS, Long.toString(300));
+    if (!appLauncherProperties.contains(ServiceBasedAppLauncher.APP_STOP_TIME_SECONDS)) {
+      appLauncherProperties.setProperty(ServiceBasedAppLauncher.APP_STOP_TIME_SECONDS, Long.toString(300));
     }
     this.config = config;
     this.metricContext = Instrumented.getMetricContext(ConfigUtils.configToState(config), this.getClass());
-    this.metrics = new Metrics(this.metricContext, this.config);
+    this.metrics = new Metrics(this.metricContext, config);
     this.serviceName = serviceName;
     this.serviceId = serviceId;
-    this.serviceLauncher = new ServiceBasedAppLauncher(properties, serviceName);
+    this.serviceLauncher = new ServiceBasedAppLauncher(appLauncherProperties, serviceName);
 
     this.fs = buildFileSystem(config);
     this.serviceWorkDir = serviceWorkDirOptional.isPresent() ? serviceWorkDirOptional.get()
@@ -188,22 +213,20 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
       this.isGitConfigMonitorEnabled = ConfigUtils.getBoolean(config,
           ServiceConfigKeys.GOBBLIN_SERVICE_GIT_CONFIG_MONITOR_ENABLED_KEY, false);
 
-      this.isDagManagerEnabled = ConfigUtils.getBoolean(config, ServiceConfigKeys.GOBBLIN_SERVICE_DAG_MANAGER_ENABLED_KEY, false);
-
       if (this.isGitConfigMonitorEnabled) {
         this.gitConfigMonitor = new GitConfigMonitor(config, this.flowCatalog);
         this.serviceLauncher.addService(this.gitConfigMonitor);
       }
-
-      if (this.isDagManagerEnabled) {
-        this.dagManager = new DagManager(config);
-        this.serviceLauncher.addService(this.dagManager);
-      }
     } else {
       this.isGitConfigMonitorEnabled = false;
-      this.isDagManagerEnabled = false;
     }
 
+    // Initialize Helix leader guage
+    helixLeaderGauges = Optional.of(new HelixLeaderState());
+    String helixLeaderStateGaugeName =
+        MetricRegistry.name(ServiceMetricNames.GOBBLIN_SERVICE_PREFIX, ServiceMetricNames.HELIX_LEADER_STATE);
+    ContextAwareGauge<Integer> gauge = metricContext.newContextAwareGauge(helixLeaderStateGaugeName, () -> helixLeaderGauges.get().state.getValue());
+    metricContext.register(helixLeaderStateGaugeName, gauge);
 
 
     // Initialize Helix
@@ -218,12 +241,26 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
       this.helixManager = Optional.absent();
     }
 
+    this.isDagManagerEnabled = ConfigUtils.getBoolean(config, ServiceConfigKeys.GOBBLIN_SERVICE_DAG_MANAGER_ENABLED_KEY, false);
+    // Initialize DagManager
+    if (this.isDagManagerEnabled) {
+      this.dagManager = new DagManager(config);
+      this.serviceLauncher.addService(this.dagManager);
+    }
+
+    this.isJobStatusMonitorEnabled = ConfigUtils.getBoolean(config, ServiceConfigKeys.GOBBLIN_SERVICE_JOB_STATUS_MONITOR_ENABLED_KEY, true) ;
+    // Initialize JobStatusMonitor
+    if (this.isJobStatusMonitorEnabled) {
+      this.jobStatusMonitor = new KafkaJobStatusMonitorFactory().createJobStatusMonitor(config);
+      this.serviceLauncher.addService(this.jobStatusMonitor);
+    }
+
     // Initialize ServiceScheduler
     this.isSchedulerEnabled = ConfigUtils.getBoolean(config,
         ServiceConfigKeys.GOBBLIN_SERVICE_SCHEDULER_ENABLED_KEY, true);
     if (isSchedulerEnabled) {
       this.orchestrator = new Orchestrator(config, Optional.of(this.topologyCatalog), Optional.fromNullable(this.dagManager), Optional.of(LOGGER));
-      SchedulerService schedulerService = new SchedulerService(properties);
+      SchedulerService schedulerService = new SchedulerService(ConfigUtils.configToProperties(config));
 
       this.scheduler = new GobblinServiceJobScheduler(this.serviceName, config, this.helixManager,
           Optional.of(this.flowCatalog), Optional.of(this.topologyCatalog), this.orchestrator,
@@ -261,13 +298,19 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
           binder.bindConstant()
               .annotatedWith(Names.named(FlowConfigsResource.INJECT_READY_TO_USE))
               .to(Boolean.TRUE);
+          binder.bindConstant()
+              .annotatedWith(Names.named(FlowConfigsV2Resource.INJECT_READY_TO_USE))
+              .to(Boolean.TRUE);
           binder.bind(RequesterService.class)
               .annotatedWith(Names.named(FlowConfigsResource.INJECT_REQUESTER_SERVICE))
               .toInstance(new NoopRequesterService(config));
+          binder.bind(RequesterService.class)
+              .annotatedWith(Names.named(FlowConfigsV2Resource.INJECT_REQUESTER_SERVICE))
+              .toInstance(new NoopRequesterService(config));
         }
       });
       this.restliServer = EmbeddedRestliServer.builder()
-          .resources(Lists.<Class<? extends BaseResource>>newArrayList(FlowConfigsResource.class))
+          .resources(Lists.newArrayList(FlowConfigsResource.class, FlowConfigsV2Resource.class))
           .injector(injector)
           .build();
       if (config.hasPath(ServiceConfigKeys.SERVICE_PORT)) {
@@ -335,11 +378,24 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
     return new Path(fs.getHomeDirectory(), serviceName + Path.SEPARATOR + serviceId);
   }
 
+  private FlowStatusGenerator buildFlowStatusGenerator(Config config) {
+    JobStatusRetriever jobStatusRetriever;
+    try {
+      Class jobStatusRetrieverClass = Class.forName(ConfigUtils.getString(config, JOB_STATUS_RETRIEVER_CLASS_KEY, FsJobStatusRetriever.class.getName()));
+      jobStatusRetriever =
+          (JobStatusRetriever) GobblinConstructorUtils.invokeLongestConstructor(jobStatusRetrieverClass, config);
+    } catch (ReflectiveOperationException e) {
+      LOGGER.error("Exception encountered when instantiating JobStatusRetriever");
+      throw new RuntimeException(e);
+    }
+    return FlowStatusGenerator.builder().jobStatusRetriever(jobStatusRetriever).build();
+  }
+
   /**
    * Handle leadership change.
    * @param changeContext notification context
    */
-  private void handleLeadershipChange(NotificationContext changeContext) {
+  private void  handleLeadershipChange(NotificationContext changeContext) {
     if (this.helixManager.isPresent() && this.helixManager.get().isLeader()) {
       LOGGER.info("Leader notification for {} HM.isLeader {}", this.helixManager.get().getInstanceName(),
           this.helixManager.get().isLeader());
@@ -349,21 +405,33 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
         this.scheduler.setActive(true);
       }
 
+      if (helixLeaderGauges.isPresent()) {
+        helixLeaderGauges.get().setState(LeaderState.MASTER);
+      }
+
       if (this.isGitConfigMonitorEnabled) {
         this.gitConfigMonitor.setActive(true);
       }
 
       if (this.isDagManagerEnabled) {
-        this.dagManager.setActive(true);
+        //Activate DagManager only if TopologyCatalog is initialized. If not; skip activation.
+        if (this.topologyCatalog.getInitComplete().getCount() == 0) {
+          this.dagManager.setActive(true);
+        }
       }
     } else if (this.helixManager.isPresent()) {
       LOGGER.info("Leader lost notification for {} HM.isLeader {}", this.helixManager.get().getInstanceName(),
           this.helixManager.get().isLeader());
+
       if (this.isSchedulerEnabled) {
         LOGGER.info("Gobblin Service is now running in slave instance mode, disabling Scheduler.");
         this.scheduler.setActive(false);
       }
 
+      if (helixLeaderGauges.isPresent()) {
+        helixLeaderGauges.get().setState(LeaderState.SLAVE);
+      }
+
       if (this.isGitConfigMonitorEnabled) {
         this.gitConfigMonitor.setActive(false);
       }
@@ -394,6 +462,7 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
         }
       });
 
+
       // Update for first time since there might be no notification
       if (helixManager.get().isLeader()) {
         if (this.isSchedulerEnabled) {
@@ -405,14 +474,17 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
           this.gitConfigMonitor.setActive(true);
         }
 
-        if (this.isDagManagerEnabled) {
-          this.dagManager.setActive(true);
+        if (helixLeaderGauges.isPresent()) {
+          helixLeaderGauges.get().setState(LeaderState.MASTER);
         }
 
       } else {
         if (this.isSchedulerEnabled) {
           LOGGER.info("[Init] Gobblin Service is running in slave instance mode, not enabling Scheduler.");
         }
+        if (helixLeaderGauges.isPresent()) {
+          helixLeaderGauges.get().setState(LeaderState.SLAVE);
+        }
       }
     } else {
       // No Helix manager, hence standalone service instance
@@ -423,11 +495,6 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
       if (this.isGitConfigMonitorEnabled) {
         this.gitConfigMonitor.setActive(true);
       }
-
-      if (this.isDagManagerEnabled) {
-        this.dagManager.setActive(true);
-      }
-
     }
 
     // Populate TopologyCatalog with all Topologies generated by TopologySpecFactory
@@ -447,6 +514,17 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
 
     // Notify now topologyCatalog has the right information
     this.topologyCatalog.getInitComplete().countDown();
+
+    //Activate the SpecCompiler, after the topologyCatalog has been initialized.
+    this.orchestrator.getSpecCompiler().setActive(true);
+
+    //Activate the DagManager service, after the topologyCatalog has been initialized.
+    if (!this.helixManager.isPresent() || this.helixManager.get().isLeader()){
+      if (this.isDagManagerEnabled) {
+        this.dagManager.setActive(true);
+        this.eventBus.register(this.dagManager);
+      }
+    }
   }
 
   @Override
@@ -518,13 +596,15 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
     }
   }
 
-  private static String getServiceId() {
-    return "1";
+  private static String getServiceId(CommandLine cmd) {
+    return cmd.getOptionValue(SERVICE_ID_OPTION_NAME) == null ? "1" : cmd.getOptionValue(SERVICE_ID_OPTION_NAME);
   }
 
   private static Options buildOptions() {
     Options options = new Options();
-    options.addOption("a", ServiceConfigKeys.SERVICE_NAME_OPTION_NAME, true, "Gobblin application name");
+    options.addOption("a", SERVICE_NAME_OPTION_NAME, true, "Gobblin Service application's name");
+    options.addOption("i", SERVICE_ID_OPTION_NAME, true, "Gobblin Service application's ID, "
+        + "this needs to be globally unique");
     return options;
   }
 
@@ -537,11 +617,16 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
     Options options = buildOptions();
     try {
       CommandLine cmd = new DefaultParser().parse(options, args);
-      if (!cmd.hasOption(ServiceConfigKeys.SERVICE_NAME_OPTION_NAME)) {
+      if (!cmd.hasOption(SERVICE_NAME_OPTION_NAME)) {
         printUsage(options);
         System.exit(1);
       }
 
+      if (!cmd.hasOption(SERVICE_ID_OPTION_NAME)) {
+        printUsage(options);
+        LOGGER.warn("Please assign globally unique ID for a GobblinServiceManager instance, or it will use default ID");
+      }
+
       boolean isTestMode = false;
       if (cmd.hasOption("test_mode")) {
         isTestMode = Boolean.parseBoolean(cmd.getOptionValue("test_mode", "false"));
@@ -549,9 +634,9 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
 
       Config config = ConfigFactory.load();
       try (GobblinServiceManager gobblinServiceManager = new GobblinServiceManager(
-          cmd.getOptionValue(ServiceConfigKeys.SERVICE_NAME_OPTION_NAME), getServiceId(),
+          cmd.getOptionValue(SERVICE_NAME_OPTION_NAME), getServiceId(cmd),
           config, Optional.<Path>absent())) {
-
+        gobblinServiceManager.getOrchestrator().setFlowStatusGenerator(gobblinServiceManager.buildFlowStatusGenerator(config));
         gobblinServiceManager.start();
 
         if (isTestMode) {
@@ -591,4 +676,21 @@ public class GobblinServiceManager implements ApplicationLauncher, StandardMetri
       throw new RuntimeException(e);
     }
   }
+
+  @Setter
+  private static class HelixLeaderState {
+    private LeaderState state = LeaderState.UNKNOWN;
+  }
+
+  private enum LeaderState {
+    UNKNOWN(-1),
+    SLAVE(0),
+    MASTER(1);
+
+    @Getter private int value;
+
+    LeaderState(int value) {
+      this.value = value;
+    }
+  }
 }
