@@ -19,63 +19,54 @@ package org.apache.gobblin.service.modules.flowgraph;
 
 import com.google.common.base.Preconditions;
 import com.typesafe.config.Config;
-import com.typesafe.config.ConfigFactory;
+
+import joptsimple.internal.Strings;
+import lombok.EqualsAndHashCode;
+import lombok.Getter;
+import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.util.ConfigUtils;
 
-import joptsimple.internal.Strings;
-import lombok.Getter;
-import lombok.extern.slf4j.Slf4j;
-
 
 /**
  * An implementation of {@link DataNode}.
  */
 @Alpha
 @Slf4j
+@EqualsAndHashCode (exclude = {"rawConfig", "resolvedConfig", "active"})
 public class BaseDataNode implements DataNode {
   @Getter
   private String id;
   @Getter
   private Config rawConfig;
   @Getter
+  private Config resolvedConfig;
+  @Getter
   private boolean active = true;
 
   public BaseDataNode(Config nodeProps) throws DataNodeCreationException {
     try {
-      String nodeId = ConfigUtils.getString(nodeProps, FlowGraphConfigurationKeys.DATA_NODE_ID_KEY, "");
+      this.rawConfig = nodeProps;
+      this.resolvedConfig = nodeProps.resolve();
+      String nodeId = ConfigUtils.getString(this.resolvedConfig, FlowGraphConfigurationKeys.DATA_NODE_ID_KEY, "");
       Preconditions.checkArgument(!Strings.isNullOrEmpty(nodeId), "Node Id cannot be null or empty");
       this.id = nodeId;
-      if (nodeProps.hasPath(FlowGraphConfigurationKeys.DATA_NODE_IS_ACTIVE_KEY)) {
-        this.active = nodeProps.getBoolean(FlowGraphConfigurationKeys.DATA_NODE_IS_ACTIVE_KEY);
+      if (this.resolvedConfig.hasPath(FlowGraphConfigurationKeys.DATA_NODE_IS_ACTIVE_KEY)) {
+        this.active = this.resolvedConfig.getBoolean(FlowGraphConfigurationKeys.DATA_NODE_IS_ACTIVE_KEY);
       }
-      this.rawConfig = nodeProps;
     } catch (Exception e) {
       throw new DataNodeCreationException(e);
     }
   }
 
-  /**
-   * The comparison between two nodes should involve the configuration.
-   * Node name is the identifier for the node.
-   * */
   @Override
-  public boolean equals(Object o) {
-    if (this == o) {
-      return true;
-    }
-    if (o == null || getClass() != o.getClass()) {
-      return false;
-    }
-
-    BaseDataNode that = (BaseDataNode) o;
-
-    return id.equals(that.getId());
+  public String getDefaultDatasetDescriptorClass() {
+    return null;
   }
 
   @Override
-  public int hashCode() {
-    return this.id.hashCode();
+  public String getDefaultDatasetDescriptorPlatform() {
+    return null;
   }
 }
