@@ -25,13 +25,16 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 
 import lombok.Getter;
+import lombok.Setter;
 
 import org.apache.gobblin.annotation.Alpha;
+import org.apache.gobblin.service.ExecutionStatus;
 
 
 /**
@@ -47,6 +50,11 @@ public class Dag<T> {
   private Map<DagNode, List<DagNode<T>>> parentChildMap;
   private List<DagNode<T>> nodes;
 
+  @Setter
+  private String message;
+  @Setter
+  private String flowEvent;
+
   public Dag(List<DagNode<T>> dagNodes) {
     this.nodes = dagNodes;
     //Build dag
@@ -238,6 +246,9 @@ public class Dag<T> {
     return this;
   }
 
+  /**
+   * DagNode is essentially a job within a Dag, usually they are used interchangeably.
+   */
   @Getter
   public static class DagNode<T> {
     private T value;
@@ -274,4 +285,12 @@ public class Dag<T> {
       return this.getValue().hashCode();
     }
   }
+
+  /**
+   * @return A string representation of the Dag as a JSON Array.
+   */
+  @Override
+  public String toString() {
+    return this.getNodes().stream().map(node -> node.getValue().toString()).collect(Collectors.toList()).toString();
+  }
 }
