@@ -17,25 +17,22 @@
 
 package org.apache.gobblin.service.modules.flowgraph.pathfinder;
 
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
-import java.util.concurrent.ExecutionException;
-
-import org.apache.commons.lang3.StringUtils;
-import org.apache.commons.lang3.tuple.Pair;
-
 import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 import com.typesafe.config.Config;
+import com.typesafe.config.ConfigException;
+import com.typesafe.config.ConfigFactory;
 import com.typesafe.config.ConfigValue;
 import com.typesafe.config.ConfigValueFactory;
-
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.stream.Collectors;
 import lombok.extern.slf4j.Slf4j;
-
+import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.gobblin.runtime.api.FlowSpec;
@@ -44,6 +41,7 @@ import org.apache.gobblin.runtime.api.SpecExecutor;
 import org.apache.gobblin.runtime.api.SpecNotFoundException;
 import org.apache.gobblin.service.ServiceConfigKeys;
 import org.apache.gobblin.service.modules.dataset.DatasetDescriptor;
+import org.apache.gobblin.service.modules.dataset.DatasetDescriptorUtils;
 import org.apache.gobblin.service.modules.flow.FlowEdgeContext;
 import org.apache.gobblin.service.modules.flow.FlowGraphPath;
 import org.apache.gobblin.service.modules.flow.FlowUtils;
@@ -84,8 +82,8 @@ public abstract class AbstractPathFinder implements PathFinder {
       throws ReflectiveOperationException {
     this.flowGraph = flowGraph;
     this.flowSpec = flowSpec;
-    this.flowConfig = flowSpec.getConfig();
     this.flowExecutionId = FlowUtils.getOrCreateFlowExecutionId(flowSpec);
+    this.flowConfig = flowSpec.getConfig().withValue(ConfigurationKeys.FLOW_EXECUTION_ID_KEY, ConfigValueFactory.fromAnyRef(flowExecutionId));
 
     //Get src/dest DataNodes from the flow config
     String srcNodeId = ConfigUtils.getString(flowConfig, ServiceConfigKeys.FLOW_SOURCE_IDENTIFIER_KEY, "");
@@ -102,6 +100,11 @@ public abstract class AbstractPathFinder implements PathFinder {
       this.destNodes.add(destNode);
     }
 
+    // All dest nodes should be the same class
+    if (this.destNodes != null && this.destNodes.stream().map(Object::getClass).collect(Collectors.toSet()).size() > 1) {
+      throw new RuntimeException("All destination nodes must use the same DataNode class");
+    }
+
     //Should apply retention?
     boolean shouldApplyRetention = ConfigUtils.getBoolean(flowConfig, ConfigurationKeys.FLOW_APPLY_RETENTION, true);
     //Should apply retention on input dataset?
@@ -143,15 +146,27 @@ public abstract class AbstractPathFinder implements PathFinder {
     flowConfig = flowConfig.withValue(ConfigurationKeys.FLOW_APPLY_RETENTION, ConfigValueFactory.fromAnyRef(shouldApplyRetention));
     flowConfig = flowConfig.withValue(ConfigurationKeys.FLOW_APPLY_INPUT_RETENTION, ConfigValueFactory.fromAnyRef(shouldApplyRetentionOnInput));
 
-    Class srcdatasetDescriptorClass =
-        Class.forName(srcDatasetDescriptorConfig.getString(DatasetDescriptorConfigKeys.CLASS_KEY));
-    this.srcDatasetDescriptor = (DatasetDescriptor) GobblinConstructorUtils
-        .invokeLongestConstructor(srcdatasetDescriptorClass, srcDatasetDescriptorConfig);
-    Class destDatasetDescriptorClass =
-        Class.forName(destDatasetDescriptorConfig.getString(DatasetDescriptorConfigKeys.CLASS_KEY));
-    this.destDatasetDescriptor = (DatasetDescriptor) GobblinConstructorUtils
-        .invokeLongestConstructor(destDatasetDescriptorClass, destDatasetDescriptorConfig);
+    srcDatasetDescriptorConfig = srcDatasetDescriptorConfig.withFallback(getDefaultConfig(this.srcNode));
+    if (this.destNodes != null) {
+      destDatasetDescriptorConfig = destDatasetDescriptorConfig.withFallback(getDefaultConfig(this.destNodes.get(0)));
+    }
 
+    this.srcDatasetDescriptor = DatasetDescriptorUtils.constructDatasetDescriptor(srcDatasetDescriptorConfig);
+    this.destDatasetDescriptor = DatasetDescriptorUtils.constructDatasetDescriptor(destDatasetDescriptorConfig);
+  }
+
+  private Config getDefaultConfig(DataNode dataNode) {
+    Config defaultConfig = ConfigFactory.empty();
+
+    if (dataNode.getDefaultDatasetDescriptorClass() != null) {
+      defaultConfig = defaultConfig.withValue(DatasetDescriptorConfigKeys.CLASS_KEY, ConfigValueFactory.fromAnyRef(dataNode.getDefaultDatasetDescriptorClass()));
+    }
+
+    if (dataNode.getDefaultDatasetDescriptorPlatform() != null) {
+      defaultConfig = defaultConfig.withValue(DatasetDescriptorConfigKeys.PLATFORM_KEY, ConfigValueFactory.fromAnyRef(dataNode.getDefaultDatasetDescriptorPlatform()));
+    }
+
+    return defaultConfig;
   }
 
   boolean isPathFound(DataNode currentNode, DataNode destNode, DatasetDescriptor currentDatasetDescriptor,
@@ -170,7 +185,11 @@ public abstract class AbstractPathFinder implements PathFinder {
   List<FlowEdgeContext> getNextEdges(DataNode dataNode, DatasetDescriptor currentDatasetDescriptor,
       DatasetDescriptor destDatasetDescriptor) {
     List<FlowEdgeContext> prioritizedEdgeList = new LinkedList<>();
+    List<String> edgeIds = ConfigUtils.getStringList(this.flowConfig, ConfigurationKeys.WHITELISTED_EDGE_IDS);
     for (FlowEdge flowEdge : this.flowGraph.getEdges(dataNode)) {
+      if (!edgeIds.isEmpty() && !edgeIds.contains(flowEdge.getId())) {
+        continue;
+      }
       try {
         DataNode edgeDestination = this.flowGraph.getNode(flowEdge.getDest());
         //Base condition: Skip this FLowEdge, if it is inactive or if the destination of this edge is inactive.
@@ -181,13 +200,20 @@ public abstract class AbstractPathFinder implements PathFinder {
         boolean foundExecutor = false;
         //Iterate over all executors for this edge. Find the first one that resolves the underlying flow template.
         for (SpecExecutor specExecutor : flowEdge.getExecutors()) {
-          Config mergedConfig = getMergedConfig(flowEdge, specExecutor);
+          Config mergedConfig = getMergedConfig(flowEdge);
           List<Pair<DatasetDescriptor, DatasetDescriptor>> datasetDescriptorPairs =
-              flowEdge.getFlowTemplate().getResolvingDatasetDescriptors(mergedConfig);
+              flowEdge.getFlowTemplate().getDatasetDescriptors(mergedConfig, false);
           for (Pair<DatasetDescriptor, DatasetDescriptor> datasetDescriptorPair : datasetDescriptorPairs) {
             DatasetDescriptor inputDatasetDescriptor = datasetDescriptorPair.getLeft();
             DatasetDescriptor outputDatasetDescriptor = datasetDescriptorPair.getRight();
 
+            try {
+              flowEdge.getFlowTemplate().tryResolving(mergedConfig, datasetDescriptorPair.getLeft(), datasetDescriptorPair.getRight());
+            } catch (JobTemplate.TemplateException | ConfigException | SpecNotFoundException e) {
+              this.flowSpec.getCompilationErrors().add(e.toString());
+              continue;
+            }
+
             if (inputDatasetDescriptor.contains(currentDatasetDescriptor)) {
               DatasetDescriptor edgeOutputDescriptor = makeOutputDescriptorSpecific(currentDatasetDescriptor, outputDatasetDescriptor);
               FlowEdgeContext flowEdgeContext = new FlowEdgeContext(flowEdge, currentDatasetDescriptor, edgeOutputDescriptor, mergedConfig,
@@ -212,8 +238,7 @@ public abstract class AbstractPathFinder implements PathFinder {
             break;
           }
         }
-      } catch (IOException | ReflectiveOperationException | InterruptedException | ExecutionException | SpecNotFoundException
-          | JobTemplate.TemplateException e) {
+      } catch (IOException | ReflectiveOperationException | SpecNotFoundException | JobTemplate.TemplateException e) {
         //Skip the edge; and continue
         log.warn("Skipping edge {} with config {} due to exception: {}", flowEdge.getId(), flowConfig.toString(), e);
       }
@@ -258,9 +283,7 @@ public abstract class AbstractPathFinder implements PathFinder {
       throws ReflectiveOperationException {
     Config config = outputDescriptor.getRawConfig();
 
-    for (Iterator<Map.Entry<String, ConfigValue>> iterator = currentDescriptor.getRawConfig().entrySet().iterator();
-        iterator.hasNext(); ) {
-      Map.Entry<String, ConfigValue> entry = iterator.next();
+    for (Map.Entry<String, ConfigValue> entry : currentDescriptor.getRawConfig().entrySet()) {
       String entryValue = entry.getValue().unwrapped().toString();
       if (!isPlaceHolder(entryValue)) {
         String entryValueInOutputDescriptor = ConfigUtils.getString(config, entry.getKey(), StringUtils.EMPTY);
@@ -285,21 +308,17 @@ public abstract class AbstractPathFinder implements PathFinder {
    * <ul>
    *   <p> the user provided flow config </p>
    *   <p> edge specific properties/overrides </p>
-   *   <p> spec executor config/overrides </p>
    *   <p> source node config </p>
    *   <p> destination node config </p>
    * </ul>
    * Each {@link JobTemplate}'s config will eventually be resolved against this merged config.
    * @param flowEdge An instance of {@link FlowEdge}.
-   * @param specExecutor A {@link SpecExecutor}.
    * @return the merged config derived as described above.
    */
-  private Config getMergedConfig(FlowEdge flowEdge, SpecExecutor specExecutor)
-      throws ExecutionException, InterruptedException {
+  private Config getMergedConfig(FlowEdge flowEdge) {
     Config srcNodeConfig = this.flowGraph.getNode(flowEdge.getSrc()).getRawConfig().atPath(SOURCE_PREFIX);
     Config destNodeConfig = this.flowGraph.getNode(flowEdge.getDest()).getRawConfig().atPath(DESTINATION_PREFIX);
-    Config mergedConfig = flowConfig.withFallback(specExecutor.getConfig().get()).withFallback(flowEdge.getConfig())
-        .withFallback(srcNodeConfig).withFallback(destNodeConfig);
+    Config mergedConfig = flowConfig.withFallback(flowEdge.getConfig()).withFallback(srcNodeConfig).withFallback(destNodeConfig);
     return mergedConfig;
   }
 
@@ -325,13 +344,15 @@ public abstract class AbstractPathFinder implements PathFinder {
   public FlowGraphPath findPath() throws PathFinderException {
 
     FlowGraphPath flowGraphPath = new FlowGraphPath(flowSpec, flowExecutionId);
-    //Path computation must be thread-safe to guarantee read consistency. In other words, we prevent concurrent read/write access to the
+    // Path computation must be thread-safe to guarantee read consistency. In other words, we prevent concurrent read/write access to the
     // flow graph.
     for (DataNode destNode : this.destNodes) {
       List<FlowEdgeContext> path = findPathUnicast(destNode);
       if (path != null) {
+        log.info("Path to destination node {} found for flow {}. Path - {}", destNode.getId(), flowSpec.getUri(), path);
         flowGraphPath.addPath(path);
       } else {
+        log.error("Path to destination node {} could not be found for flow {}.", destNode.getId(), flowSpec.getUri());
         //No path to at least one of the destination nodes.
         return null;
       }
