@@ -29,6 +29,7 @@ import org.apache.commons.lang3.tuple.Pair;
 import com.google.common.base.Joiner;
 import com.google.common.collect.Lists;
 import com.typesafe.config.Config;
+import com.typesafe.config.ConfigException;
 import com.typesafe.config.ConfigResolveOptions;
 import com.typesafe.config.ConfigValueFactory;
 
@@ -37,12 +38,15 @@ import lombok.extern.slf4j.Slf4j;
 
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
+import org.apache.gobblin.runtime.api.JobSpec;
 import org.apache.gobblin.runtime.api.JobTemplate;
 import org.apache.gobblin.runtime.api.SpecNotFoundException;
+import org.apache.gobblin.runtime.job_spec.JobSpecResolver;
+import org.apache.gobblin.runtime.job_spec.ResolvedJobSpec;
 import org.apache.gobblin.service.modules.dataset.DatasetDescriptor;
+import org.apache.gobblin.service.modules.dataset.DatasetDescriptorUtils;
 import org.apache.gobblin.service.modules.flowgraph.DatasetDescriptorConfigKeys;
 import org.apache.gobblin.service.modules.template_catalog.FlowCatalogWithTemplates;
-import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 
 
 /**
@@ -66,6 +70,8 @@ public class StaticFlowTemplate implements FlowTemplate {
 
   private transient Config rawConfig;
 
+  private final transient JobSpecResolver jobSpecResolver;
+
   public StaticFlowTemplate(URI flowTemplateDirUri, String version, String description, Config config,
       FlowCatalogWithTemplates catalog)
       throws IOException, SpecNotFoundException, JobTemplate.TemplateException, URISyntaxException {
@@ -75,6 +81,7 @@ public class StaticFlowTemplate implements FlowTemplate {
     this.rawConfig = config;
     this.catalog = catalog;
     this.jobTemplates = this.catalog.getJobTemplatesForFlow(flowTemplateDirUri);
+    this.jobSpecResolver = JobSpecResolver.builder(config).build();
   }
 
   //Constructor for testing purposes
@@ -85,17 +92,23 @@ public class StaticFlowTemplate implements FlowTemplate {
     this.rawConfig = config;
     this.catalog = catalog;
     this.jobTemplates = jobTemplates;
+    try {
+      this.jobSpecResolver = JobSpecResolver.builder(config).build();
+    } catch (IOException ioe) {
+      throw new RuntimeException(ioe);
+    }
   }
 
 
   /**
    * Generate the input/output dataset descriptors for the {@link FlowTemplate}.
-   * @param userConfig
-   * @return a List of Input/Output DatasetDescriptors that resolve this {@link FlowTemplate}.
+   * @param userConfig User supplied Config
+   * @param resolvable Whether to return only resolvable dataset descriptors
+   * @return a List of Input/Output DatasetDescriptors that correspond to this {@link FlowTemplate}. If resolvable is true,
+   * only return descriptors that fully resolve it.
    */
   @Override
-  public List<Pair<DatasetDescriptor, DatasetDescriptor>> getResolvingDatasetDescriptors(Config userConfig)
-      throws IOException, SpecNotFoundException, JobTemplate.TemplateException {
+  public List<Pair<DatasetDescriptor, DatasetDescriptor>> getDatasetDescriptors(Config userConfig, boolean resolvable) throws IOException {
     Config config = this.getResolvedFlowConfig(userConfig).resolve(ConfigResolveOptions.defaults().setAllowUnresolved(true));
 
     if (!config.hasPath(DatasetDescriptorConfigKeys.FLOW_EDGE_INPUT_DATASET_DESCRIPTOR_PREFIX)
@@ -109,13 +122,20 @@ public class StaticFlowTemplate implements FlowTemplate {
     while (config.hasPath(inputPrefix)) {
       try {
         Config inputDescriptorConfig = config.getConfig(inputPrefix);
-        DatasetDescriptor inputDescriptor = getDatasetDescriptor(inputDescriptorConfig);
+        DatasetDescriptor inputDescriptor = DatasetDescriptorUtils.constructDatasetDescriptor(inputDescriptorConfig);
         String outputPrefix = Joiner.on(".")
             .join(DatasetDescriptorConfigKeys.FLOW_EDGE_OUTPUT_DATASET_DESCRIPTOR_PREFIX, Integer.toString(i));
         Config outputDescriptorConfig = config.getConfig(outputPrefix);
-        DatasetDescriptor outputDescriptor = getDatasetDescriptor(outputDescriptorConfig);
+        DatasetDescriptor outputDescriptor = DatasetDescriptorUtils.constructDatasetDescriptor(outputDescriptorConfig);
 
-        if (isResolvable(userConfig, inputDescriptor, outputDescriptor)) {
+        if (resolvable) {
+          try {
+            tryResolving(userConfig, inputDescriptor, outputDescriptor);
+            result.add(ImmutablePair.of(inputDescriptor, outputDescriptor));
+          } catch (JobTemplate.TemplateException | ConfigException | SpecNotFoundException e) {
+            // Dataset descriptor cannot be resolved so don't add it to result
+          }
+        } else {
           result.add(ImmutablePair.of(inputDescriptor, outputDescriptor));
         }
       } catch (ReflectiveOperationException e) {
@@ -126,12 +146,6 @@ public class StaticFlowTemplate implements FlowTemplate {
     return result;
   }
 
-  private DatasetDescriptor getDatasetDescriptor(Config descriptorConfig)
-      throws ReflectiveOperationException {
-    Class datasetDescriptorClass = Class.forName(descriptorConfig.getString(DatasetDescriptorConfigKeys.CLASS_KEY));
-    return (DatasetDescriptor) GobblinConstructorUtils.invokeLongestConstructor(datasetDescriptorClass, descriptorConfig);
-  }
-
   @Override
   public Config getRawTemplateConfig() {
     return this.rawConfig;
@@ -147,27 +161,23 @@ public class StaticFlowTemplate implements FlowTemplate {
   }
 
   /**
-   * Checks if the {@link FlowTemplate} is resolvable using the provided {@link Config} object. A {@link FlowTemplate}
-   * is resolvable only if each of the {@link JobTemplate}s in the flow is resolvable
+   * Try to resolve the {@link FlowTemplate} using the provided {@link Config} object. A {@link FlowTemplate}
+   * is resolvable only if each of the {@link JobTemplate}s in the flow is resolvable. Throws an exception if the flow is
+   * not resolvable.
    * @param userConfig User supplied Config
-   * @return true if the {@link FlowTemplate} is resolvable
    */
   @Override
-  public boolean isResolvable(Config userConfig, DatasetDescriptor inputDescriptor, DatasetDescriptor outputDescriptor)
+  public void tryResolving(Config userConfig, DatasetDescriptor inputDescriptor, DatasetDescriptor outputDescriptor)
       throws SpecNotFoundException, JobTemplate.TemplateException {
     Config inputDescriptorConfig = inputDescriptor.getRawConfig().atPath(DatasetDescriptorConfigKeys.FLOW_EDGE_INPUT_DATASET_DESCRIPTOR_PREFIX);
     Config outputDescriptorConfig = outputDescriptor.getRawConfig().atPath(DatasetDescriptorConfigKeys.FLOW_EDGE_OUTPUT_DATASET_DESCRIPTOR_PREFIX);
     userConfig = userConfig.withFallback(inputDescriptorConfig).withFallback(outputDescriptorConfig);
 
-    ConfigResolveOptions resolveOptions = ConfigResolveOptions.defaults().setAllowUnresolved(true);
+    JobSpec.Builder jobSpecBuilder = JobSpec.builder().withConfig(userConfig);
 
     for (JobTemplate template: this.jobTemplates) {
-      Config templateConfig = template.getResolvedConfig(userConfig).resolve(resolveOptions);
-      if (!template.getResolvedConfig(userConfig).resolve(resolveOptions).isResolved()) {
-        return false;
-      }
+      this.jobSpecResolver.resolveJobSpec(jobSpecBuilder.withTemplate(template).build());
     }
-    return true;
   }
 
   @Override
@@ -179,9 +189,11 @@ public class StaticFlowTemplate implements FlowTemplate {
     userConfig = userConfig.withFallback(inputDescriptorConfig).withFallback(outputDescriptorConfig);
 
     List<Config> resolvedJobConfigs = new ArrayList<>();
+    JobSpec.Builder jobSpecBuilder = JobSpec.builder().withConfig(userConfig);
     for (JobTemplate jobTemplate: getJobTemplates()) {
-      Config resolvedJobConfig = jobTemplate.getResolvedConfig(userConfig).resolve().withValue(
-          ConfigurationKeys.JOB_TEMPLATE_PATH, ConfigValueFactory.fromAnyRef(jobTemplate.getUri().toString()));;
+      ResolvedJobSpec resolvedJobSpec = this.jobSpecResolver.resolveJobSpec(jobSpecBuilder.withTemplate(jobTemplate).build());
+      Config resolvedJobConfig = resolvedJobSpec.getConfig().withValue(
+          ConfigurationKeys.JOB_TEMPLATE_PATH, ConfigValueFactory.fromAnyRef(jobTemplate.getUri().toString()));
       resolvedJobConfigs.add(resolvedJobConfig);
     }
     return resolvedJobConfigs;
