@@ -28,13 +28,17 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import org.apache.avro.AvroRuntimeException;
 import org.apache.avro.Schema;
 import org.apache.avro.Schema.Field;
 import org.apache.avro.Schema.Type;
+import org.apache.avro.SchemaCompatibility;
 import org.apache.avro.file.DataFileReader;
 import org.apache.avro.file.SeekableInput;
+import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericData.Record;
 import org.apache.avro.generic.GenericDatumReader;
 import org.apache.avro.generic.GenericDatumWriter;
@@ -49,6 +53,8 @@ import org.apache.avro.io.EncoderFactory;
 import org.apache.avro.mapred.FsInput;
 import org.apache.avro.util.Utf8;
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.math3.util.Pair;
+import org.apache.gobblin.configuration.ConfigurationKeys;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FileStatus;
@@ -70,13 +76,20 @@ import com.google.common.base.Splitter;
 import com.google.common.base.Strings;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
 import com.google.common.io.Closer;
 
+import javax.annotation.Nonnull;
+import lombok.Builder;
+import lombok.Getter;
+import lombok.ToString;
+import lombok.extern.slf4j.Slf4j;
 
 /**
  * A Utils class for dealing with Avro objects
  */
+@Slf4j
 public class AvroUtils {
 
   private static final Logger LOG = LoggerFactory.getLogger(AvroUtils.class);
@@ -85,6 +98,65 @@ public class AvroUtils {
 
   private static final String AVRO_SUFFIX = ".avro";
 
+  private static final String SCHEMA_CREATION_TIME_KEY = "CreatedOn";
+
+  /**
+   * Validates that the provided reader schema can be used to decode avro data written with the
+   * provided writer schema.
+   * @param readerSchema schema to check.
+   * @param writerSchema schema to check.
+   * @param ignoreNamespace whether name and namespace should be ignored in validation
+   * @return true if validation passes
+   */
+  public static boolean checkReaderWriterCompatibility(Schema readerSchema, Schema writerSchema, boolean ignoreNamespace) {
+    if (ignoreNamespace) {
+      List<Schema.Field> fields = deepCopySchemaFields(readerSchema);
+      readerSchema = Schema.createRecord(writerSchema.getName(), writerSchema.getDoc(), writerSchema.getNamespace(),
+          readerSchema.isError());
+      readerSchema.setFields(fields);
+    }
+
+    return SchemaCompatibility.checkReaderWriterCompatibility(readerSchema, writerSchema).getType().equals(SchemaCompatibility.SchemaCompatibilityType.COMPATIBLE);
+  }
+
+  public static Schema addSchemaCreationTime(Schema inputSchema, Schema outputSchema) {
+    if (inputSchema.getProp(SCHEMA_CREATION_TIME_KEY) != null && outputSchema.getProp(SCHEMA_CREATION_TIME_KEY) == null) {
+      outputSchema.addProp(SCHEMA_CREATION_TIME_KEY, inputSchema.getProp(SCHEMA_CREATION_TIME_KEY));
+    }
+    return outputSchema;
+  }
+
+  public static String getSchemaCreationTime(Schema inputSchema) {
+    return inputSchema.getProp(SCHEMA_CREATION_TIME_KEY);
+  }
+
+  public static Schema setSchemaCreationTime(Schema inputSchema, String creationTime) {
+    inputSchema.addProp(SCHEMA_CREATION_TIME_KEY, creationTime);
+    return inputSchema;
+  }
+
+  public static List<Field> deepCopySchemaFields(Schema readerSchema) {
+    return readerSchema.getFields().stream()
+        .map(field -> {
+          Field f = new Field(field.name(), field.schema(), field.doc(), field.defaultValue(), field.order());
+          field.getProps().forEach((key, value) -> f.addProp(key, value));
+          return f;
+        })
+        .collect(Collectors.toList());
+  }
+
+  /**
+   * Generate a {@link Schema} object from {@link Schema.Field} with Field's properties carried over to the new object.
+   * Common use cases for this method is in traversing {@link Schema} object into nested level and create {@link Schema}
+   * object for non-root level.
+   */
+  public static void convertFieldToSchemaWithProps(Map<String,JsonNode> fieldProps, Schema targetSchemaObj) {
+    for (Map.Entry<String, JsonNode> stringJsonNodeEntry : fieldProps.entrySet()) {
+      targetSchemaObj.addProp(stringJsonNodeEntry.getKey(), stringJsonNodeEntry.getValue());
+    }
+  }
+
+
   public static class AvroPathFilter implements PathFilter {
     @Override
     public boolean accept(Path path) {
@@ -189,8 +261,10 @@ public class AvroUtils {
         return AvroUtils.getFieldHelper(fieldSchema.getValueType(), pathList, ++field);
       case RECORD:
         return AvroUtils.getFieldHelper(fieldSchema, pathList, ++field);
+      case ARRAY:
+        return AvroUtils.getFieldHelper(fieldSchema.getElementType(), pathList, ++field);
       default:
-        throw new AvroRuntimeException("Invalid type in schema : " + schema);
+        throw new AvroRuntimeException("Invalid type " + fieldSchema.getType() + " in schema : " + schema);
     }
   }
 
@@ -246,7 +320,7 @@ public class AvroUtils {
       } else if (data instanceof List) {
         val = getObjectFromArray((List)data, Integer.parseInt(pathList.get(field)));
       } else {
-        val = ((Record)data).get(pathList.get(field));
+        val = ((GenericRecord)data).get(pathList.get(field));
       }
 
       if (val != null) {
@@ -278,7 +352,7 @@ public class AvroUtils {
       return;
     }
 
-    AvroUtils.getFieldHelper(retVal, ((Record) data).get(pathList.get(field)), pathList, ++field);
+    AvroUtils.getFieldHelper(retVal, ((GenericRecord) data).get(pathList.get(field)), pathList, ++field);
     return;
   }
 
@@ -317,7 +391,12 @@ public class AvroUtils {
 
   private static Object getObjectFromMap(Map map, String key) {
     Utf8 utf8Key = new Utf8(key);
-    return map.get(utf8Key);
+    Object value = map.get(utf8Key);
+    if (value == null) {
+      return map.get(key);
+    }
+
+    return value;
   }
 
   /**
@@ -345,7 +424,7 @@ public class AvroUtils {
       return reader.read(null, decoder);
     } catch (IOException e) {
       throw new IOException(
-          String.format("Cannot convert avro record to new schema. Origianl schema = %s, new schema = %s",
+          String.format("Cannot convert avro record to new schema. Original schema = %s, new schema = %s",
               record.getSchema(), newSchema),
           e);
     }
@@ -397,7 +476,7 @@ public class AvroUtils {
   }
 
   public static void writeSchemaToFile(Schema schema, Path filePath, FileSystem fs, boolean overwrite, FsPermission perm)
-    throws IOException {
+      throws IOException {
     writeSchemaToFile(schema, filePath, null, fs, overwrite, perm);
   }
 
@@ -565,10 +644,10 @@ public class AvroUtils {
    * MapReduce job.
    */
   public static Optional<Schema> removeUncomparableFields(Schema schema) {
-    return removeUncomparableFields(schema, Sets.<Schema> newHashSet());
+    return removeUncomparableFields(schema, Maps.newHashMap());
   }
 
-  private static Optional<Schema> removeUncomparableFields(Schema schema, Set<Schema> processed) {
+  private static Optional<Schema> removeUncomparableFields(Schema schema, Map<Schema, Optional<Schema>> processed) {
     switch (schema.getType()) {
       case RECORD:
         return removeUncomparableFieldsFromRecord(schema, processed);
@@ -585,13 +664,13 @@ public class AvroUtils {
     }
   }
 
-  private static Optional<Schema> removeUncomparableFieldsFromRecord(Schema record, Set<Schema> processed) {
+  private static Optional<Schema> removeUncomparableFieldsFromRecord(Schema record, Map<Schema, Optional<Schema>> processed) {
     Preconditions.checkArgument(record.getType() == Schema.Type.RECORD);
 
-    if (processed.contains(record)) {
-      return Optional.absent();
+    Optional<Schema> result = processed.get(record);
+    if (null != result) {
+      return result;
     }
-    processed.add(record);
 
     List<Field> fields = Lists.newArrayList();
     for (Field field : record.getFields()) {
@@ -603,16 +682,19 @@ public class AvroUtils {
 
     Schema newSchema = Schema.createRecord(record.getName(), record.getDoc(), record.getNamespace(), false);
     newSchema.setFields(fields);
-    return Optional.of(newSchema);
+    result = Optional.of(newSchema);
+    processed.put(record, result);
+
+    return result;
   }
 
-  private static Optional<Schema> removeUncomparableFieldsFromUnion(Schema union, Set<Schema> processed) {
+  private static Optional<Schema> removeUncomparableFieldsFromUnion(Schema union, Map<Schema, Optional<Schema>> processed) {
     Preconditions.checkArgument(union.getType() == Schema.Type.UNION);
 
-    if (processed.contains(union)) {
-      return Optional.absent();
+    Optional<Schema> result = processed.get(union);
+    if (null != result) {
+      return result;
     }
-    processed.add(union);
 
     List<Schema> newUnion = Lists.newArrayList();
     for (Schema unionType : union.getTypes()) {
@@ -624,9 +706,13 @@ public class AvroUtils {
 
     // Discard the union field if one or more types are removed from the union.
     if (newUnion.size() != union.getTypes().size()) {
-      return Optional.absent();
+      result = Optional.absent();
+    } else {
+      result = Optional.of(Schema.createUnion(newUnion));
     }
-    return Optional.of(Schema.createUnion(newUnion));
+    processed.put(union, result);
+
+    return result;
   }
 
   /**
@@ -799,6 +885,15 @@ public class AvroUtils {
     return new Path(Joiner.on(Path.SEPARATOR).join(tokens));
   }
 
+  /**
+   * Escaping ";" and "'" character in the schema string when it is being used in DDL.
+   * These characters are not allowed to show as part of column name but could possibly appear in documentation field.
+   * Therefore the escaping behavior won't cause correctness issues.
+   */
+  public static String sanitizeSchemaString(String schemaString) {
+    return schemaString.replaceAll(";",  "\\\\;").replaceAll("'", "\\\\'");
+  }
+
   /**
    * Deserialize a {@link GenericRecord} from a byte array. This method is not intended for high performance.
    */
@@ -807,4 +902,247 @@ public class AvroUtils {
     GenericDatumReader<GenericRecord> reader = new GenericDatumReader<>(schema);
     return reader.read(null, decoder);
   }
+
+  /**
+   * Decorate the {@link Schema} for a record with additional {@link Field}s.
+   * @param inputSchema: must be a {@link Record} schema.
+   * @return the decorated Schema. Fields are appended to the inputSchema.
+   */
+  public static Schema decorateRecordSchema(Schema inputSchema, @Nonnull List<Field> fieldList) {
+    Preconditions.checkState(inputSchema.getType().equals(Type.RECORD));
+    List<Field> outputFields = deepCopySchemaFields(inputSchema);
+    List<Field> newOutputFields = Stream.concat(outputFields.stream(), fieldList.stream()).collect(Collectors.toList());
+
+    Schema outputSchema = Schema.createRecord(inputSchema.getName(), inputSchema.getDoc(),
+        inputSchema.getNamespace(), inputSchema.isError());
+    outputSchema.setFields(newOutputFields);
+    copyProperties(inputSchema, outputSchema);
+    return outputSchema;
+  }
+
+  /**
+   * Decorate a {@link GenericRecord} with additional fields and make it conform to an extended Schema
+   * It is the caller's responsibility to ensure that the outputSchema is the merge of the inputRecord's schema
+   * and the additional fields. The method does not check this for performance reasons, because it is expected to be called in the
+   * critical path of processing a record.
+   * Use {@link AvroUtils#decorateRecordSchema(Schema, List)} to generate such a Schema before calling this method.
+   * @param inputRecord: record with data to be copied into the output record
+   * @param fieldMap: values can be primitive types or GenericRecords if nested
+   * @param outputSchema: the schema that the decoratedRecord will conform to
+   * @return an outputRecord that contains a union of the fields in the inputRecord and the field-values in the fieldMap
+   */
+  public static GenericRecord decorateRecord(GenericRecord inputRecord, @Nonnull Map<String, Object> fieldMap,
+      Schema outputSchema) {
+    GenericRecord outputRecord = new GenericData.Record(outputSchema);
+    inputRecord.getSchema().getFields().forEach(f -> outputRecord.put(f.name(), inputRecord.get(f.name())));
+    fieldMap.forEach((key, value) -> outputRecord.put(key, value));
+    return outputRecord;
+  }
+
+  /**
+   * Given a generic record, Override the name and namespace of the schema and return a new generic record
+   * @param input input record who's name and namespace need to be overridden
+   * @param nameOverride new name for the record schema
+   * @param namespaceOverride Optional map containing namespace overrides
+   * @return an output record with overridden name and possibly namespace
+   */
+  public static GenericRecord overrideNameAndNamespace(GenericRecord input, String nameOverride, Optional<Map<String, String>> namespaceOverride) {
+
+    GenericRecord output = input;
+    Schema newSchema = switchName(input.getSchema(), nameOverride);
+    if(namespaceOverride.isPresent()) {
+      newSchema = switchNamespace(newSchema, namespaceOverride.get());
+    }
+
+    try {
+      output = convertRecordSchema(output, newSchema);
+    } catch (Exception e){
+      log.error("Unable to generate generic data record", e);
+    }
+
+    return output;
+  }
+
+  /**
+   * Given a input schema, Override the name and namespace of the schema and return a new schema
+   * @param input
+   * @param nameOverride
+   * @param namespaceOverride
+   * @return a schema with overridden name and possibly namespace
+   */
+  public static Schema overrideNameAndNamespace(Schema input, String nameOverride, Optional<Map<String, String>> namespaceOverride) {
+
+    Schema newSchema = switchName(input, nameOverride);
+    if(namespaceOverride.isPresent()) {
+      newSchema = switchNamespace(newSchema, namespaceOverride.get());
+    }
+
+    return newSchema;
+  }
+
+  @Builder
+  @ToString
+  public static class SchemaEntry {
+    @Getter
+    final String fieldName;
+    final Schema schema;
+    String fullyQualifiedType() {
+      return schema.getFullName();
+    }
+  }
+
+  /**
+   * Check if a schema has recursive fields inside it
+   * @param schema
+   * @param logger : Optional logger if you want the method to log why it thinks the schema was recursive
+   * @return true / false
+   */
+  public static boolean isSchemaRecursive(Schema schema, Optional<Logger> logger) {
+    List<SchemaEntry> recursiveFields = new ArrayList<>();
+    dropRecursive(new SchemaEntry(null, schema), Collections.EMPTY_LIST, recursiveFields);
+    if (recursiveFields.isEmpty()) {
+      return false;
+    } else {
+      if (logger.isPresent()) {
+        logger.get().info("Found recursive fields [{}] in schema {}", recursiveFields.stream().map(f -> f.fieldName).collect(Collectors.joining(",")),
+            schema.getFullName());
+      }
+      return true;
+    }
+  }
+
+
+  /**
+   * Drop recursive fields from a Schema. Recursive fields are fields that refer to types that are part of the
+   * parent tree.
+   * e.g. consider this Schema for a User
+   * {
+   *   "type": "record",
+   *   "name": "User",
+   *   "fields": [
+   *     {"name": "name", "type": "string",
+   *     {"name": "friend", "type": "User"}
+   *   ]
+   * }
+   * the friend field is a recursive field. After recursion has been eliminated we expect the output Schema to look like
+   * {
+   *    "type": "record",
+   *    "name": "User",
+   *    "fields": [
+   *    {"name": "name", "type": "string"}
+   *    ]
+   * }
+   *
+   * @param schema
+   * @return a Pair of (The transformed schema with recursion eliminated, A list of @link{SchemaEntry} objects which
+   * represent the fields that were removed from the original schema)
+   */
+  public static Pair<Schema, List<SchemaEntry>> dropRecursiveFields(Schema schema) {
+    List<SchemaEntry> recursiveFields = new ArrayList<>();
+    return new Pair(dropRecursive(new SchemaEntry(null, schema), Collections.EMPTY_LIST, recursiveFields),
+        recursiveFields);
+  }
+
+  /**
+   * Inner recursive method called by {@link #dropRecursiveFields(Schema)}
+   * @param schemaEntry
+   * @param parents
+   * @param fieldsWithRecursion
+   * @return the transformed Schema, null if schema is recursive w.r.t parent schema traversed so far
+   */
+  private static Schema dropRecursive(SchemaEntry schemaEntry, List<SchemaEntry> parents, List<SchemaEntry> fieldsWithRecursion) {
+    Schema schema = schemaEntry.schema;
+    // ignore primitive fields
+    switch (schema.getType()) {
+      case UNION:{
+        List<Schema> unionTypes = schema.getTypes();
+        List<Schema> copiedUnionTypes = new ArrayList<Schema>();
+        for (Schema unionSchema: unionTypes) {
+          SchemaEntry unionSchemaEntry = new SchemaEntry(
+              schemaEntry.fieldName, unionSchema);
+          copiedUnionTypes.add(dropRecursive(unionSchemaEntry, parents, fieldsWithRecursion));
+        }
+        if (copiedUnionTypes.stream().anyMatch(x -> x == null)) {
+          // one or more types in the union are referring to a parent type (directly recursive),
+          // entire union must be dropped
+          return null;
+        }
+        else {
+          Schema copySchema = Schema.createUnion(copiedUnionTypes);
+          copyProperties(schema, copySchema);
+          return copySchema;
+        }
+      }
+      case RECORD:{
+        // check if the type of this schema matches any in the parents list
+        if (parents.stream().anyMatch(parent -> parent.fullyQualifiedType().equals(schemaEntry.fullyQualifiedType()))) {
+          fieldsWithRecursion.add(schemaEntry);
+          return null;
+        }
+        List<SchemaEntry> newParents = new ArrayList<>(parents);
+        newParents.add(schemaEntry);
+        List<Schema.Field> copiedSchemaFields = new ArrayList<>();
+        for (Schema.Field field: schema.getFields()) {
+          String fieldName = schemaEntry.fieldName != null ? schemaEntry.fieldName + "." + field.name() : field.name();
+          SchemaEntry fieldSchemaEntry = new SchemaEntry(fieldName, field.schema());
+          Schema copiedFieldSchema = dropRecursive(fieldSchemaEntry, newParents, fieldsWithRecursion);
+          if (copiedFieldSchema == null) {
+          } else {
+            Schema.Field copiedField =
+                new Schema.Field(field.name(), copiedFieldSchema, field.doc(), field.defaultValue(), field.order());
+            copyFieldProperties(field, copiedField);
+            copiedSchemaFields.add(copiedField);
+          }
+        }
+        if (copiedSchemaFields.size() > 0) {
+          Schema copiedRecord = Schema.createRecord(schema.getName(), schema.getDoc(), schema.getNamespace(),
+              schema.isError());
+          copiedRecord.setFields(copiedSchemaFields);
+          copyProperties(schema, copiedRecord);
+          return copiedRecord;
+        } else {
+          return null;
+        }
+      }
+      case ARRAY: {
+        Schema itemSchema = schema.getElementType();
+        SchemaEntry itemSchemaEntry = new SchemaEntry(schemaEntry.fieldName, itemSchema);
+        Schema copiedItemSchema = dropRecursive(itemSchemaEntry, parents, fieldsWithRecursion);
+        if (copiedItemSchema == null) {
+          return null;
+        } else {
+          Schema copiedArraySchema = Schema.createArray(copiedItemSchema);
+          copyProperties(schema, copiedArraySchema);
+          return copiedArraySchema;
+        }
+      }
+      case MAP: {
+        Schema valueSchema = schema.getValueType();
+        SchemaEntry valueSchemaEntry = new SchemaEntry(schemaEntry.fieldName, valueSchema);
+        Schema copiedValueSchema = dropRecursive(valueSchemaEntry, parents, fieldsWithRecursion);
+        if (copiedValueSchema == null) {
+          return null;
+        } else {
+          Schema copiedMapSchema = Schema.createMap(copiedValueSchema);
+          copyProperties(schema, copiedMapSchema);
+          return copiedMapSchema;
+        }
+      }
+      default: {
+        return schema;
+      }
+    }
+  }
+
+  /**
+   * Annoyingly, Avro doesn't provide a field constructor where you can pass in "unknown to Avro" properties
+   * to attach to the field object in the schema even though the Schema language and object model supports it.
+   * This method allows for such copiers to explicitly copy the properties from a source field to a destination field.
+   * @param sourceField
+   * @param copiedField
+   */
+  private static void copyFieldProperties(Schema.Field sourceField, Schema.Field copiedField) {
+    sourceField.getProps().forEach((key, value) -> copiedField.addProp(key, value));
+  }
+
 }
